/***************************************************************************
                          cdrclass.cpp  -  description
                             -------------------
    begin                : Fre Jul 26 2002
    copyright            : (C) 2002 by Cornelius K�bel
    email                : cornelius.koelbel@gmx.de
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cdrclass.h"
#include <qtextedit.h>
#include <qprogressbar.h>
#include <qlabel.h>
#include <sys/types.h>
#include <signal.h>


CDRclass::CDRclass(QWidget *parent, const char *name ) : ProcBaseClass(parent,name) {
  myCDRType=1;
  setCaption("CD Burning");
  myErrorString=i18n("Error starting the burning process.");

  TextLabelProgress->setText(i18n("Progress:"));
  TextLabelTime->setText(i18n("Running..."));
  TextLabelFrames->hide();
  TextLabelFrames2->setText(i18n("0 MBs written."));
}
CDRclass::~CDRclass(){
}
/** Wertet den Fortschritt vom CD-Brennen CDRDAO aus. */
int CDRclass::progress(QString sAusgabe){
  int pos1 = sAusgabe.find("Wrote");
  int pos2 = sAusgabe.find("of", pos1+5);
  QString sAnzahl = sAusgabe.mid(pos1+6, pos2 - pos1 -7);	
  int pos3 = sAusgabe.find("MB", pos2);
  QString sTotal = sAusgabe.mid(pos2+3, pos3 - pos2 -4 );
  double progress = 1000*(double)(sAnzahl.toInt()) / (double)(sTotal.toInt());
  return (int)progress;
}
/** Gets the progress (MB written) from the cdrecord output */
int CDRclass::progressCDRecord(QString sAusgabe){
  int posMB = sAusgabe.find("MB written");
  int posOF = sAusgabe.findRev("of", posMB);
  QString sAnzahl=sAusgabe.mid(posOF-4, 3);
  QString sTotal = sAusgabe.mid(posOF+3, posMB-(posOF+3) );
  double process = 1000*(double)(sAnzahl.toInt()) / (double)(sTotal.toInt() );
  return(int)process;
}
/** �erl�t die Funktion der Basisklasse ProcBaseClass.
    Denn hier wird die Ausgabe von cdrdao ausgewertet. */
void CDRclass::slotReadFromStdout(KProcess * proc, char * buffer, int buflen){
  // identifies cdrdao or cdrecord
  QString sOutput[1][3]= { { (""),("Wrote"),("MB written") }};
  QString sAusgabe;
  sAusgabe = QString(buffer);

  // Die Ausgabe ber die geschriebenen MBs soll nicht ausgegegben werden.
  if (sAusgabe.find( sOutput[0][myCDRType] ) == -1 ) {
    TextEditOutput->append( sAusgabe );
  } else {
    // Hier muss ich die Auswertung reinbringen, wie sie in ExecScriptClass drinnen ist.
    // "Wrote" ist im String vorhanden.
    int progress;
    if ( myCDRType==1) { //cdrdao
      progress = CDRclass::progress(sAusgabe);
    }else{
      progress = CDRclass::progressCDRecord(sAusgabe);
    }
    ProgressBarTranscode->setProgress(progress);
    // MBs ausgeben
    TextLabelFrames2->setText(sAusgabe);
  }
}
/** �erl�t die Funktion der Basisklasse ProcBaseClass.
    Denn hier wird die Ausgabe von cdrdao ausgewertet. */
void CDRclass::slotReadFromStderr(KProcess * proc, char * buffer, int buflen){
  // identifies cdrdao or cdrecord
  QString sOutput[1][3]= { { (""),("Wrote"),("MB written") }};
  QString sAusgabe;
  sAusgabe = QString(buffer);

  // Die Ausgabe ber die geschriebenen MBs soll nicht ausgegegben werden.
  if (sAusgabe.find( sOutput[0][myCDRType] ) == -1 ) {
    TextEditOutput->append( sAusgabe );
  } else {
    // Hier muss ich die Auswertung reinbringen, wie sie in ExecScriptClass drinnen ist.
    // "Wrote" ist im String vorhanden.
    int progress;
    if ( myCDRType==1) { //cdrdao
      progress = CDRclass::progress(sAusgabe);
    }else{
      progress = CDRclass::progressCDRecord(sAusgabe);
    }
    ProgressBarTranscode->setProgress(progress);
    // MBs ausgeben
    TextLabelFrames2->setText(sAusgabe);
  }
}
/** use cdrdao or cdrecord */
void CDRclass::setCDRType(int i){
  myCDRType=i;
}

/** No descriptions */
// This is an overload function.
void CDRclass::slotCancel(){
  // write Ctrl-C to Process. Send SIGINT to process
  // QString sPID; sPID.setNum( myProc->processIdentifier() );
  // Bad Idea! system( "kill  -SIGINT "+sPID);
  // SIGINT is int 2.
  //--if ( kill(myProc->processIdentifier(), SIGINT) == 0 )
  myProc->kill(2);
  PushButtonCancel->hide(); 
  myCanceled=true;
  myAutoClose=false;
}
/** set the GUI data */
void CDRclass::setData(cdrData d){
  myData=d;
  setCDRType( myData.cdrType );
}
/** start the process */
bool CDRclass::start(){
  bool value;
  if (myData.command.isEmpty()) {
    // Now we are in automate mode and we will have to generate a new command.
    // This is the same as in kavi2svcd::slotGenerateCDR.
    value=setKommando(generateKommando());
  } else {
    // We are NOT in automate. Just take the normal Command.
    value=setKommando(myData.command);
  }
  return value;
}
/** generates the kommando from the internal data and returns the string */
QString CDRclass::generateKommando(){
  // This Array defines the parameters for the different burning tools
  QString befehl[8][3]={{(""),("cdrdao "),("cdrecord -v -dao ")},	//1
			{(""),(" --device "),(" dev=")},	//2
			{(""),(" --speed "),(" speed=")},	//3
			{(""),(" --eject "),(" -eject ")},	//4
			{(""),(""),(" cuefile=")}, 	//5
			{(""),(" simulate "),(" -dummy ")},	//6
			{(""),(" write "),("")},	//7
			{(""),(" --overburn "),(" -overburn ")}  //8
  };
										
  //sudo cdrdao           write/simulate    --device 0,3,0 --speed 4  --eject video2.cue
  //sudo cdrecord -dao -v -simulate         dev=/dev/hdf   speed=4    -eject  cuefile=videocd.cue
  QString sKommando="";
  if ( myData.nice != 0 ) {
    sKommando.append(QString("nice -n %1 ").arg(myData.nice));
  }
  sKommando.append( befehl[0][myCDRType] );
  if ( myData.simulate ) {
    sKommando.append( befehl[5][myCDRType]);
  } else {
    sKommando.append( befehl[6][myCDRType]);
  }
  sKommando.append(myData.additionalOption);
  sKommando.append(" ");
  if ( ! myData.cueFile.isEmpty() ) {
    if (	! myData.device.isEmpty()	) {
      // Device
      sKommando.append( befehl[1][myCDRType] );
      sKommando.append( myData.device );
    }
    if ( myCDRType==1 ){
      if ( myData.driver != i18n("--none--") ) {
	sKommando.append(" --driver ");
	sKommando.append( myData.driver );
      }
    }
    // Speed
    sKommando.append( befehl[2][myCDRType] );
    sKommando.append( myData.speed );
    if ( myData.eject ) {
      // eject
      sKommando.append( befehl[3][myCDRType] );
    }
    if ( myData.overburn ) {
      sKommando.append( befehl[7][myCDRType] );
    }
    sKommando.append(" ");
    // cuefile
    sKommando.append( befehl[4][myCDRType] );
    sKommando.append('"');
    sKommando.append( myData.cueFile );
    sKommando.append('"');
  } // End of cue-file exist.
  return sKommando;
}
/** Sets the number of the CDs to be burned in automate mode */
void CDRclass::setNum(int i, int t){
  TextLabelTime->setText(i18n("Burning CD %1 of %2.").arg(i).arg(t));
}
