/***************************************************************************
                          execscriptclass.cpp  -  description
                             -------------------
    begin                : Die Aug 6 2002
    copyright            : (C) 2002 by Cornelius K�bel
    email                : cornelius.koelbel@gmx.de
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "execscriptclass.h"
#include <qtextedit.h>
#include <qpushbutton.h>
#include <qmessagebox.h>
#include <qcheckbox.h>
#include <qtimer.h>
#include <qprogressbar.h>
#include <qlabel.h>
#include <qregexp.h>
#include <qmessagebox.h>
#include "mplex.h"
#include "vcdclass.h"
#include "cdrclass.h"

ExecScriptClass::ExecScriptClass(QWidget *parent, const char *name ) : ScriptDlg(parent,name) {
  PushButtonCancel->hide();

  myPath = "/";	
  myZustand = idle;
  myProc = new QProcess( this );
  connect( myProc, SIGNAL(readyReadStdout()),
	   this, SLOT(slotReadFromStdout()) );
  connect( myProc, SIGNAL(readyReadStderr()),
	   this, SLOT(slotReadFromStdout()) );
  connect( myProc, SIGNAL( processExited()),
	   this, SLOT(slotShowOK()) );	
  QString kdeHome=QString(getenv( "KDEHOME" ));
  if (kdeHome.isEmpty()) {
    myTempScript=QDir::homeDirPath();
    myTempScript.append("/.kde");
  } else {
    myTempScript=kdeHome;
  }
  myTempScript.append("/share/apps/kavi2svcd/tmp-script.sh");
  // Der Timer fr Mplex, VCD und CDR
  myTimer = new QTimer( this );
  connect( myTimer, SIGNAL(timeout()), this, SLOT(slotTimer()) );
  connect( myProc, SIGNAL(processExited()), this, SLOT(slotTimerStop()) );
  myTimer->start( 5000, false ); // 5 seconds timer
  ProgressBarMplex->setTotalSteps(1000);
  ProgressBarVCD->setTotalSteps(1000);
  ProgressBarCDR->setTotalSteps(1000);
}
ExecScriptClass::~ExecScriptClass(){
  delete myProc;
}
/** Reagiert auf den Cancel Button */
void ExecScriptClass::slotCancel(){
  myProc->tryTerminate();
  if ( myProc->isRunning() ) {
    QTimer::singleShot( 10000, myProc, SLOT( kill() ) );
  }
  // Dann wird das Signal processExited geworfen.
}
/** Setzt das Kommando aus dem aufrufenden Dialog (ScriptEdit) */
void ExecScriptClass::setKommando(QString sKommando){
  myProc->clearArguments();
  // Wenn man die bash aufruft, wird auch der kind-process gekillt.
  myProc->addArgument("bash");
  myProc->addArgument("-c");
  myProc->addArgument(myTempScript);

  TextEditOutput->append(myTempScript);
}
/** Liest die Ausgabe des Prozesses und stellt sie grafisch dar. */
void ExecScriptClass::slotReadFromStdout(){
  QString sAusgabe;
  sAusgabe = QString(myProc->readStdout());
  sAusgabe = sAusgabe.append( QString(myProc->readStderr()) );
  if ( sAusgabe.find("[KAVI2SVCD: MPLEXING]") != -1 ) {
    CheckBoxMplex->setChecked(true);
    myZustand = mplex;
  }
  if ( sAusgabe.find("[KAVI2SVCD: VCDIMAGER") != -1 ) {
    CheckBoxVCD->setChecked(true);
    CheckBoxCDR->setChecked(false);
    // Der Zustand ist nun nicht mehr mplex, sondern vcd
    ProgressBarMplex->setProgress(1000);
    myZustand = vcd;
    // Die Ausgabe muss bearbeitet werden...
    sAusgabe = sAusgabe.replace(QRegExp("KAVI2SVCD: VCDIMAGER "),"");
    int pos = sAusgabe.find("]");
    sAusgabe = sAusgabe.left( pos+1 );
    // Die Ausgabe wird angezeigt...
    TextLabelVCD->setText(i18n("vcdimager: generating VCD image %1").arg(sAusgabe));
  }
  // evtl. folgt noch das CD-Brennen.
  QString hAusgabe = "";
  if ( sAusgabe.find("[KAVI2SVCD: BURNING") != -1 ) {
    CheckBoxCDR->setChecked(true);
    myZustand = cdr;
    // Die Ausgabe muss bearbeitet werden...
    hAusgabe = sAusgabe.replace(QRegExp("KAVI2SVCD: BURNING "),"");
    int pos = hAusgabe.find("]");
    hAusgabe = hAusgabe.left( pos+1 );
    // Die Ausgabe wird angezeigt...
    TextLabelCDR->setText(i18n("cdrdao: burning VCD image %1").arg(hAusgabe));
  }
  if ( myZustand != cdr ) {
    TextEditOutput->append( sAusgabe );
  }
  if ( myZustand == cdr ) {
    // Die ausgabe kann direkt ausgewertet werden.
    // Hierfr wird kein Timer mehr gebraucht.
    if ( sAusgabe.find("[KAVI2SVCD]") != -1 ) {
      // hier muss noch ein Dialog davor...
      QMessageBox* key = new QMessageBox( i18n("Burning..."), i18n("Insert CD %1").arg(hAusgabe),
					  QMessageBox::Information,
					  QMessageBox::Ok,
					  QMessageBox::NoButton,
					  QMessageBox::NoButton);
      key->exec();
      delete key;
      myProc->writeToStdin("\n");
    }
    // Fortschritt von cdrdao
    // Wrote 23 of 214 MB
    // Die Ausgabe ber die geschriebenen MBs soll nicht ausgegegben werden.
    if (sAusgabe.find("Wrote") == -1 ) {
      TextEditOutput->append( sAusgabe );
    } else {
      // Hier muss ich die Auswertung reinbringen, wie sie in ExecScriptClass drinnen ist.
      // "Wrote" ist im String vorhanden.
      int progress = CDRclass::progress(sAusgabe);
      ProgressBarCDR->setProgress(progress);
      // MBs ausgeben
      TextLabelMBWritten->setText(sAusgabe);
    }
  } // Ende von Zustand = CDR
}
/** Startet den Process, das Script. */
void ExecScriptClass::slotStart(){
  QString myErrorString = i18n("Error while executing");
  PushButtonStart->hide();
  PushButtonOK->hide();
  PushButtonCancel->show();
  if (!myProc->arguments().empty())
    if ( !myProc->start() ) {
      // error handling
      QMessageBox* myError = new QMessageBox();
      myError->setText(myErrorString);
      myError->exec();
      delete myError;
    }

}
/** Zeigt den OK-Button, versteckt den Cancel Button */
void ExecScriptClass::slotShowOK(){
  PushButtonOK->show();
  PushButtonCancel->hide();
}
/** Der Timer fr Mplex, CDR und VCD. */
void ExecScriptClass::slotTimer(){
  if ( myZustand == mplex ) {
    checkenMplex();
  } else if ( myZustand == vcd ) {
    checkenVCD();
  } else if ( myZustand == cdr ) {
    // Die Ausgabe von CDRDAO kann direkt in
    // slotReadFromStdout ausgewertet werden.
  }
}

/** Anhalten des Timers fr Mplex, VCD und CDR */
void ExecScriptClass::slotTimerStop(){
  myTimer->stop();
  ProgressBarMplex->setProgress(1000);
  ProgressBarVCD->setProgress(1000);
}
/** Setzt die fr Mplex-Progress notwendigen Daten */
void ExecScriptClass::setMplexData(QString v, QString a1, QString a2){
  myVideo = v;
  myAudio1 = a1;
  myAudio2 = a2;
}
/** Prft, wie gro�die Dateien vom mplexing sind,
    um dann den ProgressBar entsprechend zu fllen. */
void ExecScriptClass::checkenMplex(){
  // Hier mssen die Dateien angeschaut werden.
  int posDatei =  myVideo.findRev("/");
  myPath = myVideo.left( posDatei );
  int progress = Mplex::progress(myVideo, myAudio1, myAudio2, myPath+"/KAVI-mplex-output-*.mpeg");
  ProgressBarMplex->setProgress(progress);
}
/** Prft wie gro�die Dateien vom vcdimager sind, um dann dementsprechend
    den Progressbar zu fllen. */
void ExecScriptClass::checkenVCD(){
  // Faktor 1.02 KAVI-mplex-output-?.mpeg ist immer fest
  // Prefix-1.bin
  // Summe berechnen der mpeg - dateien.
  // INPUT: QStringList mpegListe,
  //				QString  Filter,
  //	  		QString myPrefix,
  // 				QString bin-file,
  //				QString myPath.			
  // OUTPUT: progress / 1000
  int progress  = VCDclass::progress("", "KAVI-mplex-output-*.mpeg",
				     myPrefix, "", myPath);
  ProgressBarVCD->setProgress(progress);
}

/** Setzt die Daten, die notwendig sind, um den Fortschritt von
    VCDimager festzustellen. */
void ExecScriptClass::setVCDData(bool cds, QString pref){
  myPrefix = pref;
  myCDsBrennen = cds;
  if ( !myCDsBrennen ) {
    CheckBoxCDR->hide();
    TextLabelCDR->hide();
    ProgressBarCDR->hide();
  }
}
