/***************************************************************************
                          getinfoclass.cpp  -  description
                             -------------------
    begin                : Mit Sep 10 2003
    copyright            : (C) 2003 by Cornelius K�bel
    email                : cornelius.koelbel@gmx.de
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "getinfoclass.h"
#include <qmessagebox.h>
#include <qregexp.h>

GetInfoClass::GetInfoClass(){
  // Generating the Process for GetInfo
  myGetInfoProc = new QProcess( this );
  connect( myGetInfoProc, SIGNAL(readyReadStdout()),
	   this, SLOT(slotGetInfoReadFromStdOut()) );
  // tcprobe writes the file type to stderr
  connect( myGetInfoProc, SIGNAL(readyReadStderr()),
	   this, SLOT(slotGetInfoReadFromStdErr()) );
  connect( myGetInfoProc, SIGNAL(processExited()),
	   this, SLOT( slotGetInfoExited() ) );
  myGetInfoAusgabe="";
  myGetInfoErr="";
}
GetInfoClass::~GetInfoClass(){
}
/** Gets the INformation of file */
void GetInfoClass::slotGetInfo(QString file){
  // Here the process gets started
  myFile=file;
  if ( !file.isEmpty() ) {
    myGetInfoProc->clearArguments();
    myGetInfoAusgabe="";
    myGetInfoErr="";

    if (file.find(".avi") ==
	(int)(file.length()-4) ) {
      //myGetInfoProc->addArgument( "avifix" );
      //myGetInfo = AVIFIX;
	// We will also take tcprobe, since it seems to give the
	// better results ck, 17.04.05
	myGetInfoProc->addArgument( "tcprobe");
	myGetInfo = TCPROBE;
    } else
      { // mpegs und andere mit transcode...
	// myGetInfoProc->addArgument( "transcode" );
	// myGetInfoProc->addArgument( "-c 1-2");
	// myGetInfo = TRANSCODE;
	myGetInfoProc->addArgument( "tcprobe");
	myGetInfo = TCPROBE;
      }
    myGetInfoProc->addArgument( "-i" );
    myGetInfoProc->addArgument(file);
    if ( !myGetInfoProc->start() ) {
      QString name;
      if (myGetInfo==AVIFIX) {	name="avifix"; }
      if (myGetInfo==TRANSCODE) { name="transcode"; }
      if (myGetInfo==TCPROBE) {name="tcprobe";}
      QMessageBox* error=new QMessageBox();
      QString errorStr=i18n("\nCould not start the %1-process,\n").arg(name);
      errorStr.append( "that is used for retrieving the file info." );
      error->setText( errorStr );
      error->exec();
      delete error;
    }
  }
}
/** Collect output */
void GetInfoClass::slotGetInfoReadFromStdErr(){
  myGetInfoErr += QString(myGetInfoProc->readStderr());
}
/** Collect ourput */
void GetInfoClass::slotGetInfoReadFromStdOut(){
  myGetInfoAusgabe += QString(myGetInfoProc->readStdout());
}
/** Is called, when the process is ready */
void GetInfoClass::slotGetInfoExited(){
  QString codec, bitrate, frames, width, height, fps, duration;
  codec="";  bitrate=""; frames=""; width=""; height=""; fps=""; duration="";
  long myFrames;
  if ( myGetInfo == AVIFIX ) {
    // TODO: This has to be changed if the output of avifix changes
    // search for frames per second
    int fps1 = myGetInfoAusgabe.find(QRegExp("V: (.*) fps"));
    int fps2 = myGetInfoAusgabe.find(QRegExp("fps"),fps1);
    fps=myGetInfoAusgabe.mid(fps1+3,fps2-fps1-4);
    // search for codec: [...] codec=DIV3, [...]
    int codec1=myGetInfoAusgabe.find(QRegExp("codec="),0);
    int codec2=myGetInfoAusgabe.find(QRegExp(","),codec1);
    codec=myGetInfoAusgabe.mid(codec1+6,codec2-codec1-6);
    // search for bitrate: [...] bitrate=96kHz, [...]
    int bitrate1=myGetInfoAusgabe.find(QRegExp("bitrate="),0);
    int bitrate2=myGetInfoAusgabe.find(QRegExp(","),bitrate1);
    bitrate=myGetInfoAusgabe.mid(bitrate1+8,bitrate2-bitrate1-8);
    // search for resolution
    int width1=myGetInfoAusgabe.find(QRegExp("width="),0);
    int width2=myGetInfoAusgabe.find(QRegExp(","),width1);
    width=myGetInfoAusgabe.mid(width1+6,width2-width1-6);
    int height1=myGetInfoAusgabe.find(QRegExp("height="),0);
    int height2=myGetInfoAusgabe.find(QRegExp("\n"),height1);
    height=myGetInfoAusgabe.mid(height1+7,height2-height1-7);
    // search for number of frames Frames
    int frames1=myGetInfoAusgabe.find(QRegExp("frames="),0);
    int frames2=myGetInfoAusgabe.find(QRegExp(","),frames1);
    frames=myGetInfoAusgabe.mid(frames1+7,frames2-frames1-7);
    myFrames=frames.toLong();
  } // Ende von AVI
  else if ( myGetInfo == TCPROBE ) {
    // search for resolution
    // import frame size: -g 480x576 [720x576] (*)
    int width1=myGetInfoAusgabe.find("frame size: -g ",0)+15;
    int width2=myGetInfoAusgabe.find(" ",width1);
    QString hWidth=myGetInfoAusgabe.mid(width1,width2-width1);
    width=hWidth.left( hWidth.find("x") );
    height=hWidth.right( hWidth.findRev("x") );
    // If we use tcprobe for avi-files
    if ( myFile.find(".avi") ) {
	// [avilib] V: 23.976 fps, codec=DIV3, frames=175043, width=640, height=272
	int codec1=myGetInfoAusgabe.find("codec=")+6;
 	int codec2=myGetInfoAusgabe.find(",",codec1);
	codec=myGetInfoAusgabe.mid(codec1,codec2-codec1);
    }else{
	    // first line:
	    // [tcprobe] MPEG program stream (PS)
	    int codec1=myGetInfoErr.find("[tcprobe] ",0)+10;
	    int codec2=myGetInfoErr.find("\n",codec1);
	    codec=myGetInfoErr.mid(codec1,codec2-codec1);
    }
    // anzahl frames
    // -frame rate
    // - bitrate audio + video
    int frc1=myGetInfoAusgabe.find("frame rate: -f ",0)+15;
    int frc2=myGetInfoAusgabe.find(" ",frc1);
    fps=myGetInfoAusgabe.mid(frc1,frc2-frc1);
    double iFr=fps.toDouble();
    // bitrate video
    int brv1=myGetInfoAusgabe.find("bitrate=")+8;
    int brv2=myGetInfoAusgabe.find(" kbps",brv1);
    int bra1=myGetInfoAusgabe.find("bitrate=",brv2)+8;
    int bra2=myGetInfoAusgabe.find(" kbps",bra1);
    QString bitrateV=myGetInfoAusgabe.mid(brv1,brv2-brv1);
    QString bitrateA=myGetInfoAusgabe.mid(bra1,bra2-bra1);
    int iBrV= bitrateV.toInt();
    int iBrA= bitrateA.toInt();
    // Duration of the video
    // duration=2:01:40.759
    int dv1=myGetInfoAusgabe.find("duration=")+9;
    int dv2=myGetInfoAusgabe.find(".",dv1);
    duration=myGetInfoAusgabe.mid(dv1,dv2-dv1);

    int posFrames1=myGetInfoAusgabe.find("length: ")+8;
    if (posFrames1>0) {
    	int posFrames2=myGetInfoAusgabe.find(" frames", posFrames1);
    	QString sFrames=myGetInfoAusgabe.mid(posFrames1,posFrames2-posFrames1);
    	myFrames=sFrames.toInt();
	}else{
	    // get filesize:
	    if ( iFr != 0 ) {
	      long fileSize=0;
	      QFileInfo inFile = QFileInfo(myFile);
	      if (inFile.isFile()) { fileSize = inFile.size(); }
	      double byterate=((iBrV+iBrA)/8)*1024;
	      myFrames=(long) ((fileSize/byterate) * iFr);
	    } else {
	      myFrames=0;
	    }
    }
    frames = frames.setNum(myFrames);
  } // Ende von TCPROBE
  else if ( myGetInfo == TRANSCODE )
    { // i.e. mpegs, output of transcode.
      // search for codec :import format    | MPEG   RIFF data,
      int codec1=myGetInfoAusgabe.find(QRegExp("import format"),0);
      int codec2=myGetInfoAusgabe.find(QRegExp("\n"), codec1+10);
      codec=myGetInfoAusgabe.mid(codec1+19,codec2-codec1-19);
      // search for: "import frame     | 352x288  1.22:1"
      int frame1=myGetInfoAusgabe.find(QRegExp("import frame"),codec1);
      int frame2=myGetInfoAusgabe.find(QRegExp("\\s"),frame1+19);
      QString frame=myGetInfoAusgabe.mid(frame1+19,frame2-frame1-19);
      width=  frame.left( frame.find("x") );
      height= frame.right( frame.length()-frame.find("x")-1 );
      bitrate="--";
      frames="--";
      // search for "encoding fps     |"
      int fps1=myGetInfoAusgabe.find(QRegExp("encoding fps"),0);
      int fps2=myGetInfoAusgabe.find(QRegExp("\\s"),fps1+19);
      fps=myGetInfoAusgabe.mid(fps1+19,fps2-fps1-19);
      // search for bytes per frame.
      // [transcode] A: bytes per frame  | 5884 (5885.880000)
      int bpF1 = myGetInfoAusgabe.find(QRegExp("bytes per frame"),0);
      int bpF2 = myGetInfoAusgabe.find(QRegExp("\\s"),bpF1+19);
      QString bpF = myGetInfoAusgabe.mid(bpF1+19,bpF2-bpF1-19);
      int ibpF = bpF.toLong();
      // OK, Now I know how many bytes has a frame.
      // How many byte does the file have => apporx. number of  frames
      QFileInfo inFile = QFileInfo(myFile);
      if (inFile.isFile()) {
	myFrames = inFile.size() / ibpF;
	frames = frames.setNum(myFrames);
      } else {
	frames="0";
      }
    } // End of  mpeg TRANSCODE
  // Output:  	// Emit signals for
  // frames, codec, fps, hight, width
  emit sendFrames( frames );
  emit sendFPS( fps );
  emit sendCodec( codec );
  emit sendHeight( height );
  emit sendWidth( width );
  emit sendDuration( duration );
  emit sendFileInfo( myGetInfoErr+"\n"+myGetInfoAusgabe );
  // Umrechnung
  bool ok1, ok2;
  int HeightY=height.toInt(&ok1, 10);
  int WidthX=width.toInt(&ok2, 10);
  // TODO hier noch eine Fehlerbehandlung, falls nicht ok einfgen.
  // Popupfenster.
  // Das Border-TextEdit-Feld wird befllt.
  if (ok1 && ok2) {
    int border=(int)((-1)*(0.75*WidthX-HeightY)/2);
    QString sBorder; sBorder=sBorder.setNum(border);
    emit sendBorder(sBorder);
  } /* else {
       QMessageBox* error = new QMessageBox();
       error->setText(i18n("Can not calculate the border."));
       error->exec();
       delete error;
       }*/
}
