/***************************************************************************
                          kavi2svcd.cpp  -  description
                             -------------------
    begin                : Die Jul 23 11:52:25 CEST 2002
    copyright            : (C) 2002 by Cornelius K�bel
    email                : cornelius.koelbel@gmx.de
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kavi2svcd.h"
#include "transcode.h"
#include "mplex.h"
#include "vcdclass.h"
#include "cdrclass.h"
#include "cdrwclass.h"
#include "prefclass.h"
#include "generatescriptclass.h"
#include "myqlineedit.h"
#include "initclass.h"
#include "profile.h"
#include "contextmenu.h"
#include "rangeofframesclass.h"
#include "interlaceclass.h"
#include "directorymodeclass.h"
#include "filetypeclass.h"
#include "subtitlerclass.h"
#include "aspectratioclass.h"
#include "askdlgclass.h"
#include "batchclass.h"
//----------------------
#include <qcombobox.h>
#include <qradiobutton.h>
#include <qlineedit.h>
#include <qfiledialog.h>
#include <qprocess.h>
#include <qtextedit.h>
#include <qregexp.h>
#include <qlabel.h>
#include <qstring.h>
#include <qcheckbox.h>
#include <qspinbox.h>
#include <qlistbox.h>
#include <qcheckbox.h>
#include <qtabwidget.h>
#include <qmessagebox.h>
#include <qpopupmenu.h>
#include <qdir.h>
#include <qtextstream.h>
#include <qgroupbox.h>
#include <qpushbutton.h>
#include <qslider.h>
#include <stdlib.h>
#include <qstatusbar.h>
#include <qcursor.h>


Kavi2svcd::Kavi2svcd(int pdebug, QWidget *parent, const char *name) : kavi2svcdDlg(parent, name)
{
  debug = (bool) pdebug;
  myVersion="0.8.5";
  myConfigProfile="kavi2svcd";
  // This comes first. Here also the list will be loaded, what programms
  // will be checked at startup
  initPreferences();
  if ( debug) printf("After checkIt commands. Got output from whereis transcode.\n");
  initLibList_neu();
  if (debug) printf("After initLibList();\n");
  
  initOpenDirs();
  
  // Preview Proc for cue files
  myPreviewProc = new QProcess();
  myAutomate = new AutomateClass( this );
  myScript = new GenerateScriptClass( this );	
  connect( myScript, SIGNAL(scriptLoaded(QString, QString, QString,
					 QString, QString, QString,
					 QString, QString, QString, QString)),
	   this, SLOT(slotScriptLoaded(QString, QString, QString,
				       QString, QString, QString,
				       QString, QString, QString, QString )) );
  // project data
  myConfigProfile="kavi2svcd";
  myProjectFile="";
  myProjectProgress.transcodeOK=false;
  myProjectProgress.mplexOK=false;
  myProjectProgress.vcdimagerOK=false;
  myProjectProgress.cdrOK=false;
  myProjectProgress.vcdimagerNum=0;
  myProjectProgress.cdrNum=0;
  myProjectProgress.ready=true;
  myAutomationCanceled=false;
  ASR.keepASR=false;
  ASR.exportRatio=0;
  ASR.importRatio=0;
  myFrames=1;
  myFramesAll=true;
  myFrameFirst=1;
  myFrameLast=myFrames;
  myFast = "";
  mySubtitleCommand="";
  myFps=1;
  myInterlacing=0;
  myAutomateData.runIt=false;
  myDirMode=false;
  // output directories
  myOutputTranscode="";
  myOutputMplex="";
  myOutputVCD="";

  FPS.jFilter=FPSClass::OFF;
  
  mySubData.iFontFactor=1075;
  mySubData.bUseSub=false;
  mySubData.bTranscodeMode=false;
  // Generating the Process for GetInfo
  myGetInfo = new GetInfoClass( );
  connect( myGetInfo, SIGNAL(sendCodec(QString)),
	   this, SLOT(slotGetInfoCodec(QString)) );
  connect( myGetInfo, SIGNAL(sendFPS(QString)),
	   this, SLOT(slotGetInfoFPS(QString)) );
  connect( myGetInfo, SIGNAL(sendFrames(QString)),
	   this, SLOT( slotGetInfoFrames(QString) ) );
  connect( myGetInfo, SIGNAL(sendWidth(QString)),
	   this, SLOT( slotGetInfoWidth(QString) ) );
  connect( myGetInfo, SIGNAL(sendDuration(QString)),
	   this, SLOT( slotGetInfoDuration(QString) ) );
  connect( myGetInfo, SIGNAL(sendHeight(QString)),
	   this, SLOT( slotGetInfoHeight(QString) ) );
  connect( myGetInfo, SIGNAL(sendBorder(QString)),
	   this, SLOT( slotGetInfoBorder(QString) ) );
  connect( myGetInfo, SIGNAL( sendFileInfo(QString)),
	   this, SLOT( slotGetInfoFileInfo(QString)) );
  // Initializiation of the Radiobuttons VCD/SVCD/User
  slotVCD();
  
  // Filling the kavi2svcd-Menu
  kaviMenu->insertItem( i18n("&Automate CD Generation"), this, SLOT(slotAutomate()), CTRL+Key_A, 11);
  kaviMenu->insertItem( i18n("&Generate/show Script"), this, SLOT(slotGenerateScript()), CTRL+Key_G , 10);
  kaviMenu->setItemEnabled( 10,false);
  
  updateCaption();
  pushButtonBatch->hide();
  pushButtonBatchMplex->hide();
  //
  // B a s i c  M o d e   /   E x p e r t   M o d e
  //
  // Initialize the dialog before doing any other thing :)
  myModeDialog = new SvcdModeClass(this);
  connect( myModeDialog, SIGNAL( signalExpertMode()),
	   this, SLOT( slotSwitchExpert()) );
  connect( myModeDialog, SIGNAL( signalStart()),
	   this, SLOT( slotStartSVCDMode()) );
  connect( myModeDialog, SIGNAL( signalOpen()),
	   this, SLOT( slotOpenAviFile()) );
  connect( myModeDialog, SIGNAL( signalKtoggled() ),
	   this, SLOT( slotKtoggled()) );
  connect( myModeDialog, SIGNAL( signalzToggled() ),
	   this, SLOT( slotzToggled()) );
  connect( myModeDialog, SIGNAL( signalVCDToggled(bool) ),
	   this, SLOT( slotVCDToggled(bool) ) );
  connect( myModeDialog, SIGNAL( signalDVDToggled(bool) ),
	   this, SLOT( slotDVDToggled(bool) ) );
  connect( myModeDialog, SIGNAL( signalSVCDToggled(bool) ),
	   this, SLOT( slotSVCDToggled(bool) )
	   );

  connect (myModeDialog, SIGNAL(signalRight()),
	   this, SLOT(slotUpdateBasicModeImage()) );
  connect (myModeDialog, SIGNAL(signalRight()),
	   this, SLOT(slotUpdateBasicModeRef()) );			
  // took it from the prefs
  if ( myModeType==ASK ) {
    myTab->hide();
    AskDlgClass* aDlg=new AskDlgClass(this);
    if ( aDlg->exec() ) {
    	myModeType=aDlg->mode();
    }else{
    	// fixme: Here we leave kavi2svcd
	this->~ Kavi2svcd();
    }
    delete aDlg;
  }
  switch ( myModeType ) {
  case EXPERT: slotSwitchExpert(); break;
  case SVCD: RadioButtonSVCD->setChecked(true);
		slotSwitchSVCDMode(); break;
  case VCD: RadioButtonVCD->setChecked(true);
		slotSwitchSVCDMode(); break;
  case DVD: RadioButtonDVD->setChecked(true);
		slotSwitchSVCDMode(); break;
  case ASK: break;
  }
}

Kavi2svcd::~Kavi2svcd()
{
  delete myAutomate;
  delete myScript;
  delete myGetInfo;
  delete myModeDialog;
}

//--------------------------------------------------------------------------------
//
//  INIT
//
//---------------------------------------------------------------------------------
/** Test if the Conf-Dir exists.
    Otherwise make the directory.
    Input of the Configuration
    Saving of the Configuration
    Loading the Configuration */
void Kavi2svcd::initPreferences(){
  // Initialization
  myKeepAspectRatio=false;
  /* ONLY AT THE FIRST START.
     There for we test, if the Directory exists */
  QString Verzeichnis;
  QString kdeHome=QString(getenv( "KDEHOME" ));
  if (kdeHome.isEmpty()) {
    Verzeichnis=QDir::homeDirPath();
    Verzeichnis.append("/.kde");
  } else {
    Verzeichnis=kdeHome;
  }
  Verzeichnis.append("/share/apps/kavi2svcd");


  QDir confDir( Verzeichnis );
  if ( !confDir.exists() ) {
    // Hinweis zu den Programmen, die genutzt werden...
    QMessageBox* myHinweis = new QMessageBox();
    QString HinweisString=i18n("\nFor Kavi2svcd the directory\n %1 \nwill be created.\n To have Kavi2svcd run correctly with all functions you need\n the following programms to be installed:\n- transcode by Thomas Oestreich\n- mjpegools (see mjpeg.sourceforge.net)\n- cdrdao\nFor burning CDs you need to have a sudo for cdrdao.\nHave Fun.").arg(Verzeichnis);
    myHinweis->setText(HinweisString);
    if ( myHinweis->exec() == QDialog::Accepted )
      {
	// Erzeugen des Verzeichnisses ~/.kde/share/apps/kavi2svcd
	// darin wird die Datei kavi2svcd.config gespeichert.
	if ( !confDir.mkdir(Verzeichnis, true) ) {
	  QMessageBox* error=new QMessageBox();
	  QString errorStr=i18n("\nCould not generate the Directory\n");
	  errorStr.append( Verzeichnis );
	  error->setText( errorStr );
	  error->setIcon( QMessageBox::Warning );
	  error->exec();
	  delete error;
     	}
	// Rufe Preferences auf,
     	// damit die Daten gespeichert werden.
	// Hinten mit "false", damit die Daten nicht geladen werden, weil sie
	// noch nicht existieren.
	PrefClass* myPrefs = new PrefClass(this, "test", false);
	myPrefs->exec();
	delete myPrefs;
      }
    delete myHinweis;
  } // Ende von (confDir existiert nicht).
  // Konfiuration einlesen.
  // Das bernimmt die PrefClass Klasse.
  PrefClass* myPrefs = new PrefClass(this, "test", true);
  myPrefs->setPrefs();
  /*
    Check if all programs exist!
  */
  InitClass* checkIt= new InitClass( );
  QString sPrograms[9]={ ("tcmodinfo"), ("mplex"),
			 ("vcdimager"),("cdrdao"),
			 ("cdrecord"),("pkill"),
			 ("kill"),("avifix"),("transcode") };
  for ( int i=0; i<=8; i++) {
    if (myCheckPrograms.find(sPrograms[i])==-1) {
      bool check=checkIt->checkExec(sPrograms[i]);
      // do not check it next time
      if (!check) myCheckPrograms.append("|"+sPrograms[i]);
    }
  }
  // The myCheckPrograms are stored 
  myPrefs->slotSetCheckPrograms(myCheckPrograms);
  delete myPrefs;
  delete checkIt;
}
//-------------------------------------------------------------------------------------
//
// Hilfe 
//
//
/** Dieser Slot wird angesprungen von dem Help-About button.
    In dieser Memberfunktion wird dann der neue Dialog erzeugt. */
void Kavi2svcd::helpAbout(){
  //	 About *myAbout = new About(this);
  KAboutApplication* myAbout = new KAboutApplication( myAboutData );
  myAbout->setCaption(i18n("About"));
  //myAbout->setImage(KApplication::kde_icondir()+"kavi2svcd.png");
  myAbout->exec();
  delete myAbout;
}
//----------------------------------------------------------------------------
//
//  VCD
//
//
/** Dieser Slot wird aufgerufen, wenn ein Radiobutton ge�dert wird
    (VCD oder SVCD). Danach werden die Combobox und die beiden Textfelder befllt. */
void Kavi2svcd::slotVCD(){
  myTab->setTabEnabled(myTab->page(2), true);
  myTab->setTabEnabled(myTab->page(3), true);
  if (RadioButtonDVD->isChecked()) {
    // disable the vcd stuff
    myTab->setTabEnabled(myTab->page(2), false);
    myTab->setTabEnabled(myTab->page(3), false);
    RadioButtonDVD->setFocus();
    ComboBoxCodec->setCurrentItem(8);	
    ComboBoxMplexCodec->setCurrentItem(8);
    LineEditResolution->setText(myResolutionSVCD);
    LineEditBitrate->setText(myAudioBitrateSVCD);
    LineEditVideoBitrate->setText(QString("%1").arg(myDVDBitrate));
    LineEditMplexSize->setText("2000");
  } else if (RadioButtonSVCD->isChecked()) {
    RadioButtonSVCD->setFocus();
    ComboBoxCodec->setCurrentItem(4);	
    ComboBoxMplexCodec->setCurrentItem(4);
    LineEditResolution->setText(myResolutionSVCD);
    LineEditBitrate->setText(myAudioBitrateSVCD);
    LineEditMplexSize->setText(mySegmentSize);
  } else if (RadioButtonVCD->isChecked()) {
    RadioButtonVCD->setFocus();
    // Die VCD-Aufl�ung ist nicht fast-resizable.
    ComboBoxCodec->setCurrentItem(1);
    ComboBoxMplexCodec->setCurrentItem(1);
    LineEditResolution->setText(myResolutionVCD);
    LineEditBitrate->setText(myAudioBitrateVCD);
    LineEditMplexSize->setText(mySegmentSize);
    LineEditVideoBitrate->setText("1152"); // fix
    slotCalcVideoSize();
  } 

}
/** No descriptions */
void Kavi2svcd::slotFreiEingabe(){
  // Wenn SVCD oder VCD den Focus haben, soll hier nichts gemacht werden
  if ( LineEditResolution->hasFocus() ||
       ComboBoxCodec->hasFocus() ||
       LineEditVideoBitrate->hasFocus() ||
       LineEditBitrate->hasFocus() ){
    RadioButtonFrei->setChecked(true);
    RadioButtonVCD->setChecked(false);
    RadioButtonSVCD->setChecked(false);
    // Falls es vorher eine VCD (1) war, soll nun Typ 2 genommen werden
    if (ComboBoxCodec->currentItem()==1) {
      ComboBoxCodec->setCurrentItem(2);
    }
    // Falls es vorher eine SVCD (4) war, soll nun Typ 5 genommen werden
    if (ComboBoxCodec->currentItem()==4) {
      ComboBoxCodec->setCurrentItem(5);
    }
    ComboBoxMplexCodec->setCurrentItem(ComboBoxCodec->currentItem());
  }
}
/** Hier wird die QFileDialog aufgerufen,
    in dem das input-Avi-File ausgew�lt wird. */
void Kavi2svcd::slotOpenInputFile(){
  QString fileName;
  QFileDialog *myInputFile = new QFileDialog(myDirTranscodeIn);
  // only if we are in Expert mode, we can select
  // multiple files and use batch mode
  if ( myModeType == EXPERT ){
    myInputFile->setMode(QFileDialog::ExistingFiles);
  } else {
    myInputFile->setMode(QFileDialog::ExistingFile);
  }
  myInputFile->setFilters(i18n("video files (*.avi *.mpeg *.mpg);;*.avi;;MPEG (*.mpg *.mpeg);;Video data (*.m2v *.m1v);;Audio data (*.mpa);;QuickTime (*.mov);;All (*)"));
  if ( myInputFile->exec() == QDialog::Accepted )
    {
      if ( myModeType == EXPERT ) {
	myInputFileList = myInputFile->selectedFiles();
	if ( myInputFileList.count()>1 ) {
	  pushButtonBatch->show();
	  LineEditOutputFile->setEnabled(false);
	} else {
	  pushButtonBatch->hide();
	  LineEditOutputFile->setEnabled(true);				
	}
	QStringList::Iterator it = myInputFileList.begin();
	while( it != myInputFileList.end() ) {
	  fileName=*it;
	  ++it;
	}
      } else {
	fileName = myInputFile->selectedFile();
      }
      QFileInfo qf = QFileInfo(fileName);
      if (!qf.isReadable()) {
	  QMessageBox* error=new QMessageBox();
	  QString errorStr=i18n("\nCan not open file \n");
	  errorStr.append( fileName );
	  errorStr.append(i18n("\nFile is not readable!\n"));
	  error->setText( errorStr );
	  error->setIcon( QMessageBox::Warning );
	  error->exec();
	  delete error;
      }else{
	      LineEditInputFile->setText(fileName);
	      // Put the filename into myModeDialog
	      myModeDialog->lineEditAviFile->setText(fileName);
	      // Show a preview image
	      if ( myModeType != EXPERT ) {
		// here we have to extract the image from the videos.
		/*
		  Dafr eine allgemeingltige GeneratePixmapClass erzeugen.
		  Wenn etwas ge�dert wird (Dateiname, oder rot/blau)
		  dann wird ein neues Pixmap erzeugt.
		  Wenn der Prozess des pixmap erzeugens am Ende ist,
		  wird ein slot aufgerufen, der es anzeigt.
		*/
		myModeDialog->setInputFile(LineEditInputFile->text());
		slotUpdateBasicModeImage();
		slotUpdateBasicModeRef();
	      }
	      slotGetInfo();
	      myDirTranscodeIn=myInputFile->dirPath();
	      QString outName;
	      outName=fileName.left(fileName.findRev("."));
	      // use myOutputTranscode, if set!
	      if ( !myOutputTranscode.isEmpty() ) {
		  int pfadLength=myInputFile->dirPath().length();
		  // this path has to be stripped from outName.
		  outName=outName.right( outName.length() - pfadLength -1 );
		  outName=myOutputTranscode+"/"+outName;
	       } else {
		  // set myDirTranscodeOut 
		  myDirTranscodeOut=myInputFile->dirPath();
	       }
	       LineEditOutputFile->setText(outName);
	} // End of file is reable
    } // end of myInputFile
  delete myInputFile;
}

/** Hier werden die Daten von dem AVI-File bestimmt */
void Kavi2svcd::slotGetInfo(){
  // Hier wird der Prozess nur noch gestartet.
  if ( !LineEditInputFile->text().isEmpty() ) {
    TextLabelCodec->setText("--");
    //TextLabelFrames->setText("--");
    lineEditFrames->setText("--");
    lineEditFrames->setEnabled(false);
    lineEditFPS->setText("--");
    lineEditFPS->setEnabled(false);
    //TextLabelWidth->setText("--");
    lineEditWidthX->setText("--");
    lineEditWidthX->setEnabled(false);
    //TextLabelHeight->setText("--");
    lineEditHeightY->setText("--");
    lineEditHeightY->setEnabled(false);
    // Cursor modifizieren (14.06.2005)
    this->setCursor( QCursor(Qt::WaitCursor) );
    // get the Info of the file
    myGetInfo->slotGetInfo( LineEditInputFile->text() );
  }
}
/** gets the codec from the getinfoClass */
void Kavi2svcd::slotGetInfoCodec(QString c){
  TextLabelCodec->setText(c);
  // Das können wir in einer der beliebigen Funktionen machen,
  // in denen ein emit vom getInfoClass gefangen wird.
  // Cursor modifizieren (14.06.2005)
  this->setCursor( QCursor(Qt::ArrowCursor) );
}
/** gets the FPS from the getinfoClass */
void Kavi2svcd::slotGetInfoFPS(QString f){
  myFps=f.toFloat();
  //TextLabelFrames->setText(QString("%1 / %2").arg(myFrames).arg(myFps));
  lineEditFPS->setText( QString("%1").arg(myFps) );
  if ( myFps == 0 ) {
    QMessageBox* error = new QMessageBox();
    error->setText(i18n("Oups, no frames per second found! Please insert the reasonble value!"));
    error->exec();
    lineEditFPS->setEnabled(true);
    delete error;
  }else{
  	// if we are in PAL and have 29.97 frames
	// if we are in NTSC and have 25 frames, 
	// we will set the filter
	// myVideoFormat=1 : PAL
	// myVideoFormat=0 : NTSC
	if ((myFps==25) &&(myVideoFormat==0)){
		FPS.jFilter=FPSClass::PALNTSC;	
	} else if ((myFps>29) && (myFps<30) && (myVideoFormat==1)) {
		FPS.jFilter=FPSClass::NTSCPAL;	
	} else {
		FPS.jFilter=FPSClass::OFF;	
	}
  }
  // Now I ve got the fps
  slotCalcVideoSize();
}
/** gets the frames from the getifnoclass */
void Kavi2svcd::slotGetInfoFrames(QString f){
  myFrames=f.toLong();
  myFrameLast=myFrames;
  myFrameFirst=1;
  //TextLabelFrames->setText(QString("%1 / %2").arg(myFrames).arg(myFps));
  lineEditFrames->setText(QString("%1").arg(myFrames));
  if ( myFrames ==0 ) {
    QMessageBox* error = new QMessageBox();
    error->setText(i18n("Oups, no number of frames found! Please insert some reasonalbe value."));
    error->exec();
    lineEditFrames->setEnabled(true);
    delete error;
  }
  // Now I have got the Frames
  slotCalcVideoSize();
  // Pass the frames to GeneratePixmapClass
  // if we are not in expert mode
  if ( myModeType != EXPERT ) {
    myModeDialog->setMaxFrames(myFrames);
  }
}
/** gets the height from the getifnoclass */
void Kavi2svcd::slotGetInfoHeight(QString h){
  myHeightY=h.toInt();
  if ( myHeightY!=0 ) {
    //TextLabelHeight->setText(h);
    lineEditHeightY->setText(h);
  } else {
    QMessageBox* error = new QMessageBox();
    error->setText(i18n("Oups, no height found! Please insert the height. You could test the file with mplayer. "));
    error->exec();
    lineEditHeightY->setEnabled(true);
    delete error;
  }
}
/** gets the frames from the getifnoclass */
void Kavi2svcd::slotGetInfoWidth(QString w){
  myWidthX=w.toInt();
  if ( myWidthX!=0 ) {
    //TextLabelWidth->setText(w);
    lineEditWidthX->setText(w);
  } else {
    QMessageBox* error = new QMessageBox();
    error->setText(i18n("Oups, no width found! Please insert the width. You could test the file with mplayer."));
    error->exec();
    lineEditWidthX->setEnabled(true);
    delete error;
  }
}
/** gets the duration from the getifnoclass */
void Kavi2svcd::slotGetInfoDuration(QString d){
      lineEditDuration->setText(d);
}

//------------------------------------------------------------------------------------------
//
// Erzeugen der Kommandozeilen
//
//-----------------------------------------------------------------------------------------

/** Erzeugt die Befehlszeile fr VCDimager */
void Kavi2svcd::slotGenerateVCD(){
  // vcdimager -t svcd -c /data2/video2.cue -b /data2/video2.bin
  // --volume-count=3 --volume-number=3 galaxy3.mpg
  // Only if there are input files
  if ( ListBoxMpegFiles->count() > 0 ) {
    // construct the vcd object and generate a command.
    VCDclass* vcd=new VCDclass();
    VCDclass::vcdData data;
    data.nice=myNiceVCDImager;
    data.additionalOption=myOptionVCD;
    data.binFile=LineEditBin->text();
    data.cueFile=LineEditCue->text();
    data.offset=checkBoxVCDScanOffsets->isChecked();
    data.command=LineEditVCDKommando->text();
    data.count=SpinBoxVCount->text();
    data.number=SpinBoxVNumber->text();
    data.broken=checkBoxVCDBrokenSVCD->isChecked();
    data.type=ComboBoxVCDIType->currentText();
    data.use2336=checkBoxVCD2336->isChecked();
    data.command=LineEditVCDKommando->text();
    QStringList sListe;
    for ( uint i=0; i< ListBoxMpegFiles->count(); i++) {
      QString test = ListBoxMpegFiles->text(i);
      sListe += test;
    }
    data.fileList=sListe;
    vcd->setData(data);
    LineEditVCDKommando->setText( vcd->generateKommando() );
    delete vcd;
  } // End of listbox not empty
}

/** Erzeugt den Mplex Befehl */
void Kavi2svcd::slotGenerateMplex(){
  QString befehl[8][3]={{(""),("mplex "),("tcmplex ")}, // command
			{(""),(" -S "),("")}, // max segment size
			{(""),(" -f "),(" -m ")}, // format / mode
			{(""),(" -O "),(" -D")}, // sync
			{(""),(" -o "),(" -o ")}, // output
			{(""),(" "),(" -i ")}, // video
			{(""),(" "),(" -p ")}, // audio 1
			{(""),(" "),(" -s ")} }; // audio 2
  // These are the possible modes of tcmplex
  QString mode[8]={ ("1"),("v"),("v"),("2"),("s"),("s"),("v"),("s") };
  QString sKommando;
  QString combo;
  int mplexType;
  if ( RadioButtonMplex->isChecked() ) mplexType=1; else mplexType=2;
  //  tcmplex -o test.mpg -i test.m1v -p test.mpa -m v
  //  mplex -f 4 -S 700 -o galaxy%d.mpg galaxy.m2v galaxy.mpa
  if ( !LineEditMplexVideo->text().isEmpty() ) {
    if ( myNiceMplex != 0 ) {
      sKommando.append(QString("nice -n %1 ").arg(myNiceMplex));
    }
    sKommando.append( befehl[0][mplexType]);
    sKommando.append(myOptionMplex);
    sKommando.append( befehl[2][mplexType]);
    if ( mplexType==1 ){
      combo=combo.setNum(ComboBoxMplexCodec->currentItem());
      sKommando.append(combo);
    } else {
      sKommando.append(mode[ComboBoxMplexCodec->currentItem()]);
    }
    if ( CheckBoxSegment->isChecked() &&
	 !LineEditMplexSize->text().isEmpty() &&
	 mplexType==1) {
      sKommando.append( befehl[1][mplexType] );
      sKommando.append(LineEditMplexSize->text());
    }
    if ( SpinBoxSync->text()!="0" ) {
      sKommando.append(befehl[3][mplexType]);
      sKommando.append(SpinBoxSync->text());
    }
    if ( !LineEditMplexOutput->text().isEmpty() ) {
      sKommando.append(befehl[4][mplexType]);
      sKommando.append('"');
      sKommando.append(LineEditMplexOutput->text());
      sKommando.append('"');
    }
    if (!LineEditMplexVideo->text().isEmpty()) {
      sKommando.append(befehl[5][mplexType]);
      sKommando.append('"');
      sKommando.append(LineEditMplexVideo->text());
      sKommando.append('"');
    }
    if (!LineEditMplexAudio1->text().isEmpty()) {
      sKommando.append(befehl[6][mplexType]);
      sKommando.append('"');
      sKommando.append(LineEditMplexAudio1->text());
      sKommando.append('"');
    }
    if (!LineEditMplexAudio2->text().isEmpty()) {
      sKommando.append(befehl[7][mplexType]);
      sKommando.append('"');
      sKommando.append(LineEditMplexAudio2->text());
      sKommando.append('"');
    }
  }
  LineEditMplexKommando->setText(sKommando);
}


/** Erzeugt die Kommandozeile von transcode */
void Kavi2svcd::slotGenerateTranscode(){
  QString sKommando;
  if ( !(LineEditInputFile->text().isEmpty()) )
    {
      if ( myNiceTranscode != 0 ) {
	sKommando.append(QString("nice -n %1 ").arg(myNiceTranscode));
      }
      sKommando.append("transcode ");
      if ( !myFramesAll ) {
	sKommando.append(" -c ");
	sKommando.append( QString("%1-%2").arg(myFrameFirst).arg(myFrameLast) );
      }
      // --encode_fields
      if ( myEncode >0 ) {
	      sKommando.append(" --encode_fields ");
	      sKommando.append(InterlaceClass::getStringEncoding(myEncode));
      }
      //if ( myKeepAspectRatio && myFast.isEmpty() ) {
      if ( ASR.keepASR && myFast.isEmpty() ) {
	sKommando.append(" --keep_asr");
      }
      if ( ASR.importRatio ) {
	sKommando.append(QString(" --import_asr %1").arg(ASR.importRatio));
      }
      // Im Falle von MPEGs muss der export_asr angegeben werden (4:3)
      //if ( FileTypeClass::fileType(LineEditInputFile->text())==FileTypeClass::MPG ) {
      //	sKommando.append(" --export_asr 2");
      //}
      if ( ASR.exportRatio ) {
	sKommando.append(QString(" --export_asr %1").arg(ASR.exportRatio));
      }
      // Frames per second
      if (!FPS.exportFPS.isEmpty()) {
      	sKommando.append(QString(" --export_fps %1").arg(FPS.exportFPS));
      }
      // Directory mode
      // This option is only set in OPTION mode.
      if ( myDirMode && (myDirModeType==DirectoryModeClass::OPTION)) {
	sKommando.append(" --dir_mode \"");
	sKommando.append( myBasename );
	sKommando.append("\"");
	if ( myDirModeSplit ) {
	  sKommando.append(" --no_split");
	}
      }
      sKommando.append(myOptionTranscode);
      sKommando.append(" -i ");
      sKommando.append('"');
      // Directory mode
      // Here only the directory as Input gets printed.
      if ( myDirMode ) {
    	QString h=LineEditInputFile->text();
     	int pos=h.findRev("/");
     	h=h.left( pos );
	sKommando.append( h);
	myDirModeFile=h;
      } else {
	sKommando.append(LineEditInputFile->text());
      }
      sKommando.append('"');
      if (!CheckBoxV->isChecked()) {
	//The -V is now the default behaviour.
	// If it is not checked, we will use RGB internally
	sKommando.append("  --use_rgb ");
      }
      if (CheckBoxZ->isChecked()) {
	sKommando.append(" -z ");
      }
      // Interlacing CORNY:01082003
      if ( myInterlacing > 0 ) {
    	sKommando.append( QString(" -I%1").arg( myInterlacing ) );
      }
      if (ComboBoxImportVideo->currentItem() != 0){
	sKommando.append(" -x "+ComboBoxImportVideo->currentText());
	// Add Option for mplayer , if we want subtitles and mplayer
	if (( ComboBoxImportVideo->currentText() == "mplayer" ) &&
	    mySubData.bUseSub && !mySubData.bTranscodeMode ) {
	  sKommando.append(QString("=\'-sub %1\'").arg(mySubData.subFile));
	}
	if (ComboBoxImportAudio->currentItem() != 0){ sKommando.append(","+ComboBoxImportAudio->currentText()+" ");   }
	sKommando.append(" ");
      }
      // Video output.
      if ( CheckBoxOnlyAudio->isChecked() ) {
    	sKommando.append(" -a "+SpinBoxAudioChannel->text());
	sKommando.append(" -y null,");
      } else {
	// transcode tool
	if ( ComboBoxTranscodeTool->currentText()=="mjpeg" ) {
	  sKommando.append(" -y mpeg2enc,");
	} else if (ComboBoxTranscodeTool->currentText()=="kvcd") {
	  sKommando.append(" -y mpeg2enc,");	
	}else { // bbmpeg
	  sKommando.append(" -y mpeg ");
	}
      }
      // Audio Output.
      if (  CheckBoxOnlyAudio->isChecked()  ||
	    (ComboBoxTranscodeTool->currentText()=="mjpeg") ||
	    (ComboBoxTranscodeTool->currentText()=="kvcd")  ) {
	if  (ComboBoxModulOutput->currentText().isEmpty()) {
	  sKommando.append("mp2enc");
	} else {
	  sKommando.append(ComboBoxModulOutput->currentText());
	}
      }
			
      if( RadioButtonDVD->isChecked()) 
	// DVD;
	sKommando.append(" -E 48000");
      else 
	sKommando.append(" -E 44100");
      sKommando.append(" -F");
      // transcode tool
      int combo;
      combo=ComboBoxCodec->currentItem();
      if ( (ComboBoxTranscodeTool->currentText()=="mjpeg")||(ComboBoxTranscodeTool->currentText()=="kvcd") ) {
	sKommando.append(QString("%1").arg(combo));
	if ( RadioButtonDVD->isChecked()) // DVD
	  sKommando.append(",-q4");
	else if (!LineEditSequenceLength->text().isEmpty()) { //mpeg2enc 
	  sKommando.append(",-S");
	  sKommando.append(LineEditSequenceLength->text());
	}
	if (ComboBoxTranscodeTool->currentText()=="kvcd") {
	  sKommando.append(",'-K kvcd'");
	}
      } else // bbmpeg
	switch (combo) {
	case 4:
	case 5:
	case 7:sKommando.append("s");
	  if ( myUseProfile ) sKommando.append(",/tmp/xsvcd.prof");
	  break;
	case 8:sKommando.append("d");
	  if ( myUseProfile ) sKommando.append(",,/tmp/xsvcd.prof");
	  break;
	default: sKommando.append("v");
	  break;
	}
      // end mjpeg
      if (CheckBoxSMP->isChecked()) {
	sKommando.append(",-M3");
      }
      if ( !LineEditVideoBitrate->text().isEmpty() ) {
	sKommando.append(" -w ");
	sKommando.append(LineEditVideoBitrate->text());
      }
      sKommando.append(" -b ");
      sKommando.append(LineEditBitrate->text());
      if ( myFast.isEmpty() ) { // Fast Resizing?
	sKommando.append(" -Z ");
	sKommando.append(LineEditResolution->text());
	if (CheckBoxFastZ->isChecked() ) sKommando.append(",fast");
	if (!(LineEditBorder->text()).isEmpty()) {
	  sKommando.append(" -j ");
	  sKommando.append(LineEditBorder->text());
	}
      } else {
	sKommando.append(" "+myFast+" ");
      }
      if (CheckBoxK->isChecked()) {
	sKommando.append(" -k ");
      }
      if (LineEditGamma->text() != "1.0") {
    	sKommando.append(" -G ");
     	sKommando.append(LineEditGamma->text());
      }
      // Subtitles
      if ( !mySubtitleCommand.isEmpty() ) {
    	sKommando.append(" "+mySubtitleCommand);
      }
      if ( !(LineEditOutputFile->text().isEmpty()) )
	{
	  if (LineEditSplitFrames->text().isEmpty()){
		sKommando.append(" -o ");
	  }else{
          	sKommando.append(" -t ");
                sKommando.append(LineEditSplitFrames->text());
                sKommando.append(",");
          }		
	  sKommando.append('"');
	  sKommando.append(LineEditOutputFile->text());
	  sKommando.append('"');
	}
	// -J Filter
	if ( FPS.jFilter==FPSClass::PALNTSC ) {
		sKommando.append(" -J fps=25:29.97");
	}else if (FPS.jFilter==FPSClass::NTSCPAL) {
		sKommando.append(" -J fps=29.97:25");
	}
    }
  LineEditKommandoTranscode->setText(sKommando);
}

/** Erzeugt die Kommandozeile fr den
    Brennprozess. */
void Kavi2svcd::slotGenerateCDR(){
  CDRclass* myCDR = new CDRclass();
  // set the gui data for cdr
  CDRclass::cdrData data;
  data.nice=myNiceCdrdao;
  data.additionalOption=myOptionCDR;
  data.device=LineEditCDRDevice->text();
  data.driver=ComboBoxCDRDriver->currentText();
  data.speed=SpinBoxCDRSpeed->text();
  data.eject=CheckBoxCDREject->isChecked();
  data.overburn=checkBoxCDROverburn->isChecked();
  data.simulate=CheckBoxCDRSimulate->isChecked();
  data.cdrw=CheckBoxBlankCDRW->isChecked();
  data.cueFile=LineEditCDRFile->text();
  data.command=LineEditCDRKommando->text();
  data.cdrType=myCDRType;
  myCDR->setData(data);
  // Output the result
  LineEditCDRKommando->setText(myCDR->generateKommando());
  delete myCDR;
}

//-----------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------
/** �fnet den Dialog fr die Outputdatei */
void Kavi2svcd::slotOpenOutputFile(){
  QString fileName;
  QFileDialog *myOutputFile = new QFileDialog(myDirTranscodeOut);
  // Directory -> AnyFile CORNY:01082003
  myOutputFile->setMode(QFileDialog::AnyFile);
  myOutputFile->setFilters(QString("*;;*.mpg;;*.mpeg;;*.m1v;;*.m2v"));
  if ( myOutputFile->exec() == QDialog::Accepted )
    {
      fileName = myOutputFile->selectedFile();
      LineEditOutputFile->setText(fileName);
      myDirTranscodeOut=myOutputFile->dirPath();
    } // Ende vom myInputFile
  delete myOutputFile;
}
//------------------------------------------------------------------------------
//
//  Mplexer
//
//-------------------------------------------------------------------------------
/** �fnet den DateiDialog, in dem der Output von Plex bestimmt wird. */
void Kavi2svcd::slotOpenMplexOutputFile(){
  QString fileName;
  QFileDialog *myFile = new QFileDialog(myDirMplexOut);
  myFile->setMode(QFileDialog::AnyFile);
  myFile->setFilters(QString("*.mpeg;;*.mpg;;*"));
  if ( myFile->exec() == QDialog::Accepted )
    {
      fileName = myFile->selectedFile();
      LineEditMplexOutput->setText(fileName);
      myDirMplexOut=myFile->dirPath();
      // hier einen Hinweis aufrufen, dass im Namen ein
      // %d vorkommen sollte.
      QMessageBox* info=new QMessageBox();
      QString errorStr=i18n("\nThe output filename must contain a %d\n so that mplex can generate several files,\nwhere the %d is replaced with a number.");
      info->setText( errorStr );
      info->exec();
    } // Ende vom myFile
  delete myFile;
}
/** �fnet Dialog, in dem die Datei bestimmt wird, die die Videodaten
    fr mplex enth�t: *.m1v oder *.m2v */
void Kavi2svcd::slotOpenVideoFile(){
  QString fileName;
  QFileDialog *myFile = new QFileDialog(myDirMplexVideo);
  myFile->setFilters(QString("*.m?v;;*.m1v;;*.m2v;;*"));
  if ( myModeType == EXPERT ){
    myFile->setMode(QFileDialog::ExistingFiles);
  }
  if ( myFile->exec() == QDialog::Accepted )
    {
      //fileName = myFile->selectedFile();
      myMplexFileList[0] = myFile->selectedFiles();
      if ( myMplexFileList[0].count()>1 ) {
	pushButtonBatchMplex->show();
	LineEditMplexOutput->setEnabled(false);
      } else {
	pushButtonBatchMplex->hide();
	LineEditMplexOutput->setEnabled(true);
      }
      QStringList::Iterator it = myMplexFileList[0].begin();
      while( it != myMplexFileList[0].end() ) {
	fileName=*it;
	++it;
      }
      LineEditMplexVideo->setText(fileName);	
      myDirMplexVideo=myFile->dirPath();
      myDirMplexAudio=myDirMplexVideo;
      // preset Audiofile: CORNY:01082003
      if ( LineEditMplexAudio1->text().isEmpty() ) {
      	QString audio=fileName.left( fileName.findRev(".") );
        LineEditMplexAudio1->setText( audio+".mpa" );
        QString out=audio;
       	// if myOutputMplex dir exists...
	if ( !myOutputMplex.isEmpty() ) {
	  // strip the path length
	  int pfadLength = myDirMplexVideo.length();
	  out = out.right( out.length() - pfadLength -1 );
	  out = myOutputMplex+"/"+out;
	}
        LineEditMplexOutput->setText( out+"-%d.mpg" );
      }  
    } // end of myFile
  delete myFile;
}
/** Audiofile mpa */
void Kavi2svcd::slotOpenAudioFile1(){
  QString fileName;
  QFileDialog *myFile = new QFileDialog(myDirMplexAudio);
  myFile->setFilters(QString("*.mpa;;*"));
  if ( myModeType == EXPERT ){
    myFile->setMode(QFileDialog::ExistingFiles);
  }
  if ( myFile->exec() == QDialog::Accepted )
    {
      //fileName = myFile->selectedFile();
      myMplexFileList[1] = myFile->selectedFiles();
      if ( myMplexFileList[1].count()>1 ) {
	pushButtonBatchMplex->show();
	LineEditMplexOutput->setEnabled(false);
      } else {
	pushButtonBatchMplex->hide();
	LineEditMplexOutput->setEnabled(true);
      }
      QStringList::Iterator it = myMplexFileList[1].begin();
      while( it != myMplexFileList[1].end() ) {
	fileName=*it;
	++it;
      }
      LineEditMplexAudio1->setText(fileName);
      myDirMplexAudio=myFile->dirPath();
    } // Ende vom myFile
  delete myFile;
}
/** Audiofile 2 mpa */
void Kavi2svcd::slotOpenAudioFile2(){
  QString fileName;
  QFileDialog *myFile = new QFileDialog(myDirMplexAudio);
  myFile->setFilters(QString("*.mpa;;*"));
  if ( myModeType == EXPERT ){
    myFile->setMode(QFileDialog::ExistingFiles);
  }
  if ( myFile->exec() == QDialog::Accepted )
    {
      //fileName = myFile->selectedFile();
      myMplexFileList[2] = myFile->selectedFiles();
      if ( myMplexFileList[2].count()>1 ) {
	pushButtonBatchMplex->show();
	LineEditMplexOutput->setEnabled(false);
      } else {
	pushButtonBatchMplex->hide();
	LineEditMplexOutput->setEnabled(true);
      }
      QStringList::Iterator it = myMplexFileList[2].begin();
      while( it != myMplexFileList[2].end() ) {
	fileName=*it;
	++it;
      }
      LineEditMplexAudio2->setText(fileName);
      myDirMplexAudio=myFile->dirPath();
    } // Ende vom myFile
  delete myFile;
}
/** Setzt den neuen maximalen Wert von Volume-Number entsprechend Volume Count. */
void Kavi2svcd::slotVolumeMax(int max){
  SpinBoxVNumber->setMaxValue(max);
}
//-------------------------------------------------------------------------------------
//
// starten externe Prozesse
//
//-----------------------------------------------------------------------------------
/** Ruft den Dialog auf, der den Transcode-Prozess
    enth�t und setzt das Kommando. */
void Kavi2svcd::slotStartTranscode(){
  bool autoClose, nichtWeiter;
  autoClose=false;
  nichtWeiter=false;
  QStringList::iterator it;
  QString inputOrig=LineEditInputFile->text();
  for ( it = myInputFileList.begin(); it != myInputFileList.end(); ++it ) {
    // if "it" differs from the LineEditInputFile, we have to set it an
    // to set the new outputfile.
    if ( (*it) != LineEditInputFile->text() ) {
      autoClose=true;
      QString inNameOld=LineEditInputFile->text();
      QString inName=(*it);
      LineEditInputFile->setText(inName);
      QString outName=inName.left(inName.findRev("."));
      QString outNameOld=inNameOld.left( inNameOld.findRev("."));
      // Replace the file names in the kommand line
      QString h=LineEditKommandoTranscode->text();
      h=h.replace( inNameOld, inName);
      h=h.replace( outNameOld, outName );
      LineEditKommandoTranscode->setText(h);
    }
    if ( (*it) == myInputFileList.last() ) { autoClose=false; }
    // if nichtWeiter, we should leave the loop.
    // But it does not work, so we do not start
    //   the other transcodes!
    if ( !nichtWeiter ) {
      nichtWeiter=StartTranscode(autoClose);
    }
  }
  LineEditInputFile->setText(inputOrig);
}

bool Kavi2svcd::StartTranscode(bool autoClose){
  bool canceled; canceled=true;
  if ( !LineEditKommandoTranscode->text().isEmpty() ) {
    if ( !LineEditOutputFile->text().isEmpty() ) {
      Transcode* myTranscode = new Transcode();
      // Logging
      myTranscode->setLogFile(myLogFile);
      if (myFramesAll) {
	myTranscode->setFrames(myFrames);
      } else {
	myTranscode->setFrames(myFrameLast - myFrameFirst+1);
      }
      myTranscode->setPlayer(myPreview);
      QString video=LineEditOutputFile->text();
      switch (ComboBoxCodec->currentItem()) {
      case 0:
      case 1:
      case 2:
      case 7: video.append(".m1v"); break;
      case 3:
      case 4:
      case 5:
      case 8:
      case 6: video.append(".m2v"); break;
      }
      myTranscode->setVideoOut(video);
      myTranscode->setDirMode(myDirMode);
      myTranscode->setDirModeFile(myDirModeFile);
      if ( myTranscode->setKommando(LineEditKommandoTranscode->text()) ){
	myTranscode->slotStart(autoClose);
	myTranscode->exec();
        canceled=myTranscode->getCanceled();
      }
      delete myTranscode;
    } else { // keine Ausgabe
      QMessageBox* error = new QMessageBox();
      error->setText(i18n("\nPlease add an output filename with the absolute path!\nOtherwise everything would be written to /dev/null"));
      error->exec();
      delete error;
    }
  }
  return canceled;
}

void Kavi2svcd::slotStartMplex(){
  bool autoClose, nichtWeiter;
  autoClose=false;
  nichtWeiter=false;
  // iterator for each List!
  QStringList::iterator itVideo;
  QStringList::iterator itAudio1;
  QStringList::iterator itAudio2;
  // the last entry as iterator:
  QStringList::iterator itVideoLast=myMplexFileList[0].end();
  itVideoLast--;
  // save old values
  QString videoOrig=LineEditMplexVideo->text();
  QString audio1Orig=LineEditMplexAudio1->text();
  QString audio2Orig=LineEditMplexAudio1->text();
  QString outOrig=LineEditMplexOutput->text();
  QString kommandoOrig=LineEditMplexKommando->text();
  // interate through the list
  itAudio1=myMplexFileList[1].begin();
  itAudio2=myMplexFileList[2].begin();
  for ( itVideo = myMplexFileList[0].begin(); itVideo != myMplexFileList[0].end(); ++itVideo ) {
    if ( (*itVideo) != videoOrig ) {
      autoClose=true;
      // insert the files into the LineEdits and replace the
      // values in the Command
      LineEditMplexVideo->setText( (*itVideo) );
      LineEditMplexAudio1->setText( (*itAudio1) );
      LineEditMplexAudio2->setText( (*itAudio2) );
      // Generate the outputfile!
      QString out=(*itVideo).left( (*itVideo).findRev(".") );
      // if myOutputMplex dir exists...
      if ( !myOutputMplex.isEmpty() ) {
	// strip the path length
	int pfadLength = myDirMplexVideo.length();
	out = out.right( out.length() - pfadLength -1 );
	out = myOutputMplex+"/"+out;
      }
      out=out.append("-%d.mpg");
      LineEditMplexOutput->setText( out );
      // Generation of output-file complete :-)
      // Replace the command
      QString h=kommandoOrig;
      h=h.replace( videoOrig, (*itVideo) );
      h=h.replace( audio1Orig, (*itAudio1) );
      h=h.replace( audio2Orig, (*itAudio2) );
      h=h.replace( outOrig, out );
      LineEditMplexKommando->setText(h);
    }
    // Execute now!
    if ( itVideo == itVideoLast ) { autoClose=false; }
    if ( !nichtWeiter ) {
      nichtWeiter=StartMplex(autoClose);
    }
    // step to next one!
    itAudio1++;
    itAudio2++;
  }
  // Restore the original values
  LineEditMplexVideo->setText(videoOrig);
  LineEditMplexAudio1->setText(audio1Orig);
  LineEditMplexAudio1->setText(audio2Orig);
  LineEditMplexOutput->setText(outOrig);
  LineEditMplexKommando->setText(kommandoOrig);
}
/** Hier wird der Dialog aufgerufen, der den MPlex Prozess enth�t. */
bool Kavi2svcd::StartMplex(bool autoClose){
  bool canceled; canceled=true;
  // Nur starten, wenn ein outputfile vorhanden ist.
  if ( !LineEditMplexOutput->text().isEmpty() )	{
    Mplex* myMplex = new Mplex();
    // Logging
    myMplex->setLogFile(myLogFile);
    if ( myMplex->setKommando(LineEditMplexKommando->text()) ){
      myMplex->setVideo(LineEditMplexVideo->text());
      myMplex->setAudio(LineEditMplexAudio1->text(), LineEditMplexAudio2->text());
      myMplex->setOutput( LineEditMplexOutput->text() );
      myMplex->slotStart(autoClose);
      myMplex->exec();
      canceled=myMplex->getCanceled();
      // check, if we shall delete the m?v and the mpa file
      if ( myAutomateData.runIt && myAutomateData.mplexDelete && !canceled) {
	QFile file1, file2;
	file1.setName( LineEditMplexVideo->text() );
	file2.setName( LineEditMplexAudio1->text() );
	file1.remove();
	file2.remove();
      }
    }
    delete myMplex;
  }
  return canceled;
}

/** starts the vcd process when the button was pressed. */
void Kavi2svcd::slotStartVCD(){
  StartVCD(false,0);
}
/** Ruft den Prozess zum
    vcdimager auf. */
bool Kavi2svcd::StartVCD(bool autoClose, int num, int totalNum){
  bool canceled; canceled=true;
  if ( !LineEditVCDKommando->text().isEmpty() ) {
    if ( !LineEditCue->text().isEmpty() && !LineEditBin->text().isEmpty() ) {
      VCDclass* myVcd = new VCDclass();
      // Logging
      myVcd->setLogFile(myLogFile);
      // set all values from the gui
      VCDclass::vcdData data;
      data.nice=myNiceVCDImager;
      data.additionalOption=myOptionVCD;
      data.binFile=LineEditBin->text();
      data.cueFile=LineEditCue->text();
      data.offset=checkBoxVCDScanOffsets->isChecked();
      data.command=LineEditVCDKommando->text();
      data.count=SpinBoxVCount->text();
      data.number=SpinBoxVNumber->text();
      data.broken=checkBoxVCDBrokenSVCD->isChecked();
      data.type=ComboBoxVCDIType->currentText();
      data.use2336=checkBoxVCD2336->isChecked();
      data.command=LineEditVCDKommando->text();
      if ( myAutomateData.runIt ) { // OK, we were called from automate
	// set total number
	myVcd->setNum( num, totalNum );
	QString sNum; sNum=sNum.setNum(num);
	// Replace the %d in the mpeg-file-List
	QStringList sListe;
	for ( uint i=0; i< ListBoxMpegFiles->count(); i++) {
	  QString test = ListBoxMpegFiles->text(i);
	  test=test.replace("%d", sNum);
	  sListe += test;
	}
	data.fileList=sListe;
	// Replace the %d in the cue file
	data.cueFile=LineEditCue->text().replace("%d", sNum);
	// Replace the %d in the bin file
	data.binFile=LineEditBin->text().replace("%d", sNum);
	// Replace the command. If the command is empty, it will be generated.
	data.command="";
      } else { // num=0
	// Set the data, if not in automate
	QStringList sListe;
	for ( uint i=0; i< ListBoxMpegFiles->count(); i++) {
	  QString test = ListBoxMpegFiles->text(i);
	  sListe += test;
	}
	data.fileList=sListe;
      }
      myVcd->setData(data);
      if ( myVcd->start() ) {
	myVcd->slotStart( autoClose );
	myVcd->exec();
	// true, if the process was canceled
	canceled=myVcd->getCanceled();
	// check if we shall delete the mpeg file
	if ( myAutomateData.vcdDelete && myAutomateData.runIt && !canceled) {
	  // data.fileList will be deleted;
	  for ( QStringList::Iterator it = data.fileList.begin(); it != data.fileList.end(); ++it ) {
	    QFile file;
	    file.setName(*it);
	    file.remove();
	  }
	}
      }
      delete myVcd;
    } else { // Kein cue file und bin file angegeben.
      QMessageBox* error = new QMessageBox();
      error->setText(i18n("\nPlease add a cue file and a bin file, which will be generated."));
      error->exec();
      delete error;
    }
  }
  return canceled;
}

/** Startet den brennprozess. */
void Kavi2svcd::slotStartCDR(){
  StartCDR(false,0);
}
bool Kavi2svcd::StartCDR(bool autoClose, int num, int totalNum){
  bool canceled; canceled=true;
  if ( !LineEditCDRKommando->text().isEmpty() ) {
    if ( CheckBoxBlankCDRW->isChecked() && CheckBoxBlankCDRW->isVisible() ) {
      QMessageBox* hinweis = new QMessageBox ("Blank your CDRW",
					      "Blanking CDRW using the command\n"+myBlankCDRWCmd+"\nPlease insert a CDRW to blank\nor skip it.",
					      QMessageBox::Information ,
					      QMessageBox::Ok     | QMessageBox::Default,
					      QMessageBox::Abort  | QMessageBox::Escape,
					      QMessageBox::NoButton );
      hinweis->exec();
      if ( hinweis->result() == QDialog::Accepted ) {
	// Start to blank CDRW
	CDRWclass* myCDRW = new CDRWclass();
	if (myCDRW->setKommando( myBlankCDRWCmd ) )
	  myCDRW->exec();
	delete myCDRW;
      }
      delete hinweis;
    }
    CDRclass* myCDR = new CDRclass();
    // Logging
    myCDR->setLogFile(myLogFile);
    // set the gui data for cdr
    CDRclass::cdrData data;
    data.nice=myNiceCdrdao;
    data.additionalOption=myOptionCDR;
    data.device=LineEditCDRDevice->text();
    data.driver=ComboBoxCDRDriver->currentText();
    data.speed=SpinBoxCDRSpeed->text();
    data.eject=CheckBoxCDREject->isChecked();
    data.overburn=checkBoxCDROverburn->isChecked();
    data.simulate=CheckBoxCDRSimulate->isChecked();
    data.cdrw=CheckBoxBlankCDRW->isChecked();
    data.cueFile=LineEditCDRFile->text();
    data.command=LineEditCDRKommando->text();
    if ( myAutomateData.runIt ){ // automate mode
      data.cueFile=data.cueFile.replace("%d", QString("%1").arg(num));
      data.command="";
    }
    myCDR->setData(data);
    myCDR->setCDRType(myCDRType);
    if (myCDR->start() ) {
      if ( myAutomateData.runIt ) {
	QMessageBox* error=new QMessageBox();
	error->setText( i18n("Please insert a CD in your drive\nCD no. %1 of %2 will be created.").arg(num).arg(totalNum) );
	error->setIcon( QMessageBox::Information );
	error->exec();
	delete error;
      }
      myCDR->slotStart( autoClose );
      myCDR->exec();
      // true, if the vcd process was canceled
      canceled=myCDR->getCanceled();
      // check wether we shall delete the cue and the bin file file!
      if (myAutomateData.cdrDelete && myAutomateData.runIt && !canceled) {
	QFile file1, file2;
	file1.setName(data.cueFile);
	file1.remove();
	QString sBin; sBin=data.cueFile.replace(".cue",".bin");
	file2.setName(sBin);
	file2.remove();
      }
    }
    delete myCDR;
  }
  return canceled;
}


//-------------------------------------------------------------------------------------
//
//   VCDIMAGER
//
//---------------------------------------------------------------------------------------
/** �fnet Dialog auf vcdimager */
void Kavi2svcd::slotOpenVCDMpeg(){
  QString fileName;
  QFileDialog *myFile = new QFileDialog(myDirVCDIn);
  myDirVCDIn=myFile->dirPath();
  myFile->setMode(QFileDialog::ExistingFiles);
  myFile->setFilters(i18n("mpeg files (*.mpeg *.mpg *.MPG *.MPEG);;*.mpeg;;*.mpg;;*.MPG;;*.MPEG"));
  if ( myFile->exec() == QDialog::Accepted )
    {
      //    	fileName = myFile->selectedFile();
      //			LineEditMpegFile->setText(fileName);
      myDirVCDIn=myFile->dirPath();

      QStringList list = myFile->selectedFiles();
      QStringList::Iterator it = list.begin();
      while( it != list.end() ) {
	vcdAdd( *it );
	LineEditMpegFile->setText(*it);
	++it;
      }
    } // Ende vom myFile
  delete myFile;
}
/** W�lt eine bin-Datei aus
    fr vcdimager */
void Kavi2svcd::slotOpenVCDbin(){
  QString fileName;
  QFileDialog *myFile = new QFileDialog(myDirVCDOut);
  myFile->setMode(QFileDialog::AnyFile);
  myFile->setFilters(QString("*.bin;;*"));
  if ( myFile->exec() == QDialog::Accepted )
    {
      fileName = myFile->selectedFile();
      // wenn filename nicht auf bin endet, eines dranh�gen.
      if ( fileName.find(".bin", fileName.length()-4) == -1 ) {
	fileName.append(".bin");
      }
      LineEditBin->setText(fileName);
      myDirVCDOut=myFile->dirPath();
    } // Ende vom myFile
  delete myFile;
}
/** W�lt eine cue Datei aus
    fr vcdimager */
void Kavi2svcd::slotOpenVCDcue(){
  QString fileName;
  QFileDialog *myFile = new QFileDialog(myDirVCDOut);
  myFile->setMode(QFileDialog::AnyFile);
  myFile->setFilters(QString("*.cue;;*"));
  if ( myFile->exec() == QDialog::Accepted )
    {
      fileName = myFile->selectedFile();
      // wenn filename nicht auf bin endet, eines dranh�gen.
      if ( fileName.find(".cue", fileName.length()-4) == -1 ) {
	fileName.append(".cue");
      }
      LineEditCue->setText(fileName);
      myDirVCDOut=myFile->dirPath();
      slotVCDBinAusCue();
    } // Ende vom myFile
  delete myFile;
}
/** Fgt das File zur Liste */
void Kavi2svcd::slotVCDadd(){
  vcdAdd( LineEditMpegFile->text() );
}
/** Adds a string to the mpeg list for vcdimager */
void Kavi2svcd::vcdAdd(QString s){
  ListBoxMpegFiles->insertItem( s );
  int pathLength=myDirVCDIn.length();
  // Now, cue and bin are preset.
  s=s.left( 	s.findRev(".") );
  // if myOutputVCD is set!
  if ( !myOutputVCD.isEmpty() ) {
    s = s.right( s.length() - pathLength -1 );
    s = myOutputVCD+"/"+s;
  }
  if ( LineEditCue->text().isEmpty() ) {
    LineEditCue->setText( s + ".cue" );
    LineEditBin->setText( s + ".bin" );
    if ( LineEditCDRFile->text().isEmpty() ) {
      LineEditCDRFile->setText( s+".cue" );
    }
  }
  TextLabelVCDTotalSize->setText(getVCDTotalSize());
}


/** L�cht den aktuellen Eintrag */
void Kavi2svcd::slotVCDremove(){
  ListBoxMpegFiles->removeItem( ListBoxMpegFiles->currentItem() );
  TextLabelVCDTotalSize->setText(getVCDTotalSize());
}
/** Schiebt den currentItem() einen nach oben. */
void Kavi2svcd::slotVCDup(){
  if ( ListBoxMpegFiles->currentItem() > 0 ) {
    QString sEintrag = ListBoxMpegFiles->currentText();
    // currentItem ge�dert
    ListBoxMpegFiles->changeItem( ListBoxMpegFiles->text( ListBoxMpegFiles->currentItem()-1 ),
				  ListBoxMpegFiles->currentItem() );
    ListBoxMpegFiles->changeItem( sEintrag, ListBoxMpegFiles->currentItem()-1 );
  }
}
/** Schiebt den currentItem() einen nach unten. */
void Kavi2svcd::slotVCDdown(){
  if ( (int)ListBoxMpegFiles->currentItem() < (int)ListBoxMpegFiles->count()-1 ) {
    QString sEintrag = ListBoxMpegFiles->currentText();
    ListBoxMpegFiles->changeItem( ListBoxMpegFiles->text( ListBoxMpegFiles->currentItem()+1 ),
				  ListBoxMpegFiles->currentItem() );
    ListBoxMpegFiles->changeItem( sEintrag, ListBoxMpegFiles->currentItem()+1 );
  }
}

/** Wenn Checkbox bei mplex auf Auto
    gesetzt wird, braucht man den Tab
    vcdimager nicht mehr. */
void Kavi2svcd::slotVCDauto(){
  bool vcdAuto = !CheckBoxVCDauto->isChecked();
  GroupBoxVCDOut->setEnabled(vcdAuto);
  GroupBoxVCDIn->setEnabled(vcdAuto);
  LineEditVCDKommando->setEnabled(vcdAuto);
  PushButtonVCD->setEnabled(vcdAuto);
  LineEditMplexOutput->setEnabled(vcdAuto);
  PushButtonMplexOutput->setEnabled(vcdAuto);
  // Nur wenn VCDs erstellt werden, ist es auch m�lich CDs zu brennen.
  CheckBoxCDRauto->setEnabled(!vcdAuto);
  if ( vcdAuto ) { // Wenn also keine VCDImages mehr erstellt werden sollen...
    CheckBoxCDRauto->setChecked( false );
  }
  if ( !CheckBoxVCDauto->isChecked() ) {
    LineEditPrefix->setEnabled( false );
    TextLabelPrefix->setEnabled( false );
  } else {
    TextLabelPrefix->setEnabled( true);
    LineEditPrefix->setEnabled( true );
  }
  // ge�dert 19.08.2002
  LineEditMplexKommando->setEnabled(vcdAuto);
  PushButtonMplex->setEnabled(vcdAuto);
  TextLabelMplex->setEnabled(vcdAuto);

  // Den Menpunkt generate aktivieren.setNum(myLast)
  kaviMenu->setItemEnabled( 10,!vcdAuto);

  // Auch nochmal den CDRDAO berprfen.
  slotCDRauto();
}

/** Deaktiviert das CDR-Tab... */
void Kavi2svcd::slotCDRauto(){
  bool cdrAuto=!CheckBoxCDRauto->isChecked();
  GroupBoxCDR->setEnabled(cdrAuto);
  LineEditCDRKommando->setEnabled(cdrAuto);
  PushButtonCDR->setEnabled(cdrAuto);
  // Wenn CDs gebrannt werden, ist es egal, wo die vcd-images stehen!
  if ( CheckBoxVCDauto->isChecked() ) {
    TextLabelPrefix->setEnabled(cdrAuto);
    LineEditPrefix->setEnabled(cdrAuto);
  }
}

/** Wenn Die Zeile Outputfile bei mplex ge�dert wird, wird gleich das
    Inputfile bei vcdimager
    gefllt. */
void Kavi2svcd::slotMplexOutVCDIn(){
  LineEditMpegFile->setText( LineEditMplexOutput->text() );
}
//------------------------------------------------------------------------
//
// CDRDAO
//
//--------------------------------------------------------------------------

/** �fnet das cuefile, das gebrannt werden soll. */
void Kavi2svcd::slotOpenCDRcue(){
  QString fileName;
  QFileDialog *myFile = new QFileDialog(myDirCDRIn);
  myFile->setMode(QFileDialog::ExistingFile);
  myFile->setFilters(QString("*.cue;;*"));
  if ( myFile->exec() == QDialog::Accepted )
    {
      fileName = myFile->selectedFile();
      LineEditCDRFile->setText(fileName);
      myDirCDRIn=myFile->dirPath();
    } // Ende vom myFile
  delete myFile;
}

//---------------------------------------------------------------------------------------------------
//
//  Preferences
//
//---------------------------------------------------------------------------------------------------
/** Ruft den Dialog auf, 
    der die Preferences anzeigt und
    in dem man sie �dern kann. */
void Kavi2svcd::slotPreferences(){
  PrefClass* myPrefs = new PrefClass(this, "test", true, myConfigProfile);
  myPrefs->fillImportModules(myImportVideo);
  myPrefs->exec();
  updateCaption();
  delete myPrefs;
  this->statusBar()->message(i18n("Profile: ")+myConfigProfile);
}
/** Setzt die Variablen
    myVideoPfad
    myStandardType
    myStandardSize
    myScsiDevice
    myBurnSpeed
    myEject
    myOptionTranscode
    myOptionMplex
    myOptionVCD
    myOptionCDR
    myDriverList
    myDriverStandard
*/
void Kavi2svcd::setPreferences(prefData data){
  // basic or expert mode?
  myModeType=(modeType)data.startup;
  setVideoFormat(data.videoFormat);
  // output directories
  myOutputTranscode=data.outputTranscode;
  myOutputMplex=data.outputMplex;
  myOutputVCD=data.outputVCD;
  mySubView=data.subtitleViewProgram;
  mySubEdit=data.subtitleEditProgram;
  myLogFile=data.logFile;
  myCheckPrograms=data.programsNotCheck;
  // nice-level:
  if ( data.niceTranscode ) {	myNiceTranscode=data.niceValueTranscode; } else {myNiceTranscode=0;}
  if ( data.niceMplex ) { myNiceMplex=data.niceValueMplex; } else {myNiceMplex=0;}
  if ( data.niceVcd ) { myNiceVCDImager=data.niceValueVcd; } else {myNiceVCDImager=0;}
  if ( data.niceCdr ) { myNiceCdrdao=data.niceValueCdr; } else {myNiceCdrdao=0;}

  mySegmentSize=data.segment;
  myStandardSize=data.size;
  myScsiDevice=data.device;
  myBurnSpeed=data.speed;
  myEject=data.eject;
  myOverburn=data.overburn;
  myDVDBitrate=data.DVDBitrate;
  myK=data.K;
  myV=data.V;
  myZ=data.Z;
  myBlankCDRW=data.bBlankCDRW;
  myBlankCDRWCmd=data.sBlankCDRW;
  if ( myBlankCDRWCmd.isEmpty() ) {
    CheckBoxBlankCDRW->hide();
  } else {
    CheckBoxBlankCDRW->show();
  }
  myPreview=data.previewCommand;
  myPreviewCUE=data.previewCUE;
  myOptionTranscode=data.transcodeOption;
  myOptionMplex=data.mplexOption;
  myOptionVCD=data.vcdOption;
  myOptionCDR=data.cdrOption;
  myDriverList=data.driverList;
  myEncode=data.encoding;
  myDVDBitrate=data.DVDBitrate;
	                                                                 	                                
  myConfigProfile=data.profile;
  myVideoPfad=data.editDir;
  myStandardType=data.standardVcdType;
  // CDR Type
  // 1= cdrdao
  // 2= cdrecord
  int oldCDRType=myCDRType;
  // check wether the cdrdao-line has to be regenerated
  if (data.useCDRDAOnotCDRECORD) { myCDRType=1; } else { myCDRType=2; }
  if ( oldCDRType != myCDRType ) { slotGenerateCDR(); }

  RadioButtonFrei->setChecked(true);	
  if ( myStandardType==4 ) RadioButtonSVCD->setChecked( true );
  if ( myStandardType==1 ) RadioButtonVCD->setChecked(true);
  if ( myStandardType==8 ) RadioButtonDVD->setChecked(true);
  slotVCD();
  //   ComboBoxMplexCodec->setCurrentItem( myStandardType );
  if ( myStandardType==4 ) {
    ComboBoxVCDIType->setCurrentItem( 2 ); // SVCD
  } else {
    ComboBoxVCDIType->setCurrentItem( 1 ); // VCD2.0
  }
  ComboBoxCDRDriver->clear();
  ComboBoxCDRDriver->insertStringList(myDriverList);
  myDriverStandard=data.driver;
  ComboBoxCDRDriver->setCurrentItem(myDriverStandard);

  // Pfade initialisieren;
  initOpenDirs();

  // set the widget contents
  // this must be done at the end, since several signals are emitted
  // when the widget-contents is changed!!!
  LineEditMplexSize->setText( mySegmentSize );
  LineEditSequenceLength->setText( myStandardSize );
  LineEditCDRDevice->setText( myScsiDevice );
  SpinBoxCDRSpeed->setValue(myBurnSpeed);
  CheckBoxCDREject->setChecked( myEject );
  checkBoxCDROverburn->setChecked( myOverburn );
  CheckBoxK->setChecked( myK );
  CheckBoxV->setChecked( myV );
  CheckBoxZ->setChecked( myZ );
  CheckBoxFastZ->setChecked( data.fastZ );
  CheckBoxBlankCDRW->setChecked( myBlankCDRW );
  ComboBoxTranscodeTool->setCurrentText( data.TranscodeTool );
  
  ComboBoxImportVideo->setCurrentText(data.ImportModule);
  FPS.exportFPS=data.ExportFPS;
  ASR.keepASR= data.KeepASR;
  ASR.importRatio= data.ImportASR;
  ASR.exportRatio= data.ExportASR;
}

/** sets wether PAL=1 or NTSC=0 */
void Kavi2svcd::setVideoFormat(int i){
  myVideoFormat=i;
  // set the Resolution:
  // PAL:
  //  SVCD = 480x576 128 kbit audio
  //  VCD = 352x288  224 kbit audio
  //  25 fps
  // NTSC:
  //  SVCD = 480x480 128 kbit Audio
  //  VCD = 352x240  224 kbit audio
  //  30 fps video mode, 24 fps film mode
  if ( myVideoFormat==1 ) { // PAL
    myResolutionSVCD="480x576";
    myAudioBitrateSVCD="128";
    myResolutionVCD="352x288";
    myAudioBitrateVCD="224";
  } else if ( myVideoFormat==0 ) { //NTSC
    myResolutionSVCD="480x480";
    myAudioBitrateSVCD="128";
    myResolutionVCD="352x240";
    myAudioBitrateVCD="224";
  }
  // at the end: set the widgets
  slotVCD();
}


//--------------------------------------------------------------
//
// Generieren des Scriptes
//
//--------------------------------------------------------------

/** Generiert in GenerateScriptClass (myScript) das Script.
    Zuvor mssen die Werte gesetzt werden. */
void Kavi2svcd::slotGenerateScript(){
  // Dieser Befehl darf nur aufgerufen werden, wenn die Checkbox aktiviert ist.
  // Das Menuitem muss mit der Checkbox CheckBoxVCDauto aktiviert und deaktiviert werden.
  myScript->setAudio1( LineEditMplexAudio1->text() );
  myScript->setAudio2( LineEditMplexAudio2->text() );
  myScript->setVideo( LineEditMplexVideo->text() );
  myScript->setCDsBrennen( CheckBoxCDRauto->isChecked() );
  myScript->setDevice( LineEditCDRDevice->text() );
  myScript->setPrefix( LineEditPrefix->text() );
  myScript->setSize( LineEditMplexSize->text() );
  myScript->setSpeed( SpinBoxCDRSpeed->value() );
  myScript->setType( ComboBoxMplexCodec->currentItem() );
  myScript->setSync( SpinBoxSync->text() );

  myScript->generateScript();

}
/** Wird von dem Meneintrag Save aufgerufen. */
void Kavi2svcd::fileSave(){
  myScript->saveScript( myVideoPfad );
}
/** slot, der dann den Load-Befehl in GenerateScriptClass aufruft. */
void Kavi2svcd::fileOpen(){
  if ( myScript->loadScript( myVideoPfad ) ) {

  }
}
/** Wird aufgerufen, wenn ein Script geladen wurde. */
void Kavi2svcd::slotScriptLoaded( QString type, QString size, QString cdsbrennen,
				  QString speed, QString device, QString prefix,
				  QString video, QString audio1, QString audio2, QString sync ){
  // Tab mplex
  LineEditMplexVideo->setText(video);
  LineEditMplexAudio1->setText(audio1);
  LineEditMplexAudio2->setText(audio2);
  CheckBoxVCDauto->setChecked(true);
  CheckBoxCDRauto->setChecked( (cdsbrennen == "1") );
  LineEditMplexSize->setText(size);
  LineEditSequenceLength->setText(size);
  LineEditPrefix->setText(prefix);
  ComboBoxMplexCodec->setCurrentItem(type.toInt());
  slotVCDauto();
  slotCDRauto();
  // Tab cdrdao
  LineEditCDRDevice->setText(device);
  CheckBoxCDREject->setChecked(true);
  SpinBoxCDRSpeed->setValue( speed.toInt() );
  SpinBoxSync->setValue( sync.toInt() );
}
//----------------------------------------------------------------------------
// Transcode
//-----------------------------------------------------------------------------
/** Wird aufgerufen, wenn die Checkbox
    FastResize Options angew�lt wurde. */
void Kavi2svcd::slotFastResize(){
  bool bFast = CheckBoxFastResize->isChecked();
  LineEditBorder->setEnabled( !bFast );
  SliderBlackBorder->setEnabled( !bFast );
  if ( bFast ) {
    // Berechne die Werte
    QString zielRes = LineEditResolution->text();
    QString sx = zielRes.left( zielRes.find("x") );
    QString sy = zielRes.right( zielRes.length() - zielRes.find("x") -1);
    int x = sx.toInt();
    int y = sy.toInt();
    myFast = calculateFastResize(x, y ,myWidthX, myHeightY);
  } else {
    myFast = "";
  }
  slotGenerateTranscode();
}


/** Berechnet den String -j -X -B, fr das Fast Resizing.
    Die Werte mssen durch 32 teilbar sein. */
QString Kavi2svcd::calculateFastResize(int zielX, int zielY, int x, int y){
  // SOJUS: Change this
  // zielX und zielY sind die Zielgr�e des Frames
  // x und y sind die aktuelle Gr�e
  // Ausgegeben wird als String die notwendige Optionenzeile.
  int divideX=32;
  int divideY=32;
  if ( zielX % 32 ) { 
  	if ( zielX % 16 ) {
		if ( zielX % 8){
			divideX=0;
		} else divideX=8;
	}else divideX=16;
  }
  if ( zielY % 32 ) {
  	if ( zielY % 16 ) {
		if ( zielY % 8 ){
			divideY=0;
		} else divideY=8;
	}else divideY=16;
  }
  if ( (divideX==0) || (divideY==0) ) {
    QMessageBox* error=new QMessageBox();
    QString errorStr=i18n("WRONG RESOLUTION");
    errorStr.append(i18n("\nYou can not use fast resize!\n"));
    errorStr.append(QString(i18n("The Resolutin %1 x %2 of the destination is neither devidable by 32, 16 nor 8.")).arg(zielX).arg(zielY));
    error->setText( errorStr );
    error->setIcon( QMessageBox::Warning );
    error->exec();
    delete error;
    //return i18n("WRONG RESOLUTION");
    return i18n("");
  };
  int neuX, neuY;
  QString BX, BY, XX, XY, JX, JY;
  // Aspektratio bercksichtigen.
  int borderY = (int)(0.75*x - y);
  int borderX=0;
  // neue H�e mit dem schwarzen Rand
  neuY = y + borderY;
  neuX = x;

  // Rest vom Modulo berechnen und auf die Aufl�ung aufaddieren.
  // X: 592 / 32 = 18.5   (Rest 16)
  // y: 444 / 32 = 13.875 (Rest 4)
  int restX = (neuX % divideX);
  int restY = (neuY % divideY);
	
  if ( restX > 0 ) { restX = divideX-restX; };
  if ( restY > 0 ) { restY = divideY-restY; };

  // Die Reste mssen draufaddiert werden auch fr die Option -J
  borderX = borderX+restX;
  borderY = borderY+restY;
  // Nun werden die Reste, die fehlen damit die Werte durch 32 teilbar sind,
  // draufaddiert.

  neuX = neuX + restX;
  neuY = neuY + restY;

  // Wie sieht die Differenz zur Zielaufl�ung aus?

  if ( zielX > neuX ){
    XX.setNum( (int)( (zielX-neuX)/divideX) );
    BX.setNum( 0 );
  } else {
    XX.setNum( 0 );
    BX.setNum( (int)( (neuX-zielX)/divideX) );
  }

  if ( zielY > neuY ){
    XY.setNum( (int)( (zielY-neuY)/divideY) );
    BY.setNum( 0 );
  } else {
    XY.setNum( 0 );
    BY.setNum( (int)( (neuY-zielY)/divideY) );
  }
  // Reihenfolge der Optionen: -j -X -B
  JY.setNum( (-1)*( borderY/2 ) );
  JX.setNum( (-1)*( borderX/2 ) );
  return QString( "-j "+JY+","+JX+" -X "+XY+","+XX+",%1 -B "+BY+","+BX+",%2").arg(divideX).arg(divideY);
}
/** Setzt die Daten fr das Hilfemen */
void Kavi2svcd::setAboutData(KAboutData* data){
  myAboutData = data;
}
/** Wird aufgerufen, wenn das Widget seinen Focus verliert. */
void Kavi2svcd::focusOutEvent(QFocusEvent *){
  LineEditInputFile->setText(i18n("ready"));

}
/** Zeigt den Gamma wert vom Slider */
void Kavi2svcd::slotGamma(int v){
  double gamma;
  if (v <= 10) {
    gamma=(double) v/10;
  } else { // 11 und mehr
    gamma= v - 9;
  }
  QString sGamma; sGamma.setNum( gamma, 'f', 1);
  LineEditGamma->setText(sGamma);
}
/** Ruft die Hilfe der Kapplication auf. */
void Kavi2svcd::helpContents(){
  kapp->invokeHelp();
}

/** Berechnet die Optionen fr fastresize. Von sojus */
QString Kavi2svcd::calculateFastResizeNEW(int zielX, int zielY, int x, int y){
  int deltaX=x-zielX;
  int deltaY=y-zielY;
  int moddX=deltaX%32;
  int moddY=deltaY%32;
  int divX=deltaX/32;
  int divY=deltaY/32;

  QString JX,JY,XX,BX,XY,BY;
  JX.setNum(moddX/2);
  JY.setNum(moddY/2);

  if ( divX<0 ){
    XX.setNum( -divX );
    BX.setNum( 0 );
  } else {
    XX.setNum( 0 );
    BX.setNum( divX );
  }

  if ( divY<0 ){
    XY.setNum( -divY );
    BY.setNum( 0 );
  } else {
    XY.setNum( 0 );
    BY.setNum( divY );
  }
  QString ret=QString("-j "+JY+","+JX+" -X "+XY+","+XX+" -B "+BY+","+BX);
  return ret;
}

/** If the value for the black border got changed */
void Kavi2svcd::borderChanged(int value){
  int iRandY = myBorder.toInt();
  int iRandX=0;
  iRandY = iRandY + value*3;
  iRandX = iRandX + value*4;
  LineEditBorder->setText(QString("%1,%2").arg(iRandY).arg(iRandX));
}

/** Generate the list of the Import_Libs */
void Kavi2svcd::initLibList_neu(){
  int iModules=47;
  // [ name, import , export ]
  QString possibleModules[47][3]={
    {("ac3"),("a"),("")},
    {("af6"),("av"),("av")},
    {("avi"),("av"),("")},
    {("divx4"),(""),("av")},
    {("divx4raw"),(""),("av")},
    {("divx5"),(""),("av")},
    {("divx5raw"),(""),("av")},
    {("divx"),("v"),("")},
    {("dv"),("av"),("av")},
    {("dvd"),("av"),("")},
    {("dvraw"),(""),("av")},
    {("fame"),(""),("av")},
    {("ffmpeg"),("v"),("av")},
    {("imlist"),("v"),("")},
    {("im"),("v"),("v")},
    {("jpg"),(""),("v")},
    {("lame"),(""),("a")},
    {("lav"),("av"),("")},
    {("lzo"),("v"),("av")},
    {("mjpeg"),("v"),("av")},
    {("mov"),("av"),("av")},
    {("mp2enc"),(""),("a")},
    {("mp3"),("a"),("")},
    {("mpeg2enc"),(""),("v")},
    {("mpeg2"),("v"),("")},
    {("mpeg3"),("v"),("")},
    {("mpeg"),(""),("av")},
    {("mplayer"),("av"),("")},
    {("net"),(""),("av")},
    {("null"),("av"),("av")},
    {("nvrec"),("av"),("")},
    {("ogg"),("av"),("av")},
    {("pcm"),(""),("a")},
    {("ppm"),(""),("av")},
    {("raw"),("av"),("av")},
    {("rawlist"),("v"),("")},
    {("sampeg"),(""),("v")},
    {("toolame"),(""),("a")},
    {("v4l"),("av"),("")},
    {("vdrac3"),("a"),("")},
    {("vob"),("av"),("")},
    {("wav"),(""),("a")},
    {("xml"),("av"),("")},
    {("xvid"),("v"),("av")},
    {("xvidcvs"),(""),("av")},
    {("xvidraw"),(""),("av")},
    {("yuv4mpeg"),("av"),("av")} };
											
  myImportVideo.clear();
  myImportVideo.append(i18n("automatic"));
  myImportAudio.clear();
  myImportAudio.append(i18n("automatic"));
  myExportVideo.clear();
  myExportAudio.clear();

  // Bestimme myTranscodePath mit Hilfe von tcmodinfo -p
  InitClass* check = new InitClass();
  if ( check->getTranscodeLib() ) {  // alles klar, es existiert ein Verzeichnis mit den Transcodelibs
    myTranscodePath=check->getAusgabe();
    // cut the Newline
    myTranscodePath=myTranscodePath.left( myTranscodePath.findRev("\n") );
    if (debug) printf("Found transcode path [%s]\n", myTranscodePath.latin1());
    // Liste der import-module erstellen.
    QDir d(myTranscodePath,"import_*.so");
    d.setFilter( QDir::Files);
    d.setSorting( QDir::Name );
    if ( debug ) printf("IMPORT: Number of directory entries: %i\n",d.count());
    for ( uint i = 0; i < d.count(); i++ ) {
      QString name = d[i];
      if ( debug ) printf( "%i : name = %s\n", i, name.latin1());
      // "import_" abziehen.
      name=name.right(name.length()-7);
      if ( debug ) printf( "     name = %s =>",  name.latin1());
      // ".so" abziehen.
      name=name.left(name.length()-3);
      if ( debug ) printf( "     name = %s\n",  name.latin1());
      // check first, if this module is for video or audio
      bool found=false;
      for (int i=0; i<iModules; i++) {
      	if (possibleModules[i][0]==name) {
	  found=true;
	  // Import
	  if (possibleModules[i][1].find("a")!=-1){ myImportAudio.append(name);}
	  if (possibleModules[i][1].find("v")!=-1){ myImportVideo.append(name);}
	}
      }
      if (!found) {
      	// if it is a module, not found in the list, append in both A/V
	myImportAudio.append(name);
	myImportVideo.append(name);
      }
    }
    // Export Module
    d=QDir(myTranscodePath,"export_*.so");
    d.setFilter( QDir::Files);
    d.setSorting( QDir::Name );
    if ( debug ) printf("EXPORT: Number of directory entries: %i\n",d.count());
    for ( uint i = 0; i < d.count(); i++ ) {
      QString name = d[i];
      if ( debug ) printf( "%i : name = %s\n", i, name.latin1());
      // "export_" abziehen.
      name=name.right(name.length()-7);
      if ( debug ) printf( "     name = %s =>",  name.latin1());
      // ".so" abziehen.
      name=name.left(name.length()-3);
      if ( debug ) printf( "     name = %s\n",  name.latin1());
      bool found=false;
      for (int i=0; i<iModules; i++) {
      	if (possibleModules[i][0]==name) {
	  found=true;
	  // Export
	  if (possibleModules[i][2].find("a")!=-1){ myExportAudio.append(name);}
	  if (possibleModules[i][2].find("v")!=-1){ myExportVideo.append(name);}
	}
      }
      if (!found) {
      	// if it is a module, not found in the list, append in both A/V
	myExportAudio.append(name);
	myExportVideo.append(name);
      }

    }
    ComboBoxImportVideo->insertStringList(myImportVideo);
    ComboBoxImportAudio->insertStringList(myImportAudio);
    ComboBoxModulOutput->insertStringList(myExportAudio);
    ComboBoxModulOutput->setCurrentText("mp2enc");
  } else {
    QMessageBox* error = new QMessageBox();
    error->setText(i18n("It seems there is no library-directory of transcode.\nI can't find any import/export modules."));
    error->exec();
    delete error;
    myTranscodePath="";
  }
  delete check;


}

/** Wird aufgerufen, wenn die Mplex Option -S angew�lt wird. */
void Kavi2svcd::slotSegmentClicked(){
  if (CheckBoxSegment->isChecked() ) {
    LineEditMplexSize->setEnabled( true );
  } else {
    LineEditMplexSize->setEnabled( false );
  }
}
/** Wenn die cue Datei eingegeben wurde, wird die bin datei berechnet. */
void Kavi2svcd::slotVCDBinAusCue(){
  QString fileName2=LineEditCue->text();
  if ( !fileName2.isEmpty() )
    if ( fileName2.length()>3 ) {
      LineEditBin->setText( fileName2.replace( fileName2.length()-3,3,"bin") );
    } else {
      LineEditBin->setText( fileName2+".bin" );
    }
}
/** Initialisiert alle Strings, in denen die Pfade der Dateidialoge gespeichert werden. */
void Kavi2svcd::initOpenDirs(){
  myDirTranscodeIn=myVideoPfad;
  myDirTranscodeOut=myVideoPfad;
  myDirMplexAudio=myVideoPfad;
  myDirMplexVideo=myVideoPfad;
  myDirMplexOut=myVideoPfad;
  myDirVCDIn=myVideoPfad;
  myDirVCDOut=myVideoPfad;
  myDirCDRIn=myVideoPfad;
}
/** This calculates the Video bitrate if an Filesize is given. */
void Kavi2svcd::slotCalcVideoBitrate(){
  if ( LineEditVideoSize->hasFocus() ){
    long Sekunden = (long)(myFrames / myFps);
    bool ok1=true;
    bool ok2=true;
    int iBitAudio = LineEditBitrate->text().toInt(&ok1);
    int iBitVideo;
    int iFileSize = LineEditVideoSize->text().toInt(&ok2);
    /*
      long size = ( iBitAudio + iBitVideo ) * Sekunden; // Size in kBit
      size = size / (8 * 1024  );  //  Kbyte, Mbyte
      // In case of SVCD, the expected size is about 30% smaller
      // Due to variable bit rate
      int iItem=ComboBoxCodec->currentItem();
      switch ( iItem ) {
      case 3: case 4: case 5: case 7: size=(long)(size*0.7); break;

      We shall only do this in case of SVCD!
    */
    // calculate size in kbit
    long size=iFileSize*(8*1024);
    size=(long)(size/0.7);
    iBitVideo = (int)(size/Sekunden)-iBitAudio;
    LineEditVideoBitrate->setText( QString("%1").arg(iBitVideo) );
  }
}
/** Berechnet aus den Angaben der Bitraten die Gr�e der Datei. */
void Kavi2svcd::slotCalcVideoSize(){
  if ( LineEditVideoBitrate->hasFocus() ) {
    long Sekunden = (long)(myFrames / myFps);
    bool ok1=true;
    bool ok2=true;
    int iBitAudio = 0;
    iBitAudio=LineEditBitrate->text().toInt(&ok1);
    int iBitVideo=0;
    // Within the Videobitrate-Editline could also be the
    // keyframes and the crispness!
    if ( !LineEditVideoBitrate->text().isEmpty() ) {
      QString hVB=LineEditVideoBitrate->text();
      int iPos=LineEditVideoBitrate->text().find(",");
      if ( iPos>0 ) {
	hVB=hVB.left( iPos );
      }
      iBitVideo = hVB.toInt(&ok2);
    } else {
      // nothing is inserted in the VideoBitrate lineedit
      // assume the standard: SVCD: 2600kBits / VCD: 1152 kBits
      int iItem=ComboBoxCodec->currentItem();
      switch ( iItem ) {
      case 0:	case 1: case 2: case 6: iBitVideo=1152; break;
      case 3: case 4: case 5: case 7: iBitVideo=2600; break;
      }
    }
    // Calculate overall bitrate
    if ( ok1 && ok2 ) {
      long size = ( iBitAudio + iBitVideo ) * Sekunden; // Size in kBit
      size = size / (8 * 1024  );  //  Kbyte, Mbyte
      // In case of SVCD, the expected size is about 30% smaller
      // Due to variable bit rate
      int iItem=ComboBoxCodec->currentItem();
      switch ( iItem ) {
      case 3: case 4: case 5: case 7: size=(long)(size*0.7); break;
      }
      LineEditVideoSize->setText(QString("%1").arg(size));
    } else {
      LineEditVideoSize->setText("???");
    }
  }
}
/** Wird ausgefhrt, wenn die Checkbox only audio gedrckt ist. */
void Kavi2svcd::slotOnlyAudio(){
  if ( CheckBoxOnlyAudio->isChecked() ) {
    SpinBoxAudioChannel->setEnabled(true); }
  else {
    SpinBoxAudioChannel->setEnabled(false); }
}
/** Wenn die ComboBox Tool ver�dert wurde. */
void Kavi2svcd::slotToolChanged(){
  PushButtonProfile->setEnabled( 	( ComboBoxTranscodeTool->currentText()=="bbmpeg" ) );
  if ( ComboBoxTranscodeTool->currentText()=="bbmpeg" ) {
    if ( myUseProfile ) {
      copyProfile();
    }
    LineEditSequenceLength->hide();
    TextLabelSequenceLength->hide();
  } else {
    LineEditSequenceLength->show();
    TextLabelSequenceLength->show();
  }
}
/** Startet den Profile Dialog fr bbmpeg */
void Kavi2svcd::slotProfile(){
  Profile* profDialog = new Profile( this, "bbmpeg profile");
  profDialog->setUseProfile( myUseProfile );
  profDialog->exec();
  if ( myUseProfile ) { copyProfile(); }
  delete profDialog;
  slotGenerateTranscode();
}
/** if a profile file xsvcd shallbe used. */
void Kavi2svcd::setUseProfile(bool bProfile){
  myUseProfile=bProfile;
}
/** The contents of the profile file */
void Kavi2svcd::setProfile( QStringList profile){
  myProfileList = profile;
}
/** copys the profile from the homedir to tmp */
void Kavi2svcd::copyProfile(){
  QString datei;
  QString kdeHome=QString(getenv( "KDEHOME" ));
  if (kdeHome.isEmpty()) {
    datei=QDir::homeDirPath();
    datei.append("/.kde");
  } else {
    datei=kdeHome;
  }
  datei.append("/share/apps/kavi2svcd/xsvcd.prof");
  system( "cp -f "+datei+" /tmp/xsvcd.prof" );
}
/** Reagiert auf einen Maustastendruck */
void Kavi2svcd::mousePressEvent ( QMouseEvent * e ) {
  if ( e->button() == Qt::RightButton ) {
    if ( debug ) printf("Right button pressed...\n");
    // Show context menu
    ContextMenu* kmenu = new ContextMenu( myTab->currentPageIndex(), this );
    kmenu->setAspectRatio( myKeepAspectRatio );
    kmenu->setASR( ASR.exportRatio || ASR.importRatio || ASR.keepASR );
    kmenu->setFPS( !FPS.exportFPS.isEmpty() || (FPS.jFilter!=FPSClass::OFF ) );
    kmenu->setDirMode(myDirMode);
    kmenu->setInterlace( (myInterlacing!=0) );
    kmenu->setRangeofFrames( (myFrameFirst!=1)&&(myFrameLast!=myFrames));
    kmenu->setSubtitle( mySubData.bUseSub );
    kmenu->exec(e->globalPos());
    delete kmenu;		
  }
}
/** change ASR */
void Kavi2svcd::slotChangeASR(){
  myKeepAspectRatio=!myKeepAspectRatio;
  slotGenerateTranscode();
}
/** Here you can set the import and the export aspect ratio */
void Kavi2svcd::slotASR(){
  AspectRatioClass* asrDlg=new AspectRatioClass();
  asrDlg->setData( ASR );
  if (asrDlg->exec()){
    asrDlg->getData( ASR );
    slotGenerateTranscode();
  }
  delete asrDlg;
}
/** Here we specify the Frames per second for import and export */
void Kavi2svcd::slotFPS(){
  FPSClass* fpsDlg=new FPSClass();
  fpsDlg->setData( FPS );
  if (fpsDlg->exec()){
    fpsDlg->getData( FPS );
    slotGenerateTranscode();
  }
  delete fpsDlg;
}
/** Starts the dialog for range of frames */
void Kavi2svcd::slotRangeOfFrames(){
  RangeOfFramesClass* rangeDialog = new RangeOfFramesClass( this );
  rangeDialog->setMinMax( 1, myFrames );
  rangeDialog->setRange( myFrameFirst, myFrameLast );
  if ( rangeDialog->exec() ) {
    slotGenerateTranscode();
  }
  delete rangeDialog;
}
/** Sets the range of the frames to be encoded */
void Kavi2svcd::slotSetFrameRange( bool all, long start, long end){
  myFramesAll = all;
  myFrameFirst = start;
  myFrameLast = end;
}
/** Starts the dialog for interlcing
 */
void Kavi2svcd::slotInterlace(){
  InterlaceClass* interlaceDialog = new InterlaceClass( this );
  interlaceDialog->setInterlacing(myInterlacing);
  interlaceDialog->setEncoding(myEncode);
  if ( 	interlaceDialog->exec() ) {
    myInterlacing=interlaceDialog->getInterlacing();
    myEncode=interlaceDialog->getEncoding();
    slotGenerateTranscode();
  }
  delete interlaceDialog;
}

//
// Preset of the input lines
//

/** Fills the mplex-Tab, when the output file of transcode was changed */
void Kavi2svcd::slotFillMplexTab(){
  /* Changed this due to request from
     Michael Pederson */
  QString fileName=LineEditOutputFile->text();
  // if ( LineEditMplexAudio1->text().isEmpty() ) {
  LineEditMplexAudio1->setText( fileName+".mpa" );
  // }
  // if ( LineEditMplexVideo->text().isEmpty() ){
  if ( RadioButtonVCD->isChecked() ){
    LineEditMplexVideo->setText( fileName+".m1v");
  }else{
    LineEditMplexVideo->setText( fileName+".m2v");
  }
  // }
  // if ( LineEditMplexOutput->text().isEmpty() ){
  LineEditMplexOutput->setText( fileName+"-%d.mpg");
  // }
}

/** Presets the VCD-Tab */
void Kavi2svcd::slotFillVCDTab(){
  LineEditMpegFile->setText(LineEditMplexOutput->text());
  slotVCDadd();
  slotVCDremove();
}


/** Starts the dialog directory mode. */
void Kavi2svcd::slotDirectoryMode(){
  DirectoryModeClass* dirDialog = new DirectoryModeClass( this );
  // initialize
  dirDialog->setDirMode(myDirMode);
  dirDialog->setDirModeType(myDirModeType);
  dirDialog->setNoSplit(myDirModeSplit);
  dirDialog->setFilename( LineEditInputFile->text() );
  if ( myDirMode ) {
    dirDialog->setBasename( myBasename );
  }
  if ( 	dirDialog->exec() ) {
    // get values
    myDirMode= dirDialog->getDirMode();
    myDirModeType=dirDialog->getDirModeType();
    myDirModeSplit=dirDialog->getNoSplit();
    myBasename=dirDialog->getBasename();
    slotGenerateTranscode();
  }
  delete dirDialog;
}
/** Gets the border from the getInfo class. */
void Kavi2svcd::slotGetInfoBorder(QString s){
  myBorder=s;
  LineEditBorder->setText( myBorder);
  int iMaxAdd = myBorder.toInt();
  if (iMaxAdd<0) { iMaxAdd=-iMaxAdd; }
  SliderBlackBorder->setMaxValue( iMaxAdd/3 );
  slotFastResize();
}
//-----------------------------------------------------------------------------
// A U T O M A T I O N
/** Starts the Dialog for Automation */
//
//-----------------------------------------------------------------------------
void Kavi2svcd::slotAutomate(){
  bool bOK;
  QString sInfo;
  bOK=true;
  myAutomateData.filename=LineEditMpegFile->text();
  myAutomate->setData(myAutomateData);
  if (myAutomate->exec() ) {
    myAutomate->getData(myAutomateData);
    // User wants transcode, but there is no cammand!
    if ( myAutomateData.transcode && LineEditKommandoTranscode->text().isEmpty() ) {
      sInfo=(i18n("You have to issue a transcode command, before automation is possible.\n"));
      bOK=false;
    }
    // User wants mplex, but has no command!
    if ( myAutomateData.mplex && LineEditMplexKommando->text().isEmpty() ) {
      sInfo.append(i18n("You have to issue an mplex command, before automation is possible.\n"));
      bOK=false;
    }
    // User wants vcdimager, but there is no input file and no outputfile
    if ( myAutomateData.vcd &&
	 (LineEditMpegFile->text().isEmpty() || LineEditCue->text().isEmpty()) ) {
      sInfo.append(i18n("You have to issue an mpegfile on the vcdimager-tab and\n a cue file on the same tab, before automation is possible.\n"));
      bOK=false;
    }
    // User wants cdrdao, but there is no cue/bin-file
    if ( myAutomateData.cdr && LineEditCDRFile->text().isEmpty() ) {
      sInfo.append(i18n("You have to issue a cue-file on the CD Burning-Tab,before automation is possible."));
      bOK=false;
    }
    // Show all the problems
    if ( !bOK ) {
      QMessageBox* error=new QMessageBox(this);
      error->setIcon(QMessageBox::Warning);
      error->setText( sInfo );
      error->exec();
      delete error;
    } else {
      slotRunAutomate();
    } // end of bOK=true;
  } // End of exec=true. The OK Button was pressed.
  // In the End, we have to assure, that not again, automate will be done!
  myAutomateData.runIt=false;
}
//-------------------------------------------------------------------------
//
// A U T O M A T E
//
//--------------------------------------------------------------------------
/** runs automated process */
void Kavi2svcd::slotRunAutomate(){
  // if bOK execute all automatically
  // we have to add a bool if the cancel button is
  // pressed somewhere
  bool canceled; canceled=false;
  myAutomateData.runIt=true;
  myProjectProgress.ready=false;
  if ( myAutomateData.transcode && !myProjectProgress.transcodeOK ) {
    // if this is not the last process, there may not be the Close-Button!
    canceled=StartTranscode(myAutomateData.mplex);
    // Here we have to save the state for the project file.
    // so that we can continue the process lateron
    if ( !canceled ) {
      myProjectProgress.transcodeOK=true;
      if ( !myProjectFile.isEmpty() ) { saveProject(); }
    }
  }
  if ( myAutomateData.mplex && !canceled && !myProjectProgress.mplexOK ) {
    // if this is not the last process, there may not be the Close-Button!
    canceled=StartMplex(myAutomateData.vcd);
    // Here we have to save the state for the project file.
    // so that we can continue the process lateron
    if ( !canceled ) {
      myProjectProgress.mplexOK=true;
      if ( !myProjectFile.isEmpty() ) { saveProject(); }
    }
  }
  if ( myAutomateData.vcd && !canceled) {
    // Set the right name for the vcdimager process.
    QString sFile=(myAutomateData.filename).replace("\%d","*");
    QString sPath=sFile.left(sFile.findRev("/"));
    sFile=sFile.right(sFile.length()-sFile.findRev("/")-1);
    // get all files with video-%d.mpg!
    QDir d(sPath, sFile); // myAutomateData.filename
    d.setFilter( QDir::Files);
    d.setSorting( QDir::Name );
    if ( debug ) printf("AUTOMATE: get number of mpeg files: %i\n",d.count());
    int start=myProjectProgress.vcdimagerNum;
    if (myProjectProgress.cdrNum<start) {start=myProjectProgress.cdrNum;}
    for ( uint i = (start-1); i < d.count(); i++ ) {
      if ( !canceled ) {
	QString name = d[i];
	bool autoClose;
	autoClose=true;
	// if it is the last file and no CDs are burned!
	if ( (i+1)==d.count() && !myAutomateData.cdr ) autoClose=false;
	canceled=StartVCD(autoClose, (int)i+1, d.count());
	// Here we have to save the state for the project file.
	// so that we can continue the process lateron
	if ( !canceled ) {
	  myProjectProgress.vcdimagerOK=true;
	  myProjectProgress.vcdimagerNum=i+1;
	  if ( !myProjectFile.isEmpty() ) { saveProject(); }
	}
	// After VCDImager, we also burn the CD.
	if ( myAutomateData.cdr && !canceled ) {
	  if ( (i+1)==d.count() ) { // the last CD!
	    autoClose=false; } else {
	      autoClose=true;
	    }
	  canceled=StartCDR(autoClose, (int)i+1, d.count());
	  // Here we have to save the state for the project file.
	  // so that we can continue the process lateron
	  if ( !canceled ) {
	    myProjectProgress.cdrOK=true;
	    myProjectProgress.cdrNum=i+1;
	    if ( !myProjectFile.isEmpty() ) { saveProject(); }
	  }
	}
      } // End of !canceled
    } // End of uint -> d.count
    myProjectProgress.ready=true;
  } // End of myAutomateData.vcd
  myAutomateData.runIt=false;
  myAutomationCanceled=canceled;
  // If the process was canceled
  // we might save the project!
  if (myAutomationCanceled) {
    if ( myProjectFile.isEmpty() ) {
      this->statusBar()->message(i18n("You can save this state in a project to continue the automation later!"),3000);
    } else {
      saveProject();
    }
  }
}


/** If the cue file in the mplex-Tab is changed, this is given to the CD Burning-Tab */
void Kavi2svcd::slotCueFileChanged(){
  LineEditCDRFile->setText(LineEditCue->text());
}

//---------------------------------------------------------------
//
// S U B T I T L E R
//
//----------------------------------------------------------------
/** Starts the subtitler dialog */
void Kavi2svcd::slotSubtitler(){
  mySubtitleCommand="";
  SubtitlerClass* subTitle = new SubtitlerClass( this );
  mySubData.path=myDirTranscodeIn;myProjectProgress.transcodeOK=false;
  mySubData.movieFile=LineEditInputFile->text();
  mySubData.subView=mySubView;
  mySubData.subEdit=mySubEdit;
  mySubData.fps=myFps;
  subTitle->setData(mySubData);
  if ( subTitle->exec() ) {
    mySubtitleCommand=subTitle->getCommand();
    mySubData=subTitle->getData();
    if ( mySubData.bUseSub && !mySubData.bTranscodeMode ) {
      ComboBoxImportVideo->setCurrentText("mplayer");			
      this->statusBar()->message(i18n("Please note: I set the import module to mplayer!"),3000);
    }
    slotGenerateTranscode();
  } 
  delete subTitle;
}
/** Calculates the total size of all selected mpeg-files in vcd-tab */
QString Kavi2svcd::getVCDTotalSize(){
  double totalSize;
  totalSize=0;
  int num=ListBoxMpegFiles->count();
  for (int i=0; i<num; i++) {
    QString fileName=ListBoxMpegFiles->text(i);
    QFile f; f.setName(fileName);
    double size=f.size();
    totalSize+=size;
  }
  QString sSize;
  sSize.setNum(totalSize/(1024*1024), 'f', 0 );
  sSize=sSize+" MB";
  return sSize;
}
/** Gets the whole file info of the input file */
void Kavi2svcd::slotGetInfoFileInfo(QString s){
  myInputFileInfo=s;
}
/** Displays the info of the input file */
void Kavi2svcd::slotShowFileInfo(){
  QMessageBox* info=new QMessageBox();
  info->setIcon(QMessageBox::Information);
  info->setText(myInputFileInfo);
  info->exec();
  delete info;
}
/** Is executed, when the file infos Frame, Fps, width height in the lineedits are changed */
void Kavi2svcd::slotFileInfosChanged(){
  myFrames=lineEditFrames->text().toLong();
  myFps=lineEditFPS->text().toFloat();
  myWidthX=lineEditWidthX->text().toInt();
  myHeightY=lineEditHeightY->text().toInt();
  slotCalcVideoSize();
  int border=(int)((-1)*(0.75*myWidthX-myHeightY)/2);
  LineEditBorder->setText( QString("%1").arg( border) );
}

/** plays the cue-file */
void Kavi2svcd::slotViewCue(){
  // Get this from the preferences
  QString command=myPreviewCUE;
  myPreviewProc->clearArguments();
  while (command.length()>0) {
    int pos=command.find(" ");
    QString param=command.left(pos);
    command=command.right(command.length()-pos-1);
    if (pos==-1) command="";
    // TODO does not work for files with blank in it!
    param.replace("%f",LineEditCDRFile->text());
    //		param.replace(" ","\ ");
    myPreviewProc->addArgument( param );
  }
  if (! myPreviewProc->start() ) {
    QMessageBox* error = new QMessageBox();
    error->setText(i18n("\nError starting Preview.\nPlease set the Preview program for cue-files in the preferences!\n"));
    error->exec();
    delete error;
  }
}

//-------------------------------------------------------------------
//
// Functions need for  B a s i c  M o d e
//
//-------------------------------------------------------------------

/** switches from svcd or vcd mode to expert mode */
void Kavi2svcd::slotSwitchExpert(){
  // automatisieren
  // reset values
  myAutomateData.runIt=false;
  myAutomateData.transcode=false;
  myAutomateData.mplex=false;
  myAutomateData.mplexDelete=false;
  myAutomateData.vcd=false;
  myAutomateData.vcdDelete=false;
  myAutomateData.cdr=false;
  myAutomateData.cdrDelete=false;
  myTab->show();
  myModeDialog->hide();
  myModeType=EXPERT;
}
/** switches from Expert to svcd mode */
void Kavi2svcd::slotSwitchSVCDMode(){
  // change myModeType
  if (RadioButtonVCD->isChecked() ) {
    myModeType=VCD;
  } else if (RadioButtonSVCD->isChecked() ) {
    myModeType=SVCD;
  } else {
    myModeType=DVD;
  }
  // automatisieren
  myAutomateData.runIt=false;
  myAutomateData.transcode=true;
  myAutomateData.mplex=true;
  myAutomateData.mplexDelete=true;
  myAutomateData.vcd=true;
  myAutomateData.vcdDelete=true;
  myAutomateData.cdr=true;
  myAutomateData.cdrDelete=true;
  myTab->hide();
  switch ( myModeType ) {
  case SVCD: myModeDialog->radioButtonSVCD->setChecked(true); break;
  case VCD: myModeDialog->radioButtonVCD->setChecked(true); break;
  case ASK: break;
  case EXPERT: break;
  case DVD: myModeDialog->radioButtonDVD->setChecked(true); break;
  }
  // take the values from the expert mode:
  myModeDialog->checkBoxKmode->setChecked( CheckBoxK->isChecked() );
  if (RadioButtonVCD->isChecked()) {
    myModeDialog->radioButtonVCD->setChecked(true);
  } else if (RadioButtonSVCD->isChecked() ) {
    RadioButtonSVCD->setChecked(true);
    myModeDialog->radioButtonSVCD->setChecked(true);
  } else if (RadioButtonDVD->isChecked() ) {
    RadioButtonDVD->setChecked(true);
    myModeDialog->radioButtonDVD->setChecked(true);
  }
  // Now show the dialog
  myModeDialog->show();
  // calculate a new image
  if ( !LineEditInputFile->text().isEmpty() ) {
    myModeDialog->setInputFile(LineEditInputFile->text());
    slotUpdateBasicModeImage();
    slotUpdateBasicModeRef();
  }
}

/** Opens the AVi File load dialog for SVCD/VCD Mode */
void Kavi2svcd::slotOpenAviFile(){
  // also insert the Inputfile on the transcode tab.
  slotOpenInputFile();
  // Everything is done on the transcode tab in the background.
}

/** when the checkbox -k on SVCD/VCD mode gets toggeled */
void Kavi2svcd::slotKtoggled(){
  // also check the box on the transcode Tab.
  CheckBoxK->setChecked( myModeDialog->checkBoxKmode->isChecked() );
  // calculate a new image.
  if ( !LineEditInputFile->text().isEmpty() ) {
    slotUpdateBasicModeImage();
  }
}
/*!
    \fn Kavi2svcd::slotzToggled()
 */
void Kavi2svcd::slotzToggled()
{
 // also check the box on the transcode Tab.
  CheckBoxZ->setChecked( myModeDialog->checkBoxz->isChecked() );
  // calculate a new image.
  if ( !LineEditInputFile->text().isEmpty() ) {
    slotUpdateBasicModeImage();
  }
}
/** Gets executed, when the start button on SVCD/VCD Mode is pressed. */
void Kavi2svcd::slotStartSVCDMode(){
  // TODO are all the necessary values set?
  // i.e. vcd mpegs and cdr data?
  slotGenerateTranscode();
  slotRunAutomate();
}
/** gets the toggled checkbox from myModeDialog */
void Kavi2svcd::slotVCDToggled(bool b){
  RadioButtonVCD->setChecked(b);
}
void Kavi2svcd::slotDVDToggled(bool b){
  RadioButtonDVD->setChecked(b);
}
/** gets the toggled checkbox from myModeDialog */
void Kavi2svcd::slotSVCDToggled(bool b){
  RadioButtonSVCD->setChecked(b);
}
/** updates the new image to be shown in the basic mode dialog */
void Kavi2svcd::slotUpdateBasicModeImage(){
  myModeDialog->setOption("k",CheckBoxK->isChecked());
  myModeDialog->setOption("V",CheckBoxV->isChecked());
  myModeDialog->setOption("z",CheckBoxZ->isChecked());
  if ( !myOutputTranscode.isEmpty() ) {
    myModeDialog->setOutputDir(myOutputTranscode);
  }
  myModeDialog->clearPixmap();
  // this command gets executed
  // transcode -i Futurama\ -\ Die\ Galaxis\ Des\ Terrors.avi -c 10-11 -y im,null -o testbild.jpg  -z -k
  // test process
  myModeDialog->startProcess(false);
  // if the process exits, the slot
  // myModeDialog->slotShowPixmap(Qstring) is called
}
/** updates the reference image */
void Kavi2svcd::slotUpdateBasicModeRef(){
  if ( !myOutputTranscode.isEmpty() ) {
    myModeDialog->setOutputDir(myOutputTranscode);
  }
  myModeDialog->clearPixmapRef();
  // this command gets executed
  // transcode -i Futurama\ -\ Die\ Galaxis\ Des\ Terrors.avi -c 10-11 -y im,null -o testbild.jpg  -z -k
  // reference process
  myModeDialog->startProcess(true); 
  // if the process exits, the slot
  // myModeDialog->slotShowPixmap(Qstring) is called
}
//-----------------------------------------------------------------------
//
// P R O J E C T Load/Save
//
//------------------------------------------------------------------------

/** Loads the old data from a project */
void Kavi2svcd::slotLoadProject(){
  QString filename = QFileDialog::getOpenFileName(
						  QDir::homeDirPath(),
						  i18n("project files (*.kaviprj)"),
						  this,
						  i18n("load project file"),
						  i18n("load project file") );
  if ( !filename.isEmpty() ) {
    myProjectFile=filename;
    loadProject();
  }
  if (!myProjectProgress.ready) {
    switch( QMessageBox::warning( this, "Continue Project...",
				  i18n("\nThis project was not finished correctly.\n")+
				  i18n("Do you want to continue the process, where it ended?"),
				  i18n("Yes"),
				  i18n("No"), 0, 0, 1 ) ) {
    case 0: // The user clicked the Yes button or pressed Enter
      // try again
      slotRunAutomate();
      break;
    case 1: // The user clicked the No or pressed Escape
      myProjectProgress.transcodeOK=false;
      myProjectProgress.mplexOK=false;
      myProjectProgress.vcdimagerOK=false;
      myProjectProgress.cdrOK=false;
      myProjectProgress.vcdimagerNum=0;
      myProjectProgress.cdrNum=0;
      myProjectProgress.ready=true;
      break;
    }
  }
  updateCaption();
}
/** Saves the data as a Porject */
void Kavi2svcd::slotSaveAsProject(){
  QString filename = QFileDialog::getSaveFileName(
						  QDir::homeDirPath(),
						  i18n("project files (*.kaviprj)"),
						  this,
						  i18n("save project file"),
						  i18n("save project file") );
  if ( !filename.isEmpty() ) {
    if (filename.right(8)!=".kaviprj") {
      filename.append(".kaviprj");
    }
    myProjectFile=filename;
    saveProject();
  }
  updateCaption();
}
/** Saves all necessary data to a project */
void Kavi2svcd::slotSaveProject(){
  if ( myProjectFile.isEmpty() ) {
    slotSaveAsProject();
  }else{
    saveProject();
  }
}
/** saves the project without showing the filedialog */
void Kavi2svcd::saveProject(){
  QFile f(myProjectFile);
  if (	f.open(IO_WriteOnly) ) {
    QTextStream ts( &f );
    ts << "====Transcode Data====\n";
    ts << "myNiceTranscode:" << myNiceTranscode << "\n";
    ts << "myFramesAll:" << myFramesAll << "\n";
    ts << "myFrameFirst:" << myFrameFirst << "\n";
    ts << "myFrameLast:" << myFrameLast << "\n";
    ts << "ASR.keepASR" << ASR.keepASR << "\n";
    ts << "ASR.importRatio:" << ASR.importRatio << "\n";
    ts << "ASR.exportRatio:" << ASR.exportRatio << "\n";
    ts << "myDirMode:" << myDirMode  << "\n";
    ts << "myDirModeType:" << myDirModeType << "\n";
    ts << "myBasename:" << myBasename << "\n";
    ts << "myDirModeSplit:" << myDirModeSplit << "\n";
    ts << "myDirModeFile:" << myDirModeFile << "\n";
    ts << "LineEditInputFile:" << LineEditInputFile->text() << "\n";
    ts << "CheckBoxV:" << CheckBoxV->isChecked() << "\n";
    ts << "CheckBoxZ:" << CheckBoxZ->isChecked() << "\n";
    ts << "myInterlacing:" << myInterlacing << "\n";
    ts << "ComboBoxImportVideo:" << ComboBoxImportVideo->currentItem() << "\n";
    ts << "mySubData.bUseSub:" << mySubData.bUseSub << "\n";
    ts << "mySubData.bTranscodeMode:" << mySubData.bTranscodeMode << "\n";
    ts << "mySubData.subFile:" << mySubData.subFile << "\n";
    ts << "mySubtitleCommand:" << mySubtitleCommand << "\n";
    ts << "ComboBoxImportAudio:" << ComboBoxImportAudio->currentItem() << "\n";
    ts <<	"CheckBoxOnlyAudio:" << CheckBoxOnlyAudio->isChecked() << "\n";
    ts << "SpinBoxAudioChannel:" << SpinBoxAudioChannel->text() << "\n";
    ts << "ComboBoxTranscodeTool:" << ComboBoxTranscodeTool->currentItem() << "\n";
    ts << "ComboBoxModulOutput:" << ComboBoxModulOutput->currentItem() << "\n";
    ts <<	"ComboBoxCodec:" << ComboBoxCodec->currentItem() << "\n";
    ts << "LineEditSequenceLength:" << LineEditSequenceLength->text() << "\n";
    ts << "myUseProfile:" << myUseProfile << "\n";
    ts << "CheckBoxSMP:" << CheckBoxSMP->isChecked() << "\n";
    ts << "LineEditVideoBitrate:" << LineEditVideoBitrate->text() << "\n";
    ts << "LineEditBitrate:" << LineEditBitrate->text() << "\n";
    ts << "myFast:" << myFast << "\n";
    ts << "LineEditResolution:" << LineEditResolution->text() << "\n";
    ts << "LineEditBorder:" << LineEditBorder->text() << "\n";
    ts << "CheckBoxK:" << CheckBoxK->isChecked() << "\n";
    ts <<	"LineEditGamma:" << LineEditGamma->text() << "\n";
    ts << "LineEditOutputFile:" << LineEditOutputFile->text() << "\n";
    ts << "RadioButtonSVCD:" << RadioButtonSVCD->isChecked() << "\n";
    ts << "RadioButtonVCD:" << RadioButtonVCD->isChecked() << "\n";
    ts << "RadioButtonDVD:" << RadioButtonDVD->isChecked() << "\n";
    ts << "RadioButtonFrei:" << RadioButtonFrei->isChecked() << "\n";
    // Mplex
    ts << "====Mplex====\n";
    ts << "RadioButtonMplex:" << RadioButtonMplex->isChecked() << "\n";
    ts << "LineEditMplexVideo:" << LineEditMplexVideo->text() << "\n";
    ts << "myNiceMplex:" << myNiceMplex << "\n";
    ts << "myOptionMplex:" << myOptionMplex << "\n";
    ts << "ComboBoxMplexCodec:" << ComboBoxMplexCodec->currentItem() << "\n";
    ts << "CheckBoxSegment:" << CheckBoxSegment->isChecked() << "\n";
    ts << "LineEditMplexSize:" << LineEditMplexSize->text() << "\n";
    ts << "SpinBoxSync:" << SpinBoxSync->text() << "\n";
    ts << "LineEditMplexOutput:" << LineEditMplexOutput->text() << "\n";
    ts << "LineEditMplexAudio1:" << LineEditMplexAudio1->text() << "\n";
    ts << "LineEditMplexAudio2:" << LineEditMplexAudio2->text() << "\n";
    ts << "CheckBoxVCDauto:" << CheckBoxVCDauto->isChecked() << "\n";
    ts << "LineEditPrefix:" << LineEditPrefix->text() << "\n";
    ts << "CheckBoxCDRauto:"   << CheckBoxCDRauto->isChecked() << "\n";
    // vcdimager
    ts << "====vcdimager====\n";
    ts << "myNiceVCDImager:" << myNiceVCDImager << "\n";
    ts << "myOptionVCD:" << myOptionVCD << "\n";
    ts << "LineEditBin:" << LineEditBin->text() << "\n";
    ts << "LineEditCue:" << LineEditCue->text() << "\n";
    ts << "checkBoxVCDScanOffsets:" << checkBoxVCDScanOffsets->isChecked() << "\n";
    ts << "LineEditVCDKommando:" << LineEditVCDKommando->text() << "\n";
    ts << "SpinBoxVCount:" << SpinBoxVCount->text() << "\n";
    ts << "SpinBoxVNumber:" << SpinBoxVNumber->text() << "\n";
    ts << "checkBoxVCDBrokenSVCD:" << checkBoxVCDBrokenSVCD->isChecked() << "\n";
    ts << "ComboBoxVCDIType:" << ComboBoxVCDIType->currentText() << "\n";
    ts << "checkBoxVCD2336:" << checkBoxVCD2336->isChecked() << "\n";
    ts << "LineEditVCDKommando:" << LineEditVCDKommando->text() << "\n";
    for ( uint i=0; i< ListBoxMpegFiles->count(); i++) {
      ts << "ListBoxMpegFiles:" << ListBoxMpegFiles->text(i) << "\n";
    }
    // cdrdao
    ts << "====cdrdao====\n";
    ts << "myNiceCdrdao:" << myNiceCdrdao << "\n";
    ts << "myOptionCDR:" << myOptionCDR << "\n";
    ts << "LineEditCDRDevice:" << LineEditCDRDevice->text() << "\n";
    ts << "ComboBoxCDRDriver:" << ComboBoxCDRDriver->currentText() << "\n";
    ts << "SpinBoxCDRSpeed:" << SpinBoxCDRSpeed->text() << "\n";
    ts << "CheckBoxCDREject:" << CheckBoxCDREject->isChecked() << "\n";
    ts << "CheckBoxCDRSimulate:" << CheckBoxCDRSimulate->isChecked() << "\n";
    ts << "CheckBoxBlankCDRW:" << CheckBoxBlankCDRW->isChecked() << "\n";
    ts << "LineEditCDRFile:" << LineEditCDRFile->text() << "\n";
    ts << "LineEditCDRKommando:" << LineEditCDRFile->text() << "\n";
    ts << "myCDRType:" << myCDRType << "\n";
    // automate
    ts << "====automateData====\n";
    ts << "myAutomateData.runIt:" << myAutomateData.runIt << "\n";
    ts << "myAutomateData.transcode:" << myAutomateData.transcode << "\n";
    ts << "myAutomateData.mplex:" << myAutomateData.mplex << "\n";
    ts << "myAutomateData.vcd:" << myAutomateData.vcd << "\n";
    ts << "myAutomateData.cdr:" << myAutomateData.cdr << "\n";
    ts << "myAutomateData.mplexDelete:" << myAutomateData.mplexDelete << "\n";
    ts << "myAutomateData.vcdDelete:" << myAutomateData.vcdDelete << "\n";
    ts << "myAutomateData.cdrDelete:" << myAutomateData.cdrDelete << "\n";
    ts << "myAutomateData.filename:" << myAutomateData.filename << "\n";
    // controlling
    ts << "====controlling====\n";
    ts << "wasCanceled:" << myAutomationCanceled << "\n";
    // Where was the process canceled or till where did it run successfully
    ts << "myProjectProgress.transcodeOK:" << myProjectProgress.transcodeOK << "\n";
    ts << "myProjectProgress.mplexOK:" << myProjectProgress.mplexOK << "\n";
    ts << "myProjectProgress.vcdimagerOK:" << myProjectProgress.vcdimagerOK << "\n";
    ts << "myProjectProgress.cdrOK:" << myProjectProgress.cdrOK << "\n";
    ts << "myProjectProgress.vcdimagerNum:" << myProjectProgress.vcdimagerNum << "\n";
    ts << "myProjectProgress.cdrNum:" << myProjectProgress.cdrNum << "\n";
    ts << "myProjectProgress.ready:" << myProjectProgress.ready << "\n";
    ts << "projectDescription:" << "\n"; // TODO
    f.close();
    this->statusBar()->message(i18n("The data were saved as project %1").arg(myProjectFile),3000);
  } else {
    QMessageBox* error = new QMessageBox();
    QString errorStr=i18n("\nError writing file\n");
    errorStr.append( myProjectFile );
    error->setText( errorStr );
    error->exec();
    delete error;
  }
}
/** loads a projectFile of a given name */
void Kavi2svcd::loadProject(){
  QFile f(myProjectFile);
  if (	f.open(IO_ReadOnly) ) {
    QStringList sListe;
    ListBoxMpegFiles->clear();
    QTextStream ts( &f );
    QString test = ts.readLine();
    while ( test != NULL ) {
      int pos = test.find(":");
      QString option = test.left(pos);
      QString value = test.right( test.length()-pos-1);
      // transcode
      if ( option == "myNiceTranscode" ) { myNiceTranscode=value.toInt(); }
      else if (option == "myFramesAll" ) { myFramesAll=value.toLong(); }
      else if (option == "myFrameFirst" ) {  myFrameFirst = value.toLong(); }
      else if (option == "myFrameLast" ) { myFrameLast=value.toLong(); }
      else if (option == "ASR.keepASR" ) { ASR.keepASR=value.toInt(); }
      else if (option == "ASR.importRatio" ) { ASR.importRatio=value.toInt(); }
      else if (option == "ASR.exportRatio" ) { ASR.exportRatio=value.toInt(); }
      else if (option == "myDirMode" ) { myDirMode=value.toInt(); }
      else if (option == "myDirModeType" ) { myDirModeType=value.toInt(); }
      else if (option == "myBasename:" ) { myBasename=value; }
      else if (option == "myDirModeSplit" ) { myDirModeSplit=value.toInt(); }
      else if (option == "myDirModeFile" ) { myDirModeFile=value; }
      else if (option == "LineEditInputFile" ) { LineEditInputFile->setText(value);
      // Put the filename into myModeDialog
      myModeDialog->lineEditAviFile->setText(value); }
      else if (option == "CheckBoxV" ) { CheckBoxV->setChecked(value.toInt()); }
      else if (option == "CheckBoxZ" ) { CheckBoxZ->setChecked(value.toInt()); }
      else if (option == "myInterlacing" ) { myInterlacing=value.toInt(); }
      else if (option == "ComboBoxImportVideo" ) { ComboBoxImportVideo->setCurrentItem(value.toInt()); }
      else if (option == "mySubData.bUseSub" ) { mySubData.bUseSub=value.toInt(); }
      else if (option == "mySubData.bTranscodeMode" ) { mySubData.bTranscodeMode=value.toInt(); }
      else if (option == "mySubData.subFile" ) { mySubData.subFile=value; }
      else if (option == "mySubtitleCommand" ) { mySubtitleCommand=value; }
      else if (option == "ComboBoxImportAudio" ) { ComboBoxImportAudio->setCurrentItem(value.toInt()); }
      else if (option == "CheckBoxOnlyAudio" ) { CheckBoxOnlyAudio->setChecked(value.toInt()); }
      else if (option == "SpinBoxAudioChannel" ) { SpinBoxAudioChannel->setValue(value.toInt()); }
      else if (option == "ComboBoxTranscodeTool" ) { ComboBoxTranscodeTool->setCurrentItem(value.toInt()); }
      else if (option == "ComboBoxModulOutput" ) { ComboBoxModulOutput->setCurrentItem(value.toInt()); }
      else if (option == "ComboBoxCodec" ) { ComboBoxCodec->setCurrentItem(value.toInt()); }
      else if (option == "LineEditSequenceLength" ) { LineEditSequenceLength->setText(value); }
      else if (option == "myUseProfile" ) { myUseProfile=value.toInt(); }
      else if (option == "CheckBoxSMP" ) { CheckBoxSMP->setChecked(value.toInt()); }
      else if (option == "LineEditVideoBitrate" ) { LineEditVideoBitrate->setText(value); }
      else if (option == "LineEditBitrate" ) { LineEditBitrate->setText(value); }
      else if (option == "myFast" ) { myFast=value.toInt(); }
      else if (option == "LineEditResolution" ) { LineEditResolution->setText(value); }
      else if (option == "LineEditBorder" ) { LineEditBorder->setText(value); }
      else if (option == "CheckBoxK" ) { CheckBoxK->setChecked(value.toInt()); }
      else if (option == "LineEditGamma" ) { LineEditGamma->setText(value); }
      else if (option == "RadioButtonVCD" ) { RadioButtonVCD->setChecked(value.toInt()); }
      else if (option == "RadioButtonSVCD" ) { RadioButtonSVCD->setChecked(value.toInt()); }
      else if (option == "RadioButtonDVD" ) { RadioButtonDVD->setChecked(value.toInt()); }
      else if (option == "RadioButtonFrei" ) { RadioButtonFrei->setChecked(value.toInt()); }
      else if (option == "LineEditOutputFile" ) { LineEditOutputFile->setText(value); }
      // Mplex
      else if (option == "RadioButtonMplex" ) { RadioButtonMplex->setChecked(value.toInt()); }
      else if (option == "LineEditMplexVideo" ) { LineEditMplexVideo->setText(value); }
      else if (option == "myNiceMplex" ) { myNiceMplex=value.toInt(); }
      else if (option == "myOptionMplex" ) { myOptionMplex=value; }
      else if (option == "ComboBoxMplexCodec" ) { ComboBoxMplexCodec->setCurrentItem(value.toInt()); }
      else if (option == "CheckBoxSegment" ) { CheckBoxSegment->setChecked(value.toInt()); }
      else if (option == "LineEditMplexSize" ) { LineEditMplexSize->setText(value); }
      else if (option == "SpinBoxSync" ) { SpinBoxSync->setValue(value.toInt()); }
      else if (option == "LineEditMplexOutput" ) { LineEditMplexOutput->setText(value); }
      else if (option == "LineEditMplexAudio1" ) { LineEditMplexAudio1->setText(value); }
      else if (option == "LineEditMplexAudio2" ) { LineEditMplexAudio2->setText(value); }
      else if (option == "CheckBoxVCDauto" ) { CheckBoxVCDauto->setChecked(value.toInt()); }
      else if (option == "LineEditPrefix" ) { LineEditPrefix->setText(value); }
      else if (option == "CheckBoxCDRauto") { CheckBoxCDRauto->setChecked(value.toInt()); }
      // vcdimager
      else if (option =="myNiceVCDImager") { myNiceVCDImager = value.toInt(); }
      else if (option =="myOptionVCD") { myOptionVCD = value; }
      else if (option =="LineEditBin") { LineEditBin->setText(value); }
      else if (option =="LineEditCue") { LineEditCue->setText(value); }
      else if (option =="checkBoxVCDScanOffsets") { checkBoxVCDScanOffsets->setChecked(value.toInt()); }
      else if (option =="LineEditVCDKommando") { LineEditVCDKommando->setText(value); }
      else if (option =="SpinBoxVCount") { SpinBoxVCount->setValue(value.toInt()); }
      else if (option =="SpinBoxVNumber") { SpinBoxVNumber->setValue(value.toInt()); }
      else if (option =="checkBoxVCDBrokenSVCD") { checkBoxVCDBrokenSVCD->setChecked(value.toInt()); }
      else if (option =="ComboBoxVCDIType") { ComboBoxVCDIType->setCurrentText(value); }
      else if (option =="checkBoxVCD2336") { checkBoxVCD2336->setChecked(value.toInt()); }
      else if (option =="LineEditVCDKommando") { LineEditVCDKommando->setText(value); }
      else if (option =="ListBoxMpegFiles") { sListe += value; }			// this may occur more than once!
      // cdrdao
      else if (option == "myNiceCdrdao") { myNiceCdrdao =value.toInt(); }
      else if (option == "myOptionCDR") { myOptionCDR =value; }
      else if (option == "LineEditCDRDevice") { LineEditCDRDevice->setText(value); }
      else if (option == "ComboBoxCDRDriver") { ComboBoxCDRDriver->setCurrentText(value); }
      else if (option == "SpinBoxCDRSpeed") { SpinBoxCDRSpeed->setValue(value.toInt()); }
      else if (option == "CheckBoxCDREject") { CheckBoxCDREject->setChecked(value.toInt()); }
      else if (option == "CheckBoxCDRSimulate") { CheckBoxCDRSimulate->setChecked(value.toInt()); }
      else if (option == "CheckBoxBlankCDRW") { CheckBoxBlankCDRW->setChecked(value.toInt()); }
      else if (option == "LineEditCDRFile") { LineEditCDRFile->setText(value); }
      else if (option == "LineEditCDRKommando") { LineEditCDRFile->setText(value); }
      else if (option == "myCDRType") { myCDRType=value.toInt(); }
      // automate
      else if (option == "myAutomateData.runIt") { myAutomateData.runIt =value.toInt(); }
      else if (option == "myAutomateData.transcode") { myAutomateData.transcode =value.toInt(); }
      else if (option == "myAutomateData.mplex") { myAutomateData.mplex =value.toInt(); }
      else if (option == "myAutomateData.vcd") { myAutomateData.vcd =value.toInt(); }
      else if (option == "myAutomateData.cdr") { myAutomateData.cdr =value.toInt(); }
      else if (option == "myAutomateData.mplexDelete") { myAutomateData.mplexDelete =value.toInt(); }
      else if (option == "myAutomateData.vcdDelete") { myAutomateData.vcdDelete =value.toInt(); }
      else if (option == "myAutomateData.cdrDelete") { myAutomateData.cdrDelete =value.toInt(); }
      else if (option == "myAutomateData.filename") { myAutomateData.filename =value; }
      // controlling
      else if (option == "wasCanceled") { myAutomationCanceled=value.toInt(); }
      else if (option == "myProjectProgress.transcodeOK") {  myProjectProgress.transcodeOK=value.toInt(); }
      else if (option == "myProjectProgress.mplexOK") {  myProjectProgress.mplexOK=value.toInt(); }
      else if (option == "myProjectProgress.vcdimagerOK") {  myProjectProgress.vcdimagerOK=value.toInt(); }
      else if (option == "myProjectProgress.cdrOK") {  myProjectProgress.cdrOK=value.toInt(); }
      else if (option == "myProjectProgress.vcdimagerNum") {  myProjectProgress.vcdimagerNum=value.toInt(); }
      else if (option == "myProjectProgress.cdrNum") {  myProjectProgress.cdrNum=value.toInt(); }
      else if (option == "myProjectProgress.ready") {  myProjectProgress.ready=value.toInt(); }
      else if (option == "projectDescription") { } // TODO
      // the last line
      else if (option == "this is the end") {  };
      test = ts.readLine();
    } // end of while loop
    // clear the list for vcdimager
    ListBoxMpegFiles->clear();
    // insert the list for vcdimager
    ListBoxMpegFiles->insertStringList(sListe);
    if ( myModeType != EXPERT ) {
      myModeDialog->setInputFile(LineEditInputFile->text());
      slotUpdateBasicModeImage();
      slotUpdateBasicModeRef();
    }
    // generates the command line  myProjectProgress.ready=true;s
    slotGetInfo();
    slotGenerateTranscode();
    slotGenerateMplex();
    slotGenerateVCD();
    slotGenerateCDR();
    this->statusBar()->message(i18n("The project %1 was loaded.").arg(myProjectFile),3000);
  } else {
    QMessageBox* error = new QMessageBox();
    QString errorStr=i18n("\nError reading file\n");
    errorStr.append( myProjectFile );
    error->setText( errorStr );
    error->exec();
    delete error;
  }
}
/** updates the caption that contains the project name */
void Kavi2svcd::updateCaption(){
  QString name=myProjectFile;
  if ( myProjectFile.isEmpty() ) {
    name=i18n("<no project>");
  }
  this->setCaption("KAvi2svcd "+myVersion+" "+name+" ["+myConfigProfile+"]");
}
/** Clears the project data */
void Kavi2svcd::slotClearProject(){
  myProjectFile="";
  myProjectProgress.transcodeOK=false;
  myProjectProgress.mplexOK=false;
  myProjectProgress.vcdimagerOK=false;
  myProjectProgress.cdrOK=false;
  myProjectProgress.vcdimagerNum=0;
  myProjectProgress.cdrNum=0;
  myProjectProgress.ready=true;
  updateCaption();
}
/** Views the batchDlg */
void Kavi2svcd::slotShowBatchTranscode(){
  BatchClass* batch=new BatchClass(this);
  batch->setData(0, myInputFileList);
  if (batch->exec() ) {
    myInputFileList=batch->getData(0);
  }
  delete batch;
}
/** Views the batchDlg */
void Kavi2svcd::slotShowBatchMplex(){
  BatchClass* batch=new BatchClass(this);
  batch->setData(0,myMplexFileList[0]);
  batch->setData(1,myMplexFileList[1]);
  batch->setData(2,myMplexFileList[2]);
  if (batch->exec() ) {
    myMplexFileList[0]=batch->getData(0);
    myMplexFileList[1]=batch->getData(1);
    myMplexFileList[2]=batch->getData(2);
  }
  delete batch;
}





/*!
    \fn Kavi2svcd::whtasThis()
 */
void Kavi2svcd::whatsThis()
{
    QMainWindow::whatsThis();
}



