/***************************************************************************
                          kavi2svcd.h  -  description
                             -------------------
    begin                : Die Jul 23 11:52:25 CEST 2002
    copyright            : (C) 2002 by Cornelius K�bel
    email                : cornelius.koelbel@gmx.de
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KAVI2SVCD_H
#define KAVI2SVCD_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdlib.h>
#include <kaboutdata.h>
#include <kaboutapplication.h>
#include <klocale.h>
#include <kapp.h>
#include <klocale.h>

#include <qwidget.h>
#include <qprocess.h>

#include "kavi2svcdDlg.h"
#include "generatescriptclass.h"
#include "getinfoclass.h"
#include "automateclass.h"
#include "subtitlerclass.h"
#include "aspectratioclass.h"
#include "fpsclass.h"
#include "svcdmodeclass.h"
#include "batchclass.h"

/** Kavi2svcd is the base class of the project */
class Kavi2svcd : public kavi2svcdDlg
{
  Q_OBJECT 
    public:
  /** construtor */
  Kavi2svcd(int debug=0, QWidget* parent=0, const char *name=0);
  /** destructor */
  ~Kavi2svcd();
  enum modeType { ASK, EXPERT, SVCD, VCD, DVD };  
  
  struct prefData {
    QString editDir, size, device, previewCommand, previewCUE;
    QString transcodeOption, mplexOption, vcdOption, cdrOption;
    QString segment, sBlankCDRW, logFile, programsNotCheck;
    QString subtitleViewProgram, subtitleEditProgram;
    QStringList driverList;
    QString outputTranscode, outputMplex, outputVCD;
    int standardVcdType, speed, driver, videoFormat;
    bool eject, overburn, K,V,Z,fastZ, bBlankCDRW, useCDRDAOnotCDRECORD;
    bool niceTranscode, niceMplex, niceVcd, niceCdr;
    int niceValueMplex, niceValueTranscode, niceValueVcd, niceValueCdr, startup;
    int encoding;
    QString profile;
    QString TranscodeTool;
    QString ImportModule;
    QString ExportFPS;
    bool KeepASR;
    int ImportASR;
    int ExportASR;
    int DVDBitrate;
  };
  struct projectProgress {
    bool transcodeOK, mplexOK, vcdimagerOK, cdrOK, ready;
    int  vcdimagerNum, cdrNum;
  };
  /** Setzt die Variablen myVideoPfad myStandardType myStandardSize
      myScsiDevice myBurnSpeed myEject */
  void setPreferences(prefData data);
  /** Berechnet den String -j -X -B, fr das Fast Resizing.
      Die Werte mssen durch 32 teilbar sein. */
  QString calculateFastResize(int zielX, int zielY, int x, int y);
  /** Wird aufgerufen, wenn das Widget seinen Focus verliert. */
  virtual void focusOutEvent(QFocusEvent *);
  /** Instanz des Scripts */
  GenerateScriptClass* myScript;
  /** Gibt an, ob das Programm im Debugmodus l�ft */
  bool debug;

 signals: // Signals
  public slots: // Public slots
    /** Dieser Slot wird angesprungen von dem Help-About button.
	In dieser Memberfunktion wird dann der neue Dialog erzeugt. */
    void helpAbout();
  /** Dieser Slot wird aufgerufen, wenn ein Radiobutton ge�dert wird (VCD oder SVCD). Danach werden die Combobox und die beiden Textfelder befllt. */
  void slotVCD();
  /** No descriptions */
  void slotFreiEingabe();
  /** Hier wird die QFileDialog aufgerufen,
      in dem das input-Avi-File ausgew�lt wird. */
  void slotOpenInputFile();
  /** plays the cue-file */
  void slotViewCue();
  /** No descriptions */
  void slotGetInfo();
  /** Fills the mplex-Tab, when the output file of transcode was changed */
  void slotFillMplexTab();
  /** Erzeugt die Kommandozeile von transcode */
  void slotGenerateTranscode();
  /** �fnet den Dialog fr die Outputdatei */
  void slotOpenOutputFile();
  /** Audiofile 2 mpa */
  void slotOpenAudioFile2();
  /** Audiofile mpa */
  void slotOpenAudioFile1();
  /** �fnet Dialog, in dem die Datei bestimmt wird, die die Videodaten fr mplex enth�t: *.m1v oder *.m2v */
  void slotOpenVideoFile();
  /** �fnet den DateiDialog, in dem der Output von Plex bestimmt wird. */
  void slotOpenMplexOutputFile();
  /** Setzt den neuen maximalen Wert von Volume-Number entsprechend Volume Count. */
  void slotVolumeMax(int max);
  /** Ruft den Dialog auf, der den Transcode-Prozess
      enth�t und setzt das Kommando. */
  void slotStartTranscode();
  /** Erzeugt den Mplex Befehl */
  void slotGenerateMplex();
  /** Hier wird der Dialog aufgerufen,
      der den MPlex Prozess enth�t. */
  void slotStartMplex();
  /** Ruft den Dialog auf, 
      der die Preferences anzeigt und
      in dem man sie �dern kann. */
  void slotPreferences();
  /** Deaktiviert das CDR-Tab... */
  void slotCDRauto();
  /** Generiert in GenerateScriptClass (myScript) das Script.
      Zuvor mssen die Werte gesetzt werden. */
  void slotGenerateScript();
  /** Wird aufgerufen, wenn ein Script geladen wurde. */
  void slotScriptLoaded(QString, QString, QString, QString, QString, QString, QString, QString, QString, QString);
  /** Wird aufgerufen, wenn die Checkbox
      FastResize Options angew�lt wurde. */
  void slotFastResize();
  /** Setzt die Daten fr das Hilfemen */
  void setAboutData(KAboutData* data);
  /** Zeigt den Gamma wert vom Slider */
  void slotGamma(int v);
  /** Ruft die Hilfe der Kapplication auf. */
  void helpContents();
  /** Wenn der Wert des schwarzen Rand ge�dert wurde. */
  void borderChanged(int value);
  /** Wird aufgerufen, wenn die Mplex Option -S angew�lt wird. */
  void slotSegmentClicked();
  /** Wenn die cue Datei eingegeben wurde, wird die bin datei berechnet. */
  void slotVCDBinAusCue();
  /** The contents of the profile file */
  void setProfile( QStringList profile);
  /** if a profile file xsvcd shallbe used. */
  void setUseProfile(bool bProfile);
  /** change ASR */
  void slotChangeASR();
  /** Starts the dialog for range of frames */
  void slotRangeOfFrames();
  /** Sets the range of the frames to be encoded */
  void slotSetFrameRange( bool all, long start, long end);
  /** sets wether PAL=1 or NTSC=0 */
  void setVideoFormat(int i);
  /** Starts the dialog directory mode. */
  void slotDirectoryMode();
  /** Gets the border from the getInfo class. */
  void slotGetInfoBorder(QString s);
  /** Starts the subtitler dialog */
  void slotSubtitler();
  /** Displays the info of the input file */
  void slotShowFileInfo();
  /** Here you can set the import and the export aspect ratio */
  void slotASR();
  void slotFPS();
  /** switches from svcd or vcd mode to expert mode */
  void slotSwitchExpert();
  /** switches from Expert to svcd mode */
  void slotSwitchSVCDMode();
  /** updates the new image to be shown in the basic mode dialog */
  void slotUpdateBasicModeImage();
  /** Views the batchDlg */
  void slotShowBatchTranscode();
  /** Views the batchDlg */
  void slotShowBatchMplex();
  
 private: // Private attributes
  /** preview process for cue-files */
  QProcess* myPreviewProc;
  /** Breite des Inputfiles */
  int myWidthX;
  /** H�e der Inputdatei */
  int myHeightY;
  /** Prferences: Type svcd /vcd */
  int myStandardType;
  /** Anzahl der Frames im Film */
  long myFrames;
  /** Pfad, in dem immer die Dialoge ge�fnet werden */
  QString myVideoPfad;
  /** Preferences: -S DiskSize */
  QString myStandardSize;
  /** Segmentsize */
  QString mySegmentSize;
  /** Preferences Speed */
  int myBurnSpeed;
  /** Ausgabe beim Fastresizing */
  QString myFast;
  /** Preferences: eject after burn */
  bool myEject;
  /** Daten fr den HilfeDialog */
  KAboutData* myAboutData;
  /** K Application. */
  KApplication* myKApp;
  /** Preferences:  */
  QString myScsiDevice;
  /** Standard fr die K-Option */
  bool myK;
  /** Standard fr die V-Option */
  bool myV;
  /** Standard fr Z-Option */
  bool myZ;
  /** Berechneter Wert des schwarzen Randes */
  QString myBorder;
  /** Der Preview Player */
  QString myPreview;
  /** list of the import video modules of transcode. */
  QStringList myImportVideo;
  /** List of the import audio modules of transcode. */
  QStringList myImportAudio;
  /** Standardoptionen fr Kommandozeile */
  QString myOptionTranscode;
  /** Standardoptionen fr Kommandozeile */
  QString myOptionMplex;
  /** Standardoptionen fr Kommandozeile */
  QString myOptionVCD;
  /** Standardoptionen fr Kommandozeile */
  QString myOptionCDR;
  /** Liste der Driver fr cdrdao */
  QStringList myDriverList;
  /** Standard Treiber fr cdrdao */
  int myDriverStandard;
  /** Transcode Eingabedatei Pfad */
  QString myDirTranscodeIn;
  /**  */
  QString myDirTranscodeOut;
  /**  */
  QString myDirMplexAudio;
  /** Enth�t den Pfad zu den Libs von transcode */
  QString myTranscodePath;
  /**  */
  QString myDirMplexVideo;
  /**  */
  QString myDirMplexOut;
  /**  */
  QString myDirVCDIn;
  /**  */
  QString myDirVCDOut;
  /**  */
  QString myDirCDRIn;
  QString myConfigProfile;
  /** List of the xsvcd.prof values */
  QStringList myProfileList;
  /** Audio export module video/audio*/
  QStringList myExportVideo;
  QStringList myExportAudio;
  /** Frames per second des input files */
  float myFps;
  /** if a xsvcd.prof file should be used */
  bool myUseProfile;
  /** whether the cdrw should be erased before burning the image */
  bool myBlankCDRW;
  /** Command that is executed to blank the cdrw */
  QString myBlankCDRWCmd;
  /** Wether --keep_asr shall be used */
  bool myKeepAspectRatio;
  /** If true, all frames shall be encoded.  */
  bool myFramesAll;
  /** First Frame, that shall be encoded */
  long myFrameFirst;
  /** last frame, that shall be encoded */
  long myFrameLast;
  /** if an interlace mode (-I) shall be used */
  int myInterlacing;
  int myEncode;
  /** Nice level for transcode */
  int myNiceTranscode;
  /** Nice level for mplex */
  int myNiceMplex;
  /** Nice level for vcdimager */
  int myNiceVCDImager;
  /** Nice level for cdrdao */
  /** which type of cd burning tool shall be used
      1=cdrdao , 2=cdrecord */
  int myCDRType;
  int myNiceCdrdao;
  /** logfile */
  QString myLogFile;
  /** PAL=1, NTSC=0 */
  int myVideoFormat;
  /**  */
  QString myAudioBitrateSVCD;
  QString myAudioBitrateVCD;
  QString myResolutionSVCD;
  QString myResolutionVCD;
  /** stores the output of stderr from tcprobe */
  QString myGetInfoErr;
  /** wether directory mode shall be used. */
  bool myDirMode;
  /** The base for directory mode */
  QString myBasename;
  /** This is the base filename with the complete path of all the inputfiles in directory mode */
  QString myDirModeFile;
  /** The Class for getting the info of the input file */
  GetInfoClass* myGetInfo;
  /** List of all Programs that shall NOT be checked at startup */
  QString myCheckPrograms;
  /** wether in Option Dir Mode the option --no_split shall be set. */
  bool myDirModeSplit;
  /** The Type of Dirmode: IMPORT or OPTION */
  int myDirModeType;
  /** Object, that holds the Dialog for choosing automation */
  AutomateClass* myAutomate;
  /** That data, that is used to automate the generation of the VCD */
  AutomateClass::automateData myAutomateData;
  /** Commandline for Viewer and Editor of subtitle file */
  QString mySubView, mySubEdit;
  /** part of the transcode command for subtitles */
  QString mySubtitleCommand;
  /** Preview command for cue-files */
  QString myPreviewCUE;
  /** Data for the subtitler dialog */
  SubtitlerClass::SubtitleData mySubData;
  /** The complete info of the input file */
  QString myInputFileInfo;
  /** Data of the aspectRatio */
  AspectRatioClass::aspectData ASR;
  FPSClass::fpsData FPS;
  /** standard output path */
  QString myOutputTranscode, myOutputMplex, myOutputVCD;
  /** Stores the type of the mode: expert, svcd, vcd (future: dvd?) */
  modeType myModeType;
  /** Modeless dialog for svcd mode */
  SvcdModeClass* myModeDialog;
  /** This is true, if the automation process was canceled at some point. */
  bool myAutomationCanceled;
  /** Name of the ProjectFile */
  QString myProjectFile;
  /** In this structure the progress of the project is stored.
      This is used, if a project is canceled and shall be resumed lateron. */
  projectProgress myProjectProgress;
  /** Version of kavi2svcd */
  QString myVersion;
  /** This is a list of chosen input files for batch mode */
  QStringList myInputFileList;
  /** Inputfiles for mplex: 0=video, 1=audio1, 2=audio2 */
  QStringList myMplexFileList[3];
  bool myOverburn;
    int myDVDBitrate;
  private slots: // Private slots
    /** �fnet Dialog auf vcdimager */
    void slotOpenVCDMpeg();
  /** W�lt eine cue Datei aus
      fr vcdimager */
  void slotOpenVCDcue();
  /** W�lt eine bin-Datei aus
      fr vcdimager */
  void slotOpenVCDbin();
  /** Fgt das File zur Liste */
  void slotVCDadd();
  /** L�cht den aktuellen Eintrag */
  void slotVCDremove();
  /** Schiebt den currentItem() einen nach unten. */
  void slotVCDdown();
  /** Schiebt den currentItem() einen nach oben. */
  void slotVCDup();
  /** Erzeugt die Befehlszeile fr VCDimager */
  void slotGenerateVCD();
  /** Wenn Checkbox bei mplex auf Auto
      gesetzt wird, braucht man den Tab
      vcdimager nicht mehr. */
  void slotVCDauto();
  /** Wenn Die Zeile Outputfile bei mplex ge�ndert wird, wird gleich das
      Inputfile bei vcdimager gefllt. */
  void slotMplexOutVCDIn();
  /** Startet den brennprozess. */
  void slotStartCDR();
  /** �fnet das cuefile, das gebrannt werden soll. */
  void slotOpenCDRcue();
  /** Erzeugt die Kommandozeile fr den Brennprozess. */
  void slotGenerateCDR();
  /** Wird von dem Meneintrag Save aufgerufen. */
  void fileSave();
  /** slot, der dann den Load-Befehl in GenerateScriptClass aufruft. */
  void fileOpen();
  /** gets the FPS from the getinfoClass */
  void slotGetInfoFPS(QString f);
  /** gets the codec from the getinfoClass */
  void slotGetInfoCodec(QString c);
  /** gets the frames from the getifnoclass */
  void slotGetInfoFrames(QString f);
  /** gets the width from the getifnoclass */
  void slotGetInfoWidth(QString w);
  void slotGetInfoDuration(QString d);
  /** gets the height from the getifnoclass */
  void slotGetInfoHeight(QString h);
  /** Starts the Dialog for Automation */
  void slotAutomate();
  /** If the cue file in the mplex-Tab is changed, this is given to the CD Burning-Tab */
  void slotCueFileChanged();
  /** starts the vcd process when the button was pressed. */
  void slotStartVCD();
  /** Gets the whole file info of the input file */
  void slotGetInfoFileInfo(QString s);
  /** updates the reference image */
  void slotUpdateBasicModeRef();
 private: // Private methods
  /** Testen, ob das Conf-Verzeichnis existiert.
      Einelesen der Konfiguration */
  void initPreferences();
  /** Berechnet die Optionen fr fastresize. Von sojus */
  QString calculateFastResizeNEW(int zielX, int zielY, int x, int y);
  /** Erstelle die Liste der Import_Libs */
  void initLibList_neu();
  /** Initialisiert alle Strings, in denen die Pfade der Dateidialoge gespeichert werden. */
  void initOpenDirs();
  /** copys the profile from the homedir to tmp */
  void copyProfile();
  /** private member functions */
  bool StartCDR(bool autoClose=false, int num=0, int totalNum=0);
  /** Ruft den Prozess zum	vcdimager auf. */
  bool StartVCD(bool autoClose=false, int num=0, int totalNum=0);
  bool StartTranscode(bool autoClose=false);
  bool StartMplex(bool autoClose=false);
  /** Calculates the total size of all selected mpeg-files in vcd-tab */
  QString getVCDTotalSize();
  /** Adds a string to the mpeg list for vcdimager */
  void vcdAdd(QString s);
  /** saves the project without showing the filedialog */
  void saveProject();
  /** loads a projectFile of a given name */
  void loadProject();
  /** updates the caption that contains the project name */
  void updateCaption();
  protected slots: // Protected slots
    /** Berechnet aus den Angaben der Bitraten die Gr? der Datei. */
    void slotCalcVideoSize();
  /** Wird ausgefhrt, wenn die Checkbox only audio gedrckt ist. */
  void slotOnlyAudio();
  /** Wenn die ComboBox Tool ver�dert wurde. */
  void slotToolChanged();
  /** Startet den Profile Dialog fr bbmpeg */
  void slotProfile();
  /** Starts the dialog for interlcing  */
  void slotInterlace();
  /** Is executed, when the file infos Frame, Fps, width height in the lineedits are changed */
  void slotFileInfosChanged();
  /** This calculates the Video bitrate if an Filesize is given. */
  void slotCalcVideoBitrate();
  /** Gets executed, when the start button on SVCD/VCD Mode is pressed. */
  void slotStartSVCDMode();
  /** when the checkbox -k on SVCD/VCD mode gets toggeled */
  void slotKtoggled();
  /** Opens the AVi File load dialog for SVCD/VCD Mode */
  void slotOpenAviFile();
  /** Presets the VCD-Tab */
  void slotFillVCDTab();
  /** runs automated process */
  void slotRunAutomate();
  /** gets the toggled checkbox from myModeDialog */
  void slotSVCDToggled(bool b);
  /** gets the toggled checkbox from myModeDialog */
  void slotVCDToggled(bool b);
  void slotDVDToggled(bool b);
  /** Saves all necessary data to a project */
  void slotSaveProject();
  /** Loads the old data from a project */
  void slotLoadProject();
  /** Saves the data as a Porject */
  void slotSaveAsProject();
  /** Clears the project data */
  void slotClearProject();
    void whatsThis();
    void slotzToggled();
 protected: // Protected attributes
  /** �erladen der Groupbox */
  QGroupBox* GroupBoxOutFile;
 protected: // Protected methods
  /** Reagiert auf einen Maustastendruck */
  void mousePressEvent ( QMouseEvent * e ) ;
};

#endif
