/***************************************************************************
                          mplex.cpp  -  description
                             -------------------
    begin                : Don Jul 25 2002
    copyright            : (C) 2002 by Cornelius Klbel
    email                : cornelius.koelbel@gmx.de
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "mplex.h"
#include "qpushbutton.h"
#include <qtextedit.h>
#include <qregexp.h>
#include <qfileinfo.h>
#include <qprogressbar.h>
#include <qlabel.h>

Mplex::Mplex(QWidget *parent, const char *name ) : ProcBaseClass(parent,name) {
  // in der Basisklasse ist die Caption Transcoding!
  this->setCaption(i18n("Mplexing..."));
  myErrorString=i18n("Error starting mplexing process...");

  myTimer = new QTimer( this );
  connect( myTimer, SIGNAL(timeout()), this, SLOT(slotTimer()) );
  myTimer->start( 10000, false ); // 10 seconds timer

  connect( myProc, SIGNAL(processExited()), this, SLOT(slotTimerStop()) );
  ProgressBarTranscode->setTotalSteps(1000);

  TextLabelProgress->setText(i18n("Progress:"));
  TextLabelTime->setText(i18n("Running..."));
  TextLabelFrames->hide();
  TextLabelFrames2->hide();
}
Mplex::~Mplex(){
  delete myTimer;
}
/** Wird nach einem bestimmten Zeitintervall aufgerufen, damit der Fortschritt geprft werden kann */
void Mplex::slotTimer(){
  //TextEditOutput->append( "TIMER\n");
  int iProgress = progress(myVideo, myAudio1, myAudio2, myOutput);
  ProgressBarTranscode->setProgress(iProgress);
}
/** Wenn der Prozess zu Ende ist, wird der Timer angehalten. */
void Mplex::slotTimerStop(){
  myTimer->stop();
  ProgressBarTranscode->setProgress(1000);
}
/** Setzt die Ausgabedateien */
void Mplex::setOutput(QString o){
  myOutput = o;
}
/** Setzt die Videodatei */
void Mplex::setVideo(QString v){
  myVideo = v;
}
/** Setzt die Audiodateien */
void Mplex::setAudio(QString a1, QString a2){
  myAudio1 = a1;
  myAudio2 = a2;
}
/** Berechnet den Fortschritt beim mplexing. */
int Mplex::progress(QString myVideo, QString myAudio1, QString myAudio2, QString myOutput){
  // Input: myVideo, myAudio1, myAudio2, myOutput,
  // Output: progress
  // Hier mssen die Dateien angeschaut werden.
  // Total: (summe aus m2v und mpa's) * 1.05
  QFileInfo qfVideo = QFileInfo( myVideo );
  QFileInfo qfAudio1 = QFileInfo( myAudio1 );
  QFileInfo qfAudio2 = QFileInfo( myAudio2 );
  double totalSize=	(qfVideo.size() + qfAudio1.size() + qfAudio2.size())*1.05;
  // Fortschritt: summe aus allen mpeg's
  // myOutput
  myOutput=myOutput.replace(QRegExp("\%d"),"*");
  int posDatei =  myOutput.findRev("/");
  QString myPath = myOutput.left( posDatei );
  QString myFilter = myOutput.right( myOutput.length() - myPath.length() - 1 );
  // QDir erzeugen mit Pfad und Filter.
  QDir d = QDir(myPath, myFilter);
  const QFileInfoList *list = d.entryInfoList();
  QFileInfoListIterator it( *list );
  QFileInfo *fi = new QFileInfo();
  long summe=0;
  while ( (fi = it.current()) != 0 ) {
    summe= summe+fi->size();
    ++it;
  }
  // TODO evtl. wegen des Crashes von TODD / Angel
  delete fi;
  //delete list;
  // summe durch totalsize teilen und in der Progressbar darstellen
  int progress = (int)((summe/totalSize)*1000);
  return progress;
}
