/***************************************************************************
                          procbaseclass.cpp  -  description
                             -------------------
    begin                : Fre Jul 26 2002
    copyright            : (C) 2002 by Cornelius K�bel
    email                : cornelius.koelbel@gmx.de
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "procbaseclass.h"

#include <qprogressbar.h>
#include <qtextedit.h>
#include <qmessagebox.h>
#include <qlabel.h>
#include <qtimer.h>
#include <qdatetime.h>
#include <qpopupmenu.h>
#include <sys/types.h>
#include <signal.h>
#include <qsound.h>

ProcBaseClass::ProcBaseClass(QWidget *parent, const char *name ) : TranscodeDlg(parent,name) {
  myLogFile=QString("");
  myLog = new LogClass();
  myLog->setFileName(myLogFile);
	
  PushButtonPreview->hide();
  PushButtonCancel->hide();
  pixmapLabelOK->hide();
  pixmapLabelError->hide();
  myProc = new KProcess(  );
  connect( myProc, SIGNAL(receivedStdout (KProcess *, char *, int)),
	   this, SLOT(slotReadFromStdout(KProcess *, char *, int)) );
  connect( myProc, SIGNAL(receivedStderr(KProcess *, char *, int)),
	   this, SLOT(slotReadFromStderr(KProcess *, char *, int)) );
  connect( myProc, SIGNAL( processExited(KProcess *)),
	   this, SLOT(slotShowOK(KProcess *)) );
  myCanceled=false;

  TextEditOutput->hide();
}
ProcBaseClass::~ProcBaseClass(){
  delete myProc;
  delete myLog;
}
/** Shows the OK-Button
    This slot is called, when the process has exited.
    Maybe some sound?
*/
void ProcBaseClass::slotShowOK(KProcess *proc){
  PushButtonOK->show();
  PushButtonCancel->hide();
  TextLabelTime->setText(i18n("READY"));
  // Hier wird nun wieder die Uhrzeit genommen;
  myEndTime = QDateTime::currentDateTime();
  // Und die Differenz errechnet.
  int sekunden = myStartTime.secsTo( myEndTime );
  if ( sekunden < 0 ) { sekunden=-sekunden; }
  int stunden = sekunden/3600;
  int minuten = (sekunden%3600)/60;
  QString zeitDauer; zeitDauer.sprintf("%2i:%02i hh:mm", stunden, minuten);
  TextLabelProgress->setText(i18n("Time elapsed:"));
  TextLabelTime->setText( zeitDauer );
  ProgressBarTranscode->setProgress(ProgressBarTranscode->totalSteps());
  // If myAutoClose=true, the Dialog shall be closed imediatly.
  if (myAutoClose) {
    this->accept();
  } else {
    // If not myAutoClose, we will play a sound.
    QSound::play("/data/sound/los.wav");
  }
  if ( myCanceled ) {
    pixmapLabelError->show();
  }else{
    pixmapLabelOK->show();
  }
}
/** Reagiert auf den Startbutton */
void ProcBaseClass::slotStart(bool autoClose){
  myAutoClose=autoClose;
  PushButtonStart->hide();
  PushButtonOK->hide();
  PushButtonCancel->show();
  if (!myProc->args().empty())
    if ( !myProc->start(KProcess::NotifyOnExit, KProcess::All) ) {
      // error handling
      QMessageBox* myError = new QMessageBox();
      myError->setText(myErrorString);
      myError->exec();
      delete myError;
    }
  //--QString sPID; sPID.setNum( myProc->processIdentifier() );
  //--TextEditOutput->append( "\nPID: " + sPID );
  // Speichere die Startzeit!
  myStartTime = QDateTime::currentDateTime();
}
/** Beendet den Prozess und macht den OK-Button wieder sichtbar. */
void ProcBaseClass::slotCancel(){
  //QTimer::singleShot( 5000, this, SLOT( slotKill() ) );
  //myProc->tryTerminate();
  myProc->kill(2);
  PushButtonCancel->hide();
  myCanceled=true;
  myAutoClose=false;
}
/** Beendet den Prozess hart, bis er tot ist. */
void ProcBaseClass::slotKill(){
  bool test= myProc->isRunning();
  if ( test ) {
    myProc->kill(9);
  }
  slotShowOK(myProc);
}
/** Wertet die Ausgabe der Prozesse aus. */
void ProcBaseClass::slotReadFromStdout(KProcess * proc, char * buffer, int buflen){
  QString sAusgabe=QString(buffer);
  TextEditOutput->append( sAusgabe );
  myLog->slotLog( sAusgabe );
  //
  // Fehlerbehandlung
  //
  if  (sAusgabe.find("**ERROR") >= 0)  {
    QString addStr;
    // spezieller Fehler von Mplex
    if (sAusgabe.find("many frame drops") >=0) {
      addStr=i18n("Maybe you should try codec 4 instead of codec 5\n or codec 1 instead of 2");
    }
    QMessageBox* error=new QMessageBox();
    QString errorStr=i18n("\nError in Process. Please see the output for details.\n");
    error->setText(errorStr+addStr);
    error->exec();
    delete error;
    pixmapLabelOK->hide();
    pixmapLabelError->show();
  }
}
/** Wertet die Ausgabe der Prozesse aus. */
void ProcBaseClass::slotReadFromStderr(KProcess * proc, char * buffer, int buflen){
  QString sAusgabe=QString(buffer);
  TextEditOutput->append( sAusgabe );
  myLog->slotLog( sAusgabe );
  //
  // Fehlerbehandlung
  //
  if  (sAusgabe.find("**ERROR") >= 0)  {
    QString addStr;
    // spezieller Fehler von Mplex
    if (sAusgabe.find("many frame drops") >=0) {
      addStr=i18n("Maybe you should try codec 4 instead of codec 5\n or codec 1 instead of 2");
    }
    QMessageBox* error=new QMessageBox();
    QString errorStr=i18n("\nError in Process. Please see the output for details.\n");
    error->setText(errorStr+addStr);
    error->exec();
    delete error;
    pixmapLabelOK->hide();
    pixmapLabelError->show();
  }
}
/** setzt die Kommandozeile und die Argumente von QProcess */
bool ProcBaseClass::setKommando(QString mk){
  myKommando=mk;
  myLog->slotLog("\n=====================\n"+mk+"\n==================\n\n");
  TextEditOutput->setText(myKommando);
  myProc->clearArguments();
  QString hKommando = myKommando;
  QString hArgument;

  while (hKommando.length()>0) {
    int ipos;
    // Leerzeichen am Anfang und Ende entfernen.
    hKommando=hKommando.stripWhiteSpace();
    // NEU: Wenn ein " vor einem space vorkommen.
    // Dann muss bis zum n�hsten " gesucht werden.
    int posNextSpace=hKommando.find(" ");
    int posNextApostroph=hKommando.find('"');
    if ( ((posNextApostroph!=-1) && ( posNextSpace==-1 )) ||
	 ((posNextApostroph!=-1) && ( posNextSpace>posNextApostroph)) )
      {
	// search the second apostroph
	ipos = hKommando.find('"',posNextApostroph+1);
	// if the second could not be found
	if (ipos == -1 ) {
	  hArgument=hKommando;
	  hKommando="";
	} else {
	  hArgument = hKommando.left(ipos+1);
	  // replace the "
	  hArgument=hArgument.replace('"',"");
	  //hArgument = hArgument.mid(1,hArgument.length()-2);
	  hKommando = hKommando.right(hKommando.length()-ipos-1);
     	}
      } else {
	ipos = hKommando.find(" ");
	// Wenn das Zeichen gar nicht mehr gefunden wurde.
	if (ipos == -1 ) {
	  hArgument=hKommando;
	  hKommando="";
	} else {
	  hArgument = hKommando.left(ipos);
	  hKommando = hKommando.right(hKommando.length()-ipos-1);
	}
      } // Ende von else if
	
    hArgument=hArgument.stripWhiteSpace();
    //myProc->addArgument( hArgument );
    *myProc << hArgument;
  } // ende von while
  return true;
}


/** Sets the log File. An empty logfile means no logging */
void ProcBaseClass::setLogFile(QString lf){
  myLogFile=lf;
  myLog->setFileName(myLogFile);
  QDateTime dZeit=QDateTime::currentDateTime();
  QString sZeit=dZeit.toString("dd.MM.yyyy hh:mm:ss");
  myLog->slotLog("Starting Logging on "+sZeit);
}
/** Returns true, if the process ended with the cancel-button */
bool ProcBaseClass::getCanceled(){
  return myCanceled;
}
/** If the right mouse button was pressed... */
void ProcBaseClass::mousePressEvent( QMouseEvent *e){
  if ( e->button() == Qt::RightButton ) {
    // Show context menu
    QPopupMenu* kmenu= new QPopupMenu(this);
    kmenu->insertItem( "S&end <CR> to process",  this, SLOT(sendCR()), CTRL+Key_E, 0 );
    // SIGINT, SIGKILL, SIGTERM, SIGCONT
    kmenu->insertItem( "Send SIGINT to process",  this, SLOT(sendSignal(int)),0,SIGINT);
    kmenu->insertItem( "Send SIGKILL to process",  this, SLOT(sendSignal(int)),0,SIGKILL);
    kmenu->insertItem( "Send SIGTERM to process",  this, SLOT(sendSignal(int)),0,SIGTERM);
    kmenu->insertItem( "Send SIGSTOP to process",  this, SLOT(sendSignal(int)),0,SIGSTOP);
    kmenu->insertItem( "Send SIGCONT to process",  this, SLOT(sendSignal(int)),0,SIGCONT);
    kmenu->exec(e->globalPos());
    delete kmenu;
  }
}
/** Sends an <CR> to the process.. */
void ProcBaseClass::sendCR(){
  // Sends a carriage return
  myProc->writeStdin("\n",1);
}
/** sends the signal to STDIN */
void ProcBaseClass::sendSignal(int i){
  //--kill(myProc->processIdentifier(), i);
  myProc->kill(i);
}
/** shows or hides the output window */
void ProcBaseClass::slotShowOutput(){
  if (TextEditOutput->isVisible()) {
    TextEditOutput->hide();
  } else {
    TextEditOutput->show();
  }
  this->resize(this->minimumSizeHint());
}

