/***************************************************************************
                          procbaseclass.h  -  description
                             -------------------
    begin                : Fre Jul 26 2002
    copyright            : (C) 2002 by Cornelius K�bel
    email                : cornelius.koelbel@gmx.de
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PROCBASECLASS_H
#define PROCBASECLASS_H

#include <klocale.h>
#include <qwidget.h>
#include "TranscodeDlg.h"
#include <kprocess.h>
#include <qpushbutton.h>
#include <qdatetime.h>
#include "logclass.h"


/**Basisklasse fr Transcode und Mplex und alle anderen Prozesse.
 *@author Cornelius K�bel
 */

class ProcBaseClass : public TranscodeDlg  {
  Q_OBJECT
    public: 
  ProcBaseClass(QWidget *parent=0, const char *name=0);
  ~ProcBaseClass();
  /** No descriptions */
  bool setKommando(QString mk);
  /** Sets the log File. An empty logfile means no logging */
  void setLogFile(QString lf);
  /** Returns true, if the process ended with the cancel-button */
  bool getCanceled();
 protected: // Protected attributes
  /** Der Pointer, der den jeweiligen Prozess aufnimmt, mplex, transcode... */
  KProcess* myProc;
  /** Kommandozeile */
  QString myKommando;
  /**  */
  QString myErrorString;
  /** The LogClass */
  LogClass* myLog;
  /** If the right mouse button was pressed... */
  void mousePressEvent( QMouseEvent *e);
  public slots: // Public slots
    /** Zeigt den OK-Button */
    void slotShowOK(KProcess* proc);
  /** Reagiert auf den Startbutton */
  void slotStart(bool autoClose=0);
  protected slots: // Protected slots
    /** Beendet den Prozess und macht den OK-Button wieder sichtbar. */
    virtual void slotCancel();
  /** Beendet den Prozess hart, bis er tot ist. */
  void slotKill();
  /** Wertet die Ausgabe der Prozesse aus. */
  virtual void slotReadFromStdout(KProcess *proc, char *buffer, int buflen);
  virtual void slotReadFromStderr(KProcess *proc, char *buffer, int buflen);
  /** Sends an <CR> to the process.. */
  void sendCR();
  /** sends the signal to STDIN */
  void sendSignal(int i);
  /** shows or hides the output window */
  void slotShowOutput();
 protected:
  /** true if the dialog was ended with the cancel button */
  bool myCanceled;
  /** If true, at the end of the process the close-button will be shown.
      If false, the dialog will close emediatly. */
  bool myAutoClose;
 private: // Private attributes
  /** Zeitpunkt des Starts des Prozesses */
  QDateTime myStartTime;
  /** logfile */
  QString myLogFile;
  /** Zeitpunkt des Endes des Prozesses */
  QDateTime myEndTime;
  /** Preview process */
  /* QProcess* myPreviewProc; */
};

#endif
