/***************************************************************************
                          profile.cpp  -  description
                             -------------------
    begin                : Die Apr 8 2003
    copyright            : (C) 2003 by Cornelius Klbel
    email                : cornelius.koelbel@gmx.de
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "profile.h"
#include <qlistview.h>
#include <qcombobox.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qmessagebox.h>
#include <qtextstream.h>

Profile::Profile(Kavi2svcd *parent, const char *name ) : profileDlg(parent,name) {
  myParent = parent;
  QString kdeHome=QString(getenv( "KDEHOME" ));
  if (kdeHome.isEmpty()) {
    myDatei=QDir::homeDirPath();
    myDatei.append("/.kde");
  } else {
    myDatei=kdeHome;
  }
  myDatei.append("/share/apps/kavi2svcd/xsvcd.prof");
  load();
}
Profile::~Profile(){
}
/** Adds a key to the list */
void Profile::slotAdd(){
  if ( getValue( ComboBoxKey->currentText() ) == "not_in_list" ) {
    ListViewProfile->insertItem( new QListViewItem( ListViewProfile, ComboBoxKey->currentText(), LineEditValue->text() ));
  } else {
    ListViewProfile->currentItem()->setText( 1, LineEditValue->text());
    // fix, since takeitem crashes!!!
    ListViewProfile->currentItem()->setVisible(true);
  }
}
/** deletes an entry from the profile list */
void Profile::slotDel(){
  ListViewProfile->selectedItem()->setVisible(false);
  // takeitem crashes!!!
  // ListViewProfile->takeItem( ListViewProfile->selectedItem() );
}
/** shows the value of the selected item */
void Profile::slotShowValue(){
  LineEditValue->setText(	ListViewProfile->selectedItem()->text(1) );
  ComboBoxKey->setCurrentText(ListViewProfile->selectedItem()->text(0) );
}
/** change the value, when the key in the comboBox was changed. */
void Profile::slotChangeValue(){
  QString text = getValue( ComboBoxKey->currentText() );
  if ( text != "not_in_list" ) {
    LineEditValue->setText( text );
  } else {
    LineEditValue->setText( "" );
  }
}

/** get the value for a key. If not in list, returns "not_in_list" */
QString Profile::getValue(QString key){
  QListViewItemIterator it( ListViewProfile );
  // iterates across ListViewProfile - Items
  while ( it.current() != 0 ) {
    if (it.current()->text(0)==key) {
      it.current()->setSelected( true );
      return it.current()->text(1);
    }
    ++it;
  }
  return "not_in_list";
}
/** The Button OK is pressed. */
void Profile::slotOK(){
  // communicate to kavi2svcd class to use the profile
  myParent->setUseProfile( CheckBoxUseProfile->isChecked() );
  // Generate a List of the Key Value pairs
  exportListView();
  // hand the list to kavi2svcd
  myParent->setProfile( myStringList );
  // save the profile
  save();
  // close the dialog
  this->close();		
}
/** exports the Listview to a Stringlist */
void Profile::exportListView(){
  myStringList.clear();
  QListViewItemIterator it( ListViewProfile );
  // iterates across ListViewProfile - Items
  while ( it.current() != 0 ) {
    QString zeile;
    zeile = it.current()->text(0)+" = "+it.current()->text(1);
    myStringList.append( zeile );
    ++it;
  }
}
/** imports the Stringlist to the Listview */
void Profile::importListView(){
  ListViewProfile->clear();
  for ( QStringList::Iterator it = myStringList.begin(); it != myStringList.end(); ++it ) {
    QString text = (*it).latin1();
    int pos = text.find("=");
    QString key   = text.left(pos -1 );
    QString value = text.right( text.length()-pos-2 );
    ListViewProfile->insertItem( new QListViewItem( ListViewProfile, key, value));
  }
}
/** sets the list for profile */
void Profile::setList( QStringList list){
  myStringList = list;
  importListView();
}
/** Sets the Checkbox */
void Profile::setUseProfile(bool bUse){
  CheckBoxUseProfile->setChecked( bUse );
}
/** saves the profile */
void Profile::save(){
  // hier wird in das Verzeichnis die Datei geschrieben.
  QFile f(myDatei);
  if (	f.open(IO_WriteOnly) ) {
    QTextStream ts( &f );
    for ( QStringList::Iterator it = myStringList.begin(); it != myStringList.end(); ++it ) {
      ts << (*it).latin1();
      ts << "\n";
    }
    f.close();
  } else {
    QMessageBox* error = new QMessageBox();
    QString errorStr=i18n("\nError writing file\n");
    errorStr.append( myDatei );
    error->setText( errorStr );
    error->exec();
    delete error;
  }
}
/** loads the xsvcd.prof */
void Profile::load(){
  myStringList.clear();
  QFile f(myDatei);
  if (	f.open(IO_ReadOnly) ) {
    QTextStream ts( &f );
    QString test = ts.readLine();
    while ( test != NULL ) {
      myStringList.append( test );
      test = ts.readLine();
    }
    f.close();
    importListView();
  } else {
    QMessageBox* error = new QMessageBox();
    QString errorStr=i18n("\nError reading file\n");
    errorStr.append( myDatei );
    error->setText( errorStr );
    error->exec();
    delete error;
  }
}
/** Resets the values to reasonable values */
void Profile::slotReset(){
  while ( !myStringList.empty() ) {
    myStringList.pop_front();
  }
  myStringList.append("fixed_vbv_delay = 1");
  myStringList.append("vbv_buffer_size = 230");
  myStringList.append("cbr = 1");
  myStringList.append("cbr_bitrate = 4000000.0");
  myStringList.append("quant_value = 2");
  myStringList.append("qscale_type = 0");
  myStringList.append("vbr_mux = 0");
  myStringList.append("video_buf_size = 230");
  myStringList.append("use_comp_bitrate = 1");
  myStringList.append("max_file_size = 700");
  importListView();
}
