/***************************************************************************
                          subtitlerclass.cpp  -  description
                             -------------------
    begin                : Mon Sep 22 2003
    copyright            : (C) 2003 by Cornelius Klbel
    email                : cornelius.koelbel@gmx.de
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "subtitlerclass.h"
#include <qlineedit.h>
#include <qslider.h>
#include <qfiledialog.h>
#include <klocale.h>
#include <stdlib.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qmessagebox.h>
#include <qradiobutton.h>

SubtitlerClass::SubtitlerClass(QWidget *parent, const char *name ) : subtitleDlg(parent,name) {
  myData.path="/";
  myData.fps=0;
  myPreviewProc = new QProcess();
  myEditProc = new QProcess();
}
SubtitlerClass::~SubtitlerClass(){
}
/** Is called, when the slider is changed value = 10..10000 (.1 .. 100) */
void SubtitlerClass::slotFontChanged(int i){
  QString value;
  float f = (float)i/100;
  value=value.setNum( f, 'f', 2 );
  lineEditFont->setText(value);
}
/** is called when the textEdit is chaned. THe slider has to be set. */
void SubtitlerClass::slotFontChanged(const QString& s){
  QString value = s;
  float f;
  bool b;
  f=value.toFloat(&b);
  if (b) {
    int i=(int)(f*100);
    sliderFont->setValue(i);
  }
}
/** If the Button "check file" is pressed, this slot is called.
    Then mplayer is started with the subtitle. */
void SubtitlerClass::slotCheckFile(){
  QString command = myData.subView;
  myPreviewProc->clearArguments();
  while (command.length()>0) {
    int pos=command.find(" ");
    QString param=command.left(pos);
    command=command.right(command.length()-pos-1);
    if (pos==-1) command="";
    // %s = subtitle file
    param.replace("%s",lineEditFile->text());
    // %m = movie file
    param.replace("%m",myData.movieFile);
    // %f = frames delay
    //		param.replace("%f",spinBoxOffset->text());
    // %d = seconds delay
    // We need the fps of the movie and then we can recalculate the seconds
    //		int framesOffset=spinBoxOffset->value();
    //		float secondsOffset=(float)framesOffset/myData.fps;
    //		QString sOffset; sOffset=sOffset.setNum(secondsOffset,'f',3);
    //		param.replace("%d",sOffset);		
    myPreviewProc->addArgument( param );
  }
  // Now issue the command
  if (! myPreviewProc->start() ) {
    QMessageBox* error = new QMessageBox();
    error->setText(i18n("\nError starting Preview.\nPlease set the Preview program for subtitle files in the preferences!\n"));
    error->exec();
    delete error;
  }
}
/** The subtitle file can be edited with ksubeditor */
void SubtitlerClass::slotEditFile(){
  QString command=myData.subEdit;
  myEditProc->clearArguments();
  while (command.length()>0) {
    int pos=command.find(" ");
    QString param=command.left(pos);
    command=command.right(command.length()-pos-1);
    if (pos==-1) command="";
    // %s = subtitle file
    param.replace("%s",lineEditFile->text());
    myEditProc->addArgument( param );
  }
  // Now issue the command
  if (! myEditProc->start() ) {
    QMessageBox* error = new QMessageBox();
    error->setText(i18n("\nError starting Subtitle Edit Program.\nPlease set the Edit program in the preferences!\n"));
    error->exec();
    delete error;
  }
}
/** A fileDialog is opened and the subtitle can be chosen. */
void SubtitlerClass::slotOpenFile(){
  QFileDialog *myFile = new QFileDialog(myData.path);
  myFile->setFilters(QString(i18n("subtitles (*.ppml *.sub *.srt);;All (*)")));
  if ( myFile->exec() == QDialog::Accepted )
    {
      lineEditFile->setText(myFile->selectedFile());
    } // Ende vom myFile
  delete myFile;
}

//
// Public Methods for passing the values
//
/** returns the part of the transcode command, that is responsible for subtitles */
QString SubtitlerClass::getCommand(){
  QString command;
  if ( checkBoxSubtitle->isChecked() ) {
    command="-J subtitler=\"";
    // Options
    //if ( spinBoxOffset->value() !=0 ){
    //	command.append(QString("frameOffset=%1 ").arg(spinBoxOffset->value()));
    //}
    if ( sliderFont->value() !=1075) {
      command.append(QString("font_factor=%1 ").arg(sliderFont->value()));
    }
    // Subtitle File
    command.append("subtitle_file='"+lineEditFile->text()+"'\"");
  } else { // checkbox not checked
    command="";
  }
  if ( radioButtonTranscode->isChecked() ){
    return command; } else {
      return "";
    }
}
/** Sets all the private data of the class */
void SubtitlerClass::setData(SubtitleData d){
  myData=d;
  checkBoxSubtitle->setChecked(myData.bUseSub);
  sliderFont->setValue(myData.iFontFactor);
  //	spinBoxOffset->setValue(myData.iOffset);
  lineEditFile->setText(myData.subFile);
  radioButtonTranscode->setChecked(myData.bTranscodeMode);
  radioButtonMplayer->setChecked(!myData.bTranscodeMode);
}
/** Returns the data of the subtitler class */
SubtitlerClass::SubtitleData SubtitlerClass::getData(){
  myData.bUseSub=checkBoxSubtitle->isChecked();
  myData.iFontFactor=sliderFont->value();
  //myData.iOffset=spinBoxOffset->value();
  myData.iOffset=0;
  myData.subFile=lineEditFile->text();
  myData.bTranscodeMode=radioButtonTranscode->isChecked();
  return myData;
}
