/***************************************************************************
                          svcdmodeclass.cpp  -  description
                             -------------------
    begin                : Don Nov 20 2003
    copyright            : (C) 2003 by Cornelius K�bel
    email                : cornelius.koelbel@gmx.de
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "svcdmodeclass.h"
#include <klocale.h>
#include <qlabel.h>
#include <qmessagebox.h>
#include <qtextedit.h>
#include <qfile.h>
#include <qtimer.h>
#include <qcolor.h>
#include <qcursor.h>
#include <stdlib.h>

SvcdModeClass::SvcdModeClass(QWidget *parent, const char *name ) : svcdModeDlg(parent,name) {
  QString logFile=QString("");
  myLog = new LogClass();
  myLog->setFileName(logFile);
  filename="kavi-test";
  filenameRef="kavi-ref";
  myOutputDir=QDir::homeDirPath();
  // Remove the FIFO
  QFile stream(myOutputDir+QString("/stream.yuv"));
  if (stream.exists() ){
  	stream.remove();
  }
  // remove the images
  QFile test(myOutputDir+QString("/kavi-test000000.jpg"));
  QFile ref(myOutputDir+QString("/kavi-ref000000.jpg"));
  if (test.exists() ){
  	test.remove();
  }
  if (ref.exists() ){
  	ref.remove();
  }
  
  myStartFrame=300;
  myFrame=myStartFrame;
  myMaxFrames=10000;
  myRange=QString("%1-%2").arg(myFrame).arg(myFrame+1);
  myMplayerRef=false;
  // Generating the Process for Generate Pixmap
  myProcRef = new KProcess(  );
  myProcTest = new KProcess( );
  // when the process exited, the image is written and kavi2svcd can display it.
  
  connect( myProcRef, SIGNAL(processExited(KProcess *)),
  	   this, SLOT( slotExitedRef(KProcess *) ) );
  connect( myProcTest, SIGNAL(processExited(KProcess *)),
	   this, SLOT( slotExitedTest(KProcess *) ) );		
	   
  connect( myProcRef, SIGNAL(receivedStdout (KProcess *, char *, int)),
	   this, SLOT(slotReadFromStdoutRef(KProcess *, char *, int)) );
  //connect( myProcRef, SIGNAL(readyReadStderr()),
    //        this, SLOT(slotReadFromStdoutRef()) );	
	    
  connect( myProcTest, SIGNAL(receivedStdout (KProcess *, char *, int)),
	   this, SLOT(slotReadFromStdoutTest(KProcess *, char *, int)) );
  //connect( myProcTest, SIGNAL(readyReadStderr()),
    //        this, SLOT(slotReadFromStdoutTest()) );
}
SvcdModeClass::~SvcdModeClass(){
  delete myProcRef;
  delete myProcTest;
}
/** Switches to expert mode. For this, we have to call the parents function */
void SvcdModeClass::slotSwitchExpertMode(){
  emit signalExpertMode();
}
/** starts transcode */
void SvcdModeClass::slotStart(){
  emit signalStart();
}
/** Calls the file open dialog */
void SvcdModeClass::slotOpen(){
  emit signalOpen();
}
/** toggles the -k checkbox */
void SvcdModeClass::slotKtoggled(){
  emit signalKtoggled();
}
/*!
    \fn SvcdModeClass::slotzToggled(bool b)
 */
void SvcdModeClass::slotzToggled()
{
  emit signalzToggled();
}
/** No descriptions */
void SvcdModeClass::slotVCDToggled(bool b){
  emit signalVCDToggled(b);
}
/** No descriptions */
void SvcdModeClass::slotSVCDToggled(bool b){
  emit signalSVCDToggled(b);
}
/*!
    \fn SvcdModeClass::slotDVDToggled(bool b)
 */
void SvcdModeClass::slotDVDToggled(bool b)
{
    emit signalDVDToggled(b);
}
/** Clears the pixmap */
void SvcdModeClass::clearPixmap(){
  pixmapLabelTest->setText(i18n("Please wait..."));
}
/** Shows the new pixmap */
void SvcdModeClass::slotShowPixmap(QString pm){
  pixmapLabelTest->setPixmap(QPixmap(pm));
}
/** Clears the pixmap reference*/
void SvcdModeClass::clearPixmapRef(){
  pixmapLabelRef->setText(i18n("Please wait..."));
}
/** Shows the new pixmap reference*/
void SvcdModeClass::slotShowPixmapRef(QString pm){
  pixmapLabelRef->setPixmap(QPixmap(pm));
}

/** Starts the process of generating the test image */
bool SvcdModeClass::startProcess(bool ref){
  this->setCursor(QCursor(Qt::WaitCursor));
  if (ref) {
    pixmapLabelRef->setCursor(QCursor(Qt::WaitCursor));
    TextEditOutputRef->setCursor(QCursor(Qt::WaitCursor));
    // We have to delete an old stream.yuv
    QFile fifoFile(myOutputDir+"/stream.yuv");
    fifoFile.remove();
    TextEditOutputRef->clear();
    TextEditOutputRef->setBackgroundColor(QColor(255,10,10));
    myProcRef->clearArguments();
    *myProcRef << "transcode";
    *myProcRef << "-x";
    *myProcRef << "mplayer,null";
    *myProcRef << "-i";
    *myProcRef <<myInputFile;
    *myProcRef<<"-c";
    *myProcRef<<myRange;
    *myProcRef<<"-Z";
    *myProcRef<<"300x200,fast";
    *myProcRef<<"-y";
    // TODO: Wir könnnten dies konfigurierbar machen...
    //myProcRef->addArgument("im,null");
    *myProcRef<<"jpg,null";
    *myProcRef<<"-o";
    *myProcRef<<myOutputDir+"/"+filenameRef;
    bool ret=myProcRef->start(KProcess::NotifyOnExit, KProcess::Stdout);
    return ret;
  }else{
    pixmapLabelTest->setCursor(QCursor(Qt::WaitCursor));
    TextEditOutputTest->setCursor(QCursor(Qt::WaitCursor));
    TextEditOutputTest->clear();
    TextEditOutputTest->setBackgroundColor(QColor(255,10,10));
    myProcTest->clearArguments();
    *myProcTest<<"transcode";
    *myProcTest<<"-i";
    *myProcTest<<myInputFile;
    *myProcTest<<"-c";
    *myProcTest<<myRange;
    *myProcTest<<"-Z";
    *myProcTest<<"300x200,fast";
    *myProcTest<<"-y";
    *myProcTest<<"jpg,null";
    if ( myK ) {*myProcTest<<"-k";	}
    if ( myV ) {*myProcTest<<"--use_rgb";	}
    if ( myZ ) {*myProcTest<<"-z";	}
    *myProcTest<<"-o";
    *myProcTest<<myOutputDir+"/"+filename;
    bool ret=myProcTest->start(KProcess::NotifyOnExit, KProcess::Stdout);
    return ret;
  }
}

/** sets the input filename */
void SvcdModeClass::setInputFile(QString f){
  myInputFile=f;
  myFrame=myStartFrame;
  myRange=QString("%1-%2").arg(myFrame).arg(myFrame+1);
}
/** Sets an option */
void SvcdModeClass::setOption(QString name, bool value){
  if (name == "k") { myK=value; }
  if (name == "V") { myV=value; }
  if (name == "z") { myZ=value; }
}
/** sets the output directory */
void SvcdModeClass::setOutputDir(QString d){
  myOutputDir=d;
}

/** Wertet die Ausgabe der Prozesse aus. */
void SvcdModeClass::slotReadFromStdoutRef(KProcess *proc, char *buffer, int buflen){
   QString sAusgabe=QString(buffer);
  
    //--QString sout=myProcRef->readStderr();
    //QString serr=myProcRef->readStderr();
    //-- sAusgabe = sout;
    //sAusgabe = sAusgabe.append(serr);
    //if (sAusgabe.find("skipping")==-1) {
    	TextEditOutputRef->append( sAusgabe );
    //}
    // Show the enconded frame
    if (sAusgabe.find("clean up")>=0) {
    	// fixme
	slotExitedRef(myProcRef);
	// quit the process
	//int pid=myProcRef->processIdentifier();
	//myProcRef->tryTerminate ();
	//myProcRef->kill ();
    }
  
  // myLog->slotLog( sAusgabe );
  //
  // Fehlerbehandlung
  //
  if  ((sAusgabe.find("ERROR") >= 0)||(sAusgabe.find("failed: File exists")>=0))  {
    QMessageBox* error=new QMessageBox();
    QString errorStr=i18n("\nError in Process. Please see the output for details.\n");
    error->setText(errorStr);
    error->exec();
    delete error;
  }
}
void SvcdModeClass::slotReadFromStdoutTest(KProcess *proc, char *buffer, int buflen){
   QString sAusgabe=QString(buffer);
  
    //QString sout=myProcTest->readStderr();
    //QString serr=myProcTest->readStderr();
    //sAusgabe = sout;
    //sAusgabe = sAusgabe.append(serr);
    //if (sAusgabe.find("skipping")==-1) {
    	TextEditOutputTest->append( sAusgabe );
    //}
    // show the image
    if (sAusgabe.find("clean up")>=0) {
        // fixme
    	slotExitedTest(myProcTest);
	// quit the process
	//int pid=myProcTest->processIdentifier();
	//myProcTest->tryTerminate ();
	//myProcTest->kill ();
    }
  
  // myLog->slotLog( sAusgabe );
  //
  // Fehlerbehandlung
  //
  if  ((sAusgabe.find("ERROR") >= 0)||(sAusgabe.find("failed: File exists")>=0))  {
    QMessageBox* error=new QMessageBox();
    QString errorStr=i18n("\nError in Process. Please see the output for details.\n");
    error->setText(errorStr);
    error->exec();
    delete error;
  }
}


/** Is called, when the process has exited and the image is written. */
void SvcdModeClass::slotExitedRef(KProcess *procRef){
  TextEditOutputRef->setBackgroundColor(QColor(10,255,10));
  //TextEditOutputRef->append(i18n("Generated the preview image:")+myOutputDir+"/"+filenameRef+"000000.jpg");
  slotShowPixmapRef(myOutputDir+"/"+filenameRef+"000000.jpg");
  pixmapLabelRef->setCursor(QCursor(Qt::ArrowCursor));
  TextEditOutputRef->setCursor(QCursor(Qt::ArrowCursor));

  if (myProcTest->isRunning()==false) {
	this->setCursor(QCursor(Qt::ArrowCursor));
  }
}
/** Is called, when the process has exited and the image is written. */
void SvcdModeClass::slotExitedTest(KProcess *procTest){
  TextEditOutputTest->setBackgroundColor(QColor(10,255,10));
  //TextEditOutputTest->append(i18n("Generated the preview image:")+myOutputDir+"/"+filename+"000000.jpg");
  slotShowPixmap(myOutputDir+"/"+filename+"000000.jpg");
  pixmapLabelTest->setCursor(QCursor(Qt::ArrowCursor));
  TextEditOutputTest->setCursor(QCursor(Qt::ArrowCursor));

  if (myProcRef->isRunning()==false) {
	this->setCursor(QCursor(Qt::ArrowCursor));
  }
}

/** sets the maximum number of the frames */
void SvcdModeClass::setMaxFrames(long f){
  myMaxFrames=f;
}

/** calculates a next image in the video. */
void SvcdModeClass::slotRight(){
  myFrame=myFrame+1000;
  if (myFrame>myMaxFrames) {
    myFrame=myFrame-myMaxFrames;
  }
  myRange=QString("%1-%2").arg(myFrame).arg(myFrame+1);
  startProcess(true);
  startProcess(false);	 
}




