/***************************************************************************
                          svcdmodeclass.h  -  description
                             -------------------
    begin                : Don Nov 20 2003
    copyright            : (C) 2003 by Cornelius K�bel
    email                : cornelius.koelbel@gmx.de
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SVCDMODECLASS_H
#define SVCDMODECLASS_H

#include <qwidget.h>
#include <qprocess.h>
#include "svcdmodedlg.h"
#include "logclass.h"
#include <kprocess.h>

/**Class for simple svcd (none-expert) mode
 *@author Cornelius K�bel
 */

class SvcdModeClass : public svcdModeDlg  {
  Q_OBJECT
    public: 
  SvcdModeClass(QWidget *parent=0, const char *name=0);
  ~SvcdModeClass();
  public slots: // Public slots
    /** Starts the process of generating the image */
    bool startProcess(bool ref=false);
  /** Sets an option */
  void setOption(QString name, bool value);
  /** sets the input filename */
  void setInputFile(QString f);
  /** sets the output directory */
  void setOutputDir(QString d);
  /** sets the maximum number of the frames */
  void setMaxFrames(long f);
  /** calculates a next image in the video. */
  void slotRight();
  /** Shows the new pixmap */
  void slotShowPixmap(QString pm);
  /** Shows the new pixmap refrence*/
  void slotShowPixmapRef(QString pm);
  /** Clears the pixmap */
  void clearPixmap();
  /** Clears the pixmap reference*/
  void clearPixmapRef();
  protected slots: // Protected slots
    /** Switches to expert mode. For this, we have to call the parents function */
    void slotSwitchExpertMode();
  /** starts transcode */
  void slotStart();
  /** Calls the file open dialog */
  void slotOpen();
  /** toggles the -k checkbox */
  void slotKtoggled();
  /** No descriptions */
  void slotSVCDToggled(bool b);
  /** No descriptions */
  void slotVCDToggled(bool b);
  /** Wertet die Ausgabe der Prozesse aus. */
  virtual void slotReadFromStdoutRef(KProcess *proc, char *buffer, int buflen);
  /** Wertet die Ausgabe der Prozesse aus. */
  virtual void slotReadFromStdoutTest(KProcess *proc, char *buffer, int buflen);
  /** Is called, when the process has exited and the image is written. */
  void slotExitedRef(KProcess *procRef);
  /** Is called, when the process has exited and the image is written. */
  void slotExitedTest(KProcess *procTest);
    void slotDVDToggled(bool b);
    void slotzToggled();
 signals: // Signals
  /** switches to expert mode */
  void signalExpertMode();
  /** calls the file open dialog in kavi2svcd */
  void signalOpen();
  /** No descriptions */
  void signalSVCDToggled(bool);
  /** No descriptions */
  void signalVCDToggled(bool);
  void signalDVDToggled(bool);
  /** toggles the -k checkbox */
  void signalKtoggled();
  void signalzToggled();
  /** Starts the transcode Process */
  void signalStart();
  /** is emitted, when the button right is pressed */
  void signalRight();
 private:
  /** Name of the inputfile */
  QString myInputFile;
  /** Range of the frames, that get transcoded */
  QString myRange;
  /** if the -k switch is set */
  bool myK;
  /** if the -z switch is set */
  bool myZ;
  /** is the -V switch is set.  */
  bool myV;
  /** output dir, where the image gets saved */
  QString myOutputDir;
  /** Process, that holds the command */
  // QProcess* myProcRef;
  KProcess* myProcRef;
  /** Process, that holds the command */
  KProcess* myProcTest;
  /** output filename of the image */
  QString filename;
  /** the reference filename */
  QString filenameRef;
  /** The maximum number of frames */
  long myMaxFrames;
  /** If set to true, the image will be transcoded with the mplayer import module and will be taken as reference */
  bool myMplayerRef;
  /** The first frame, that will be made an image of */
  long myStartFrame;
  /** The frame, that will be made an image of */
  long myFrame;
  /** this class is used to log messages */
  LogClass* myLog;
};

#endif
