/***************************************************************************
                          transcode.cpp  -  description
                             -------------------
    begin                : Mit Jul 24 2002
    copyright            : (C) 2002 by Cornelius K�bel
    email                : cornelius.koelbel@gmx.de
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "transcode.h"
#include <stdlib.h>
#include <qtextedit.h>
#include <qlabel.h>
#include <qstring.h>
#include <qprogressbar.h>
#include <qpushbutton.h>
#include <qmessagebox.h>
#include <qregexp.h>
#include <qpopupmenu.h>
#include <stdio.h>
#include <qtimer.h>

Transcode::Transcode(QWidget *parent, const char *name) : ProcBaseClass(parent,name) {
  // Die Initilisierungen und connect geschehen schon in der Basisklasse
  myErrorString=i18n("Error starting transcode process.");
  // This is for the automate mode
  myGetInfo=new GetInfoClass();
  myPreviewProc=new QProcess();
  connect( myGetInfo, SIGNAL(sendFrames( QString )), this, SLOT( slotFrames( QString )) );
}
Transcode::~Transcode(){
  //TODO program crashes delete myGetInfo;
  delete myGetInfo;
  delete myPreviewProc;
}

/** Liest die Ausgabe des Transcode-Prozesses aus. */
void Transcode::slotReadFromStdout(KProcess * proc, char * buffer, int buflen){
  QString sTranscodeAusgabe;
  QString searchtextFrames="encoding frames [000000-";
  QString searchtextFPS="fps,";
  QString sTimeRemain, sFrames, sFPS;
  int ipos, hRemain, mRemain;
		
  long frames;
  double timeRemain;
  float fps;

  sTranscodeAusgabe = QString(buffer);

  // Check for DirMode
  if ( myDirMode ){
    // get the total number of all processed files in dirMode
    // "(transcode.c) processing 157 file(s)"
    ipos=sTranscodeAusgabe.find("(transcode.c) processing");
    if ( ipos > 0 ) {
      // length of "(transcode.c) processing"
      ipos=ipos+25;
      int ipos2 = sTranscodeAusgabe.find(" file(s)",ipos);
      QString sTotalFiles=sTranscodeAusgabe.mid(ipos, ipos2-ipos);
      myTotalFiles=sTotalFiles.toInt();
    }
    /* In Case of Directory mode
       we have to search for "myDirModeFile" in the output text.
       If it is found we have to get the number of frames from the
       current file AND we have to increase the number of the files.	
    */
    ipos=sTranscodeAusgabe.find(myDirModeFile);
    if ( ipos > 0 ) {
      QString out=QString(i18n("DirMode File: ")+"%1/%2").arg(myNumFile).arg(myTotalFiles);
      myNumFile++;
      TextLabelFiles->setText( out );

      ipos=sTranscodeAusgabe.find("-i \""+myDirModeFile);
      if ( ipos > 0 ) {
	int ipos2=sTranscodeAusgabe.find('"', ipos+4);
	QString nextFile=sTranscodeAusgabe.mid(ipos+4, ipos2-(ipos+4));
	// Get the number of frames of a new file.
	// The transcode-tab has only the frames onf the first (chosen) file
	// Set myFrames
	myGetInfo->slotGetInfo( nextFile );
      }	
    }
  }
  // Read Frames
  ipos=sTranscodeAusgabe.find(searchtextFrames);
  if (ipos >= 0 ) {  // fix sourceforge 890821
    TextEditOutput->append( sTranscodeAusgabe.left(ipos) );
    myLog->slotLog(sTranscodeAusgabe.left(ipos) );
    //** Wenn keine Frames ausgegeben werden, soll die Ausgabe dem USer gezeugt werden.
    //** TextEditOutput->append( sTranscodeAusgabe );
    sFrames=sTranscodeAusgabe.mid(ipos+searchtextFrames.length(),6);
    while ( sFrames.at(0)=='0' ) sFrames=sFrames.right(sFrames.length()-1);
    frames=sFrames.toLong();
    if (frames>0) {
      TextLabelFrames->setText(sFrames);
    }
    // TotalSteps sind (myFrames/10)
    ProgressBarTranscode->setProgress((int)(frames/10));
    this->setCaption( QString("%1% Transcoding...").arg((100*frames)/myFrames) );
    // Hier werden die frames per second ausgewertet und die
    // verbleibende Zeit berechnet.
    ipos=sTranscodeAusgabe.find(searchtextFPS);
    sFPS=sTranscodeAusgabe.mid(ipos-7,6);
    fps=sFPS.toFloat();
    timeRemain=(double)(myFrames-frames)/(60*fps); // Minuten gesamt
    hRemain = (int)(timeRemain/60); // Sunden
    mRemain = (int)(timeRemain-(hRemain*60)); // restminuten
    sTimeRemain.sprintf("%2i:%02i hh:mm",hRemain, mRemain);
    TextLabelTime->setText(sTimeRemain);
  }
}
/** Liest die Ausgabe des Transcode-Prozesses aus. */
void Transcode::slotReadFromStderr(KProcess * proc, char * buffer, int buflen){
  QString sTranscodeError;
  QString searchtextFrames="encoding frames [000000-";
  QString searchtextFPS="fps,";
  QString sTimeRemain, sFrames, sFPS;

  sTranscodeError =QString(buffer);

  if ( !sTranscodeError.isEmpty() ) {
    TextEditOutput->append("|E| "+sTranscodeError);
    myLog->slotLog("|E| "+sTranscodeError);
  }
  // Fehlermeldung berprfen
  if ( (sTranscodeError.find("**ERROR") >= 0) ||
       (sTranscodeError.find("++ERROR") >= 0) ||
       (sTranscodeError.find("critical error:") >= 0 ) ) {
    QMessageBox* error=new QMessageBox();
    QString errorStr=i18n("\nError in Process. Please see the output for details.\n");
    error->setText(errorStr);
    error->exec();
    delete error;
  }
}


/** Setzt die Anzahl der Frames, aus denen dann der Prozentsatz
    des transcodierten Filems berechnet wird. */
void Transcode::setFrames(long lframes){
  myFrames=lframes;
  ProgressBarTranscode->setTotalSteps((int)(myFrames/10));
}
/** Zeigt den bisher transkodierten Film */
void Transcode::slotPreview(){
  QString command;
  command=myPlayer;
  myPreviewProc->clearArguments();
  while (command.length()>0) {
    int pos=command.find(" ");
    QString param=command.left(pos);
    command=command.right(command.length()-pos-1);
    if (pos==-1) command="";
    param.replace("%f",myVideoOut);
    myPreviewProc->addArgument( param );	
  }
  if (! myPreviewProc->start() ) { 
    QMessageBox* error = new QMessageBox();
    error->setText(i18n("\nError starting Preview.\nPlease set the Preview program in the preferences!\n"));
    error->exec();
    delete error;
  }
}
/** Setzt den MPlayer */
void Transcode::setPlayer(QString player){
  myPlayer=player;
}
/** Setzt die Videodadtei */
void Transcode::setVideoOut(QString video){
  myVideoOut=video;
}
/** �erladen der Funktion 
    ProcBaseClass::slotCancel() */
void Transcode::slotCancel(){
  //QString sPID; sPID.setNum( myProc->processIdentifier() );
  // Get the list of processes:
  // system("restlosKillTranscode "+sPID);
  // system("kill -1 "+sPID);
  // There will always remain the mpeg2enc process!
  //myProc->tryTerminate();
  myProc->kill(2);
  //QTimer::singleShot( 5000, myProc, SLOT( kill() ) );
  PushButtonCancel->hide();
  myCanceled=true;
  myAutoClose=false;
}

/** Startet den Transcode-Prozess. */
void Transcode::slotStart(bool autoClose){
  PushButtonPreview->show();
  ProcBaseClass::slotStart(autoClose);
}
/** Sets the DirMode */
void Transcode::setDirMode(bool b){
  myDirMode=b;
  myNumFile=0;
  myTotalFiles=0;
}

/** Sets the basename of the files */
void Transcode::setDirModeFile(QString s){
  myDirModeFile=s;
}
/** Gets the number of frames. */
void Transcode::slotFrames(QString f){
  myFrames=f.toLong();
  // I have to set the maximum step of the progressbar
  setFrames(myFrames);
}
