/***************************************************************************
                          vcdclass.cpp  -  description
                             -------------------
    begin                : Fre Jul 26 2002
    copyright            : (C) 2002 by Cornelius Klbel
    email                : cornelius.koelbel@gmx.de
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "vcdclass.h"

#include <qpushbutton.h>
#include <qtextedit.h>
#include <qprogressbar.h>
#include <qlabel.h>

VCDclass::VCDclass(QWidget *parent, const char *name ) : ProcBaseClass(parent,name) {
  setCaption(i18n("vcdimager..."));
  myErrorString=i18n("Error starting vcdimager process");
  myTimer = new QTimer( this );
  connect( myTimer, SIGNAL(timeout()), this, SLOT(slotTimer()) );
  myTimer->start( 5000, false ); // 5 seconds timer

  connect( myProc, SIGNAL(processExited()), this, SLOT(slotTimerStop()) );
  ProgressBarTranscode->setTotalSteps(1000);

  TextLabelProgress->setText(i18n("Progress:"));
  TextLabelTime->setText(i18n("Running..."));
  TextLabelFrames->hide();
  TextLabelFrames2->hide();
}
VCDclass::~VCDclass(){
  delete myTimer;
}


/** berechnet den Progress vom VCD-Prozess */
int VCDclass::progress(QStringList sListe, QString myFilter, QString myPrefix, QString sBin, QString myPath){
  // INPUT: QStringList mpegListe,  manuell
  //				QString  Filter,				automatisch
  //	  		QString myPrefix,				automatisch
  // 				QString bin-file,				manuell
  //				QString myPath.			
  // OUTPUT: progress / 1000
  // Faktor 1.02 KAVI-mplex-output-?.mpeg ist immer fest
  // Prefix-1.bin
  // Summe berechnen der mpeg - dateien.
  // QDir erzeugen mit Pfad und Filter.
  long totalSize=0;
  if ( !myFilter.isEmpty() ) { // automatisch
    QDir d = QDir(myPath, myFilter);
    const QFileInfoList *list = d.entryInfoList();
    QFileInfoListIterator it( *list );
    QFileInfo *fi = new QFileInfo();
    while ( (fi = it.current()) != 0 ) {
      totalSize= totalSize+fi->size();
      ++it;
    }
    delete fi;
  } else { // manuell, die Liste abarbeiten.
    for ( QStringList::Iterator it = sListe.begin(); it != sListe.end(); ++it ) {
      QFileInfo* fi = new QFileInfo( *it );
      totalSize = totalSize + fi->size();
      delete fi;
    }
  }

  totalSize=(long)(totalSize*1.02);

  long summe=0;	
  if ( !myPrefix.isEmpty() ) { // automatisch, summe der bin-dateien.
    // Summe der .bin Dateien.
    myFilter = myPrefix + "-*.bin";
    QDir d = QDir(myPath, myFilter);
    const QFileInfoList *list_bin = d.entryInfoList();
    QFileInfoListIterator it_bin( *list_bin );
    QFileInfo *fi_bin = new QFileInfo();
    while ( (fi_bin = it_bin.current()) != 0 ) {
      summe = summe+fi_bin->size();
      ++it_bin;
    }
    delete fi_bin;
  } else { // manuell, eine bin-datei
    QFileInfo* fi = new QFileInfo( sBin );
    summe = fi->size();
    delete fi;
  }
  // summe durch totalsize teilen und in der Progressbar darstellen
  int progress = (int)(((double)summe/(double)totalSize)*1000);
  return progress;
}

/** Wird nach einem bestimmten Zeitintervall aufgerufen,
    damit der Fortschritt geprft werden kann */
void VCDclass::slotTimer(){
  // Es werden bentigt:
  // Die Liste mit den mpegfiles
  // Das bin -file. Daraus wird der myPath gebaut.
  int iProgress = progress(myMpegList, "", "", myBin, "");
  ProgressBarTranscode->setProgress(iProgress);
}

/** Wenn der Prozess zu Ende ist, wird der Timer angehalten. */
void VCDclass::slotTimerStop(){
  myTimer->stop();
  ProgressBarTranscode->setProgress(1000);

}/** Setzt die Daten, zur Berechnung des Fortschritts. */
void VCDclass::setData(QString bin, QStringList liste){
  myBin = bin;
  myMpegList = liste;
}
/** sets all the data from the GUI. This can be used to construct the command line */
void VCDclass::setData(vcdData d){
  myData=d;
  myBin=myData.binFile;
  myMpegList=myData.fileList;
}
/** Generates the command from the internal data structure */
bool VCDclass::start(){
  bool value;
  if (myData.command.isEmpty()) {
    // Now we are in automate mode and we will have to generate a new command.
    // This is the same as in kavi2svcd::slotGenerateTranscode.
    value=setKommando(generateKommando());
  } else {
    // We are NOT in automate. Just take the normal Command.
    value=setKommando(myData.command);
  }
  return value;
}
/** generates the Kommando for VCD from the vcdData. */
QString VCDclass::generateKommando(){
  QString sKommando;
  if ( myData.nice != 0 ) {
    sKommando.append(QString("nice -n %1 ").arg(myData.nice));
  }
  sKommando.append("vcdimager ");
  sKommando.append(myData.additionalOption);
  sKommando.append(" -t ");
  sKommando.append( myData.type );
  if ( !myData.cueFile.isEmpty() ) {
    sKommando.append(" -c ");
    sKommando.append('"');
    sKommando.append(myData.cueFile);
    sKommando.append('"');
  }
  if ( !myData.binFile.isEmpty() ) {
    sKommando.append(" -b ");
    sKommando.append('"');
    sKommando.append(myData.binFile);
    sKommando.append('"');
  }
  if ( myData.offset ) sKommando.append(" --update-scan-offsets"); 
  if ( myData.broken ) sKommando.append(" --broken-svcd-mode");
  if ( myData.use2336 )	sKommando.append(" --sector-2336");
  sKommando.append(" --volume-count=");
  sKommando.append(myData.count);
  sKommando.append(" --volume-number=");
  sKommando.append(myData.number);
  QStringList::iterator it;
  for ( it = myData.fileList.begin(); it != myData.fileList.end(); ++it ){
    sKommando.append(" ");
    sKommando.append('"');
    sKommando.append(*it);
    sKommando.append('"');
  }
  return sKommando;
}
/** Sets the current number and the total number */
void VCDclass::setNum(int i, int t){
  TextLabelFrames->setText( i18n("current file %1 of %2").arg(i).arg(t) );
}
